@@ -22,7 +22,7 @@
  * @todo Make this a core plugin!
  */
 #include "internal.h"
-#include "gtkinternal.h"
+#include "gtkgaim.h"
 
 #ifndef _WIN32
 # include <sys/un.h>
@@ -36,12 +36,14 @@
 #include "debug.h"
 #include "prpl.h"
 #include "notify.h"
+#include "version.h"
 
 /* XXX */
+#include "away.h"
 #include "gtkconv.h"
 #include "gtkplugin.h"
 #include "gaim.h"
-#include "ui.h"
+#include "prefs.h"
 
 #include <gaim-remote/remote.h>
 
@@ -78,7 +80,7 @@ gaim_remote_handle_uri(const char *uri)
 		account = gaim_connection_get_account(gc);
 		username = gaim_account_get_username(account);
 
-		if (gaim_account_get_protocol(account) == GAIM_PROTO_OSCAR &&
+		if (strcmp(gaim_account_get_protocol_id(account), "prpl-oscar") == 0 &&
 			username != NULL && isalpha(*username)) {
 
 			break;
@@ -394,6 +396,22 @@ user_handler(struct UI *ui, guchar subtype, gchar *data)
 			gaim_account_connect(account);
 		/* don't need to do anything here because the UI will get updates from other handlers */
 		break;
+       case CUI_USER_AWAY:
+                {
+                    GSList* l;
+                    const char* default_away_name = gaim_prefs_get_string("/core/away/default_message");
+
+                    for(l = away_messages; l; l = l->next) {
+                        if(!strcmp(default_away_name, ((struct away_message *)l->data)->name)) {
+                            do_away_message(NULL, l->data);
+                            break;
+                        }
+                    }
+                }
+                break;
+       case CUI_USER_BACK:
+                do_im_back(NULL, NULL);
+                break;
 	default:
 		gaim_debug(GAIM_DEBUG_WARNING, "cui",
 				   "Unhandled user subtype %d\n", subtype);
@@ -633,12 +651,12 @@ open_socket(char **error)
 {
 	struct sockaddr_un saddr;
 	gint fd;
-	
+
 	while (gaim_remote_session_exists(gaim_session))
 		gaim_session++;
-	
+
 	gaim_debug(GAIM_DEBUG_MISC, "cui", "Session: %d\n", gaim_session);
-	
+
 	if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) != -1) {
 		mode_t m = umask(0177);
 		saddr.sun_family = AF_UNIX;
@@ -648,11 +666,13 @@ open_socket(char **error)
 		if (bind(fd, (struct sockaddr *)&saddr, sizeof(saddr)) != -1)
 			listen(fd, 100);
 		else {
+			char *tmp = g_locale_to_utf8(strerror(errno), -1, NULL, NULL, NULL);
 			*error = g_strdup_printf(_("Failed to assign %s to a socket:\n%s"),
-					   saddr.sun_path, strerror(errno));
+					   saddr.sun_path, tmp);
 			g_log(NULL, G_LOG_LEVEL_CRITICAL,
-			      "Failed to assign %s to a socket (Error: %s)",	
-			      saddr.sun_path, strerror(errno));
+			      "Failed to assign %s to a socket (Error: %s)",
+			      saddr.sun_path, tmp);
+			g_free(tmp);
 			umask(m);
 			return -1;
 		}
@@ -711,7 +731,9 @@ plugin_unload(GaimPlugin *plugin)
 
 static GaimPluginInfo info =
 {
-	GAIM_PLUGIN_API_VERSION,                          /**< api_version    */
+	GAIM_PLUGIN_MAGIC,
+	GAIM_MAJOR_VERSION,
+	GAIM_MINOR_VERSION,
 	GAIM_PLUGIN_STANDARD,                             /**< type           */
 	GAIM_GTK_PLUGIN_TYPE,                             /**< ui_requirement */
 	0,                                                /**< flags          */
