@@ -71,7 +71,6 @@ typedef struct
 	AccountParserTag tag;
 
 	GaimAccount *account;
-	GaimProtocol protocol;
 	GaimProxyInfo *proxy_info;
 	char *protocol_id;
 
@@ -449,14 +448,6 @@ gaim_account_set_buddy_icon(GaimAccount *account, const char *icon)
 	schedule_accounts_save();
 }
 
-void
-gaim_account_set_protocol(GaimAccount *account, GaimProtocol protocol)
-{
-	g_return_if_fail(account != NULL);
-
-	gaim_account_set_protocol_id(account, gaim_prpl_num_to_id(protocol));
-}
-
 void
 gaim_account_set_protocol_id(GaimAccount *account, const char *protocol_id)
 {
@@ -728,14 +719,6 @@ gaim_account_get_buddy_icon(const GaimAccount *account)
 	return account->buddy_icon;
 }
 
-GaimProtocol
-gaim_account_get_protocol(const GaimAccount *account)
-{
-	g_return_val_if_fail(account != NULL, -1);
-
-	return gaim_prpl_id_to_num(gaim_account_get_protocol_id(account));
-}
-
 const char *
 gaim_account_get_protocol_id(const GaimAccount *account)
 {
@@ -744,6 +727,18 @@ gaim_account_get_protocol_id(const GaimAccount *account)
 	return account->protocol_id;
 }
 
+const char *
+gaim_account_get_protocol_name(const GaimAccount *account)
+{
+	GaimPlugin *p;
+
+	g_return_val_if_fail(account != NULL, NULL);
+
+	p = gaim_find_prpl(account->protocol_id);
+
+	return ((p && p->info->name) ? _(p->info->name) : _("Unknown"));
+}
+
 GaimConnection *
 gaim_account_get_connection(const GaimAccount *account)
 {
@@ -1216,7 +1211,7 @@ gaim_accounts_load()
 		g_free(filename);
 		accounts_loaded = TRUE;
 
-		return TRUE; /* we return TRUE so .gaimrc doesn't load */
+		return TRUE;
 	}
 
 	if (!g_markup_parse_context_end_parse(context, NULL)) {
@@ -1227,7 +1222,7 @@ gaim_accounts_load()
 		g_free(filename);
 		accounts_loaded = TRUE;
 
-		return TRUE; /* we return TRUE so .gaimrc doesn't load */
+		return TRUE;
 	}
 
 	g_markup_parse_context_free(context);
@@ -1253,7 +1248,7 @@ write_setting(gpointer key, gpointer value, gpointer user_data)
 		fprintf(fp, "   <setting name='%s' type='int'>%d</setting>\n",
 				name, setting->value.integer);
 	}
-	else if (setting->type == GAIM_PREF_STRING) {
+	else if (setting->type == GAIM_PREF_STRING && setting->value.string != NULL) {
 		fprintf(fp, "   <setting name='%s' type='string'>%s</setting>\n",
 				name, setting->value.string);
 	}
@@ -1367,6 +1362,7 @@ void
 gaim_accounts_sync(void)
 {
 	FILE *fp;
+	struct stat st;
 	const char *user_dir = gaim_user_dir();
 	char *filename;
 	char *filename_real;
@@ -1396,7 +1392,7 @@ gaim_accounts_sync(void)
 		GList *l;
 
 		fprintf(fp, "<?xml version='1.0' encoding='UTF-8' ?>\n\n");
-		fprintf(fp, "<accounts>\n");
+		fprintf(fp, "<accounts version='1.0'>\n");
 
 		for (l = gaim_accounts_get_all(); l != NULL; l = l->next)
 			gaim_accounts_write(fp, l->data);
@@ -1413,6 +1409,13 @@ gaim_accounts_sync(void)
 		return;
 	}
 
+	if (stat(filename, &st) || (st.st_size == 0)) {
+		gaim_debug_error("accounts", "Failed to save accounts\n");
+		unlink(filename);
+		g_free(filename);
+		return;
+	}
+
 	filename_real = g_build_filename(user_dir, "accounts.xml", NULL);
 
 	if (rename(filename, filename_real) < 0) {
@@ -1464,15 +1467,23 @@ gaim_accounts_delete(GaimAccount *account)
 	for (gnode = gaim_get_blist()->root; gnode != NULL; gnode = gnode->next) {
 		if (!GAIM_BLIST_NODE_IS_GROUP(gnode))
 			continue;
-		for (cnode = gnode->child; cnode; cnode = cnode->next) {
+
+		cnode = gnode->child;
+		while (cnode) {
+			GaimBlistNode *cnode_next = cnode->next;
+
 			if(GAIM_BLIST_NODE_IS_CONTACT(cnode)) {
-				for (bnode = cnode->child; bnode; bnode = bnode->next) {
+				bnode = cnode->child;
+				while (bnode) {
+					GaimBlistNode *bnode_next = bnode->next;
+
 					if (GAIM_BLIST_NODE_IS_BUDDY(bnode)) {
 						GaimBuddy *b = (GaimBuddy *)bnode;
 
 						if (b->account == account)
 							gaim_blist_remove_buddy(b);
 					}
+					bnode = bnode_next;
 				}
 			} else if (GAIM_BLIST_NODE_IS_CHAT(cnode)) {
 				GaimChat *c = (GaimChat *)cnode;
@@ -1480,9 +1491,9 @@ gaim_accounts_delete(GaimAccount *account)
 				if (c->account == account)
 					gaim_blist_remove_chat(c);
 			}
+			cnode = cnode_next;
 		}
 	}
-	gaim_blist_save();
 
 	/* Remove this account's pounces */
 	gaim_pounce_destroy_all_by_account(account);
@@ -1544,37 +1555,6 @@ gaim_accounts_get_all(void)
 	return accounts;
 }
 
-static GaimAccount *
-gaim_accounts_find_with_prpl_num(const char *name, GaimProtocol protocol)
-{
-	GaimAccount *account = NULL;
-	GList *l;
-	char *who;
-
-	g_return_val_if_fail(name != NULL, NULL);
-
-	who = g_strdup(gaim_normalize(NULL, name));
-
-	for (l = gaim_accounts_get_all(); l != NULL; l = l->next) {
-		account = (GaimAccount *)l->data;
-
-		if (!strcmp(gaim_normalize(NULL, gaim_account_get_username(account)), who)) {
-			if (protocol != -1) {
-				if (gaim_account_get_protocol(account) == protocol)
-					break;
-			}
-			else
-				break;
-		}
-
-		account = NULL;
-	}
-
-	g_free(who);
-
-	return account;
-}
-
 GaimAccount *
 gaim_accounts_find(const char *name, const char *protocol_id)
 {
@@ -1598,9 +1578,6 @@ gaim_accounts_find(const char *name, const char *protocol_id)
 		account = NULL;
 	}
 
-	if(!account && protocol_id)
-		account = gaim_accounts_find_with_prpl_num(name, atoi(protocol_id));
-
 	g_free(who);
 
 	return account;
