@@ -22,12 +22,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+#ifndef _GAIM_BLIST_H_
+#define _GAIM_BLIST_H_
 
 /* I can't believe I let ChipX86 inspire me to write good code. -Sean */
 
-#ifndef _BLIST_H_
-#define _BLIST_H_
-
 #include <glib.h>
 
 typedef struct _GaimBuddyList GaimBuddyList;
@@ -75,6 +74,14 @@ typedef enum
 		((b)->present == GAIM_BUDDY_ONLINE || \
 		 (b)->present == GAIM_BUDDY_SIGNING_ON))
 
+typedef enum
+{
+	GAIM_BLIST_NODE_FLAG_NO_SAVE = 1,	/**< node should not be saved with the buddy list */
+} GaimBlistNodeFlags;
+
+#define GAIM_BLIST_NODE_HAS_FLAG(b, f) ((b)->flags & (f))
+#define GAIM_BLIST_NODE_SHOULD_SAVE(b) (! GAIM_BLIST_NODE_HAS_FLAG(b, GAIM_BLIST_NODE_FLAG_NO_SAVE))
+
 
 /**************************************************************************/
 /* Data Structures                                                        */
@@ -91,6 +98,7 @@ struct _GaimBlistNode {
 	GaimBlistNode *child;               /**< The child of this node         */
 	GHashTable *settings;               /**< per-node settings              */
 	void          *ui_data;             /**< The UI can put data here.      */
+	GaimBlistNodeFlags flags;           /**< The buddy flags                */
 };
 
 /**
@@ -109,8 +117,8 @@ struct _GaimBuddy {
 	int uc;                                 /**< This is a cryptic bitmask that makes sense only to the prpl.  This will get changed */
 	void *proto_data;                       /**< This allows the prpl to associate whatever data it wants with a buddy */
 	GaimBuddyIcon *icon;                    /**< The buddy icon. */
-	GaimAccount *account;           /**< the account this buddy belongs to */
-	guint timer;							/**< The timer handle. */
+	GaimAccount *account;           	/**< the account this buddy belongs to */
+	guint timer;				/**< The timer handle. */
 };
 
 /**
@@ -163,7 +171,7 @@ struct _GaimBuddyList {
 /**
  * Buddy list UI operations.
  *
- * Any UI representing a buddy list must assign a filled-out gaim_blist_window_ops
+ * Any UI representing a buddy list must assign a filled-out GaimBlistUiOps
  * structure to the buddy list core.
  */
 struct _GaimBlistUiOps
@@ -181,7 +189,7 @@ struct _GaimBlistUiOps
 	void (*request_add_buddy)(GaimAccount *account, const char *username,
 							  const char *group, const char *alias);
 	void (*request_add_chat)(GaimAccount *account, GaimGroup *group, 
-							 const char *alias);
+							 const char *alias, const char *name);
 	void (*request_add_group)(void);
 };
 
@@ -255,9 +263,9 @@ void gaim_blist_update_buddy_status(GaimBuddy *buddy, int status);
  * Updates a buddy's presence.
  *
  * @param buddy    The buddy whose presence has changed
- * @param presence The new presence
+ * @param online   If the buddy is now online
  */
-void gaim_blist_update_buddy_presence(GaimBuddy *buddy, int presence);
+void gaim_blist_update_buddy_presence(GaimBuddy *buddy, gboolean online);
 
 /**
  * Updates a buddy's signon time.
@@ -493,6 +501,17 @@ void gaim_contact_set_alias(GaimContact *contact, const char *alias);
  */
 const char *gaim_contact_get_alias(GaimContact *contact);
 
+/**
+ * Determines whether an account owns any buddies in a given contact
+ *
+ * @param contact  The contact to search through.
+ * @param account  The account.
+ *
+ * @return TRUE if there are any buddies from account in the contact, or FALSE otherwise.
+ */
+gboolean gaim_contact_on_account(GaimContact *contact, GaimAccount *account);
+
+
 /**
  * Removes a buddy from the buddy list and frees the memory allocated to it.
  *
@@ -527,18 +546,33 @@ void gaim_blist_remove_group(GaimGroup *group);
  * Returns the alias of a buddy.
  *
  * @param buddy   The buddy whose name will be returned.
- * @return        The alias (if set), server alias (if option is set), or NULL.
+ * @return        The alias (if set), server alias (if set and preference
+ *                is to display server aliases), or NULL.
  */
-const char *gaim_get_buddy_alias_only(GaimBuddy *buddy);
+const char *gaim_buddy_get_alias_only(GaimBuddy *buddy);
 
 
 /**
- * Returns the correct name to display for a buddy.
+ * Returns the correct name to display for a buddy, taking the contact alias
+ * into account. In order of precedence: the buddy's alias; the buddy's
+ * contact alias; the buddy's server alias; the buddy's user name.
+ *
+ * @param buddy  The buddy whose name will be returned
+ * @return       The appropriate name or alias, or NULL.
+ *
+ */
+const char *gaim_buddy_get_contact_alias(GaimBuddy *buddy);
+
+
+/**
+ * Returns the correct name to display for a buddy. In order of precedence:
+ * the buddy's alias; the buddy's server alias; the buddy's contact alias;
+ * the buddy's user name.
  *
  * @param buddy   The buddy whose name will be returned.
- * @return        The alias (if set), server alias (if option is set), screenname, or "Unknown"
+ * @return        The appropriate name or alias, or NULL
  */
-const char *gaim_get_buddy_alias(GaimBuddy *buddy);
+const char *gaim_buddy_get_alias(GaimBuddy *buddy);
 
 /**
  * Returns the correct name to display for a blist chat.
@@ -569,7 +603,7 @@ GaimBuddy *gaim_find_buddy_in_group(GaimAccount *account, const char *name,
 		GaimGroup *group);
 
 /**
- * Finds all buddies struct given a screenname and an account
+ * Finds all GaimBuddy structs given a screenname and an account
  *
  * @param name    The buddy's screenname
  * @param account The account this buddy belongs to
@@ -677,9 +711,10 @@ int gaim_blist_get_group_online_count(GaimGroup *group);
 
 /*@{*/
 /**
- * Saves the buddy list to file
+ * Force an immediate write of the buddy list.  Normally the buddy list is
+ * saved automatically a few seconds after a change is made.
  */
-void gaim_blist_save();
+void gaim_blist_sync();
 
 /**
  * Parses the toc-style buddy list used in older versions of Gaim and for SSI in toc.c
@@ -714,9 +749,10 @@ void gaim_blist_request_add_buddy(GaimAccount *account, const char *username,
  * @param account The account the buddy is added to.
  * @param group   The optional group to add the chat to.
  * @param alias   The optional alias for the chat.
+ * @param name    The required chat name.
  */
 void gaim_blist_request_add_chat(GaimAccount *account, GaimGroup *group,
-								 const char *alias);
+								 const char *alias, const char *name);
 
 /**
  * Requests from the user information needed to add a group to the
@@ -790,79 +826,6 @@ const char *gaim_blist_node_get_string(GaimBlistNode *node, const char *key);
  */
 void gaim_blist_node_remove_setting(GaimBlistNode *node, const char *key);
 
-/**
- * Associates some data with the group in the xml buddy list
- *
- * @param g      The group the data is associated with
- * @param key    The key used to retrieve the data
- * @param value  The data to set
- * @deprecated   Replaced by gaim_blist_node_set_bool(), gaim_blist_node_set_int()
- * 		 and gaim_blist_node_set_string() to enable types and consolidate functionality.
- * 		 This function is scheduled to be removed in the near future.
- */
-void gaim_group_set_setting(GaimGroup *g, const char *key, const char *value);
-
-/**
- * Retrieves data from the XML buddy list set by gaim_group_set_setting())
- *
- * @param g      The group to retrieve data from
- * @param key    The key to retrieve the data with
- * @return       The associated data or NULL if no data is associated
- * @deprecated   Replaced by gaim_blist_node_get_bool(), gaim_blist_node_get_int()
- * 		 and gaim_blist_node_get_string() to enable types and consolidate functionality.
- * 		 This function is scheduled to be removed in the near future.
- */
-const char *gaim_group_get_setting(GaimGroup *g, const char *key);
-
-/**
- * Associates some data with the chat in the xml buddy list
- *
- * @param c      The chat the data is associated with
- * @param key    The key used to retrieve the data
- * @param value  The data to set
- * @deprecated   Replaced by gaim_blist_node_set_bool(), gaim_blist_node_set_int()
- * 		 and gaim_blist_node_set_string() to enable types and consolidate functionality.
- * 		 This function is scheduled to be removed in the near future.
- */
-void gaim_chat_set_setting(GaimChat *c, const char *key, const char *value);
-
-/**
- * Retrieves data from the XML buddy list set by gaim_chat_set_setting())
- *
- * @param c      The chat to retrieve data from
- * @param key    The key to retrieve the data with
- *
- * @return       The associated data or NULL if no data is associated
- * @deprecated   Replaced by gaim_blist_node_get_bool(), gaim_blist_node_get_int()
- * 		 and gaim_blist_node_get_string() to enable types and consolidate functionality.
- * 		 This function is scheduled to be removed in the near future.
- */
-const char *gaim_chat_get_setting(GaimChat *c, const char *key);
-
-/**
- * Associates some data with the buddy in the xml buddy list
- *
- * @param b      The buddy the data is associated with
- * @param key    The key used to retrieve the data
- * @param value  The data to set
- * @deprecated   Replaced by gaim_blist_node_set_bool(), gaim_blist_node_set_int()
- * 		 and gaim_blist_node_set_string() to enable types and consolidate functionality.
- * 		 This function is scheduled to be removed in the near future.
- */
-void gaim_buddy_set_setting(GaimBuddy *b, const char *key, const char *value);
-
-/**
- * Retrieves data from the XML buddy list set by gaim_buddy_set_setting())
- *
- * @param b      The buddy to retrieve data from
- * @param key    The key to retrieve the data with
- * @return       The associated data or NULL if no data is associated
- * @deprecated   Replaced by gaim_blist_node_get_bool(), gaim_blist_node_get_int()
- * 		 and gaim_blist_node_get_string() to enable types and consolidate functionality.
- * 		 This function is scheduled to be removed in the near future.
- */
-const char *gaim_buddy_get_setting(GaimBuddy *b, const char *key);
-
 /*@}*/
 
 
@@ -935,4 +898,4 @@ void gaim_blist_uninit(void);
 }
 #endif
 
-#endif /* _BLIST_H_ */
+#endif /* _GAIM_BLIST_H_ */
