@@ -1,9 +1,11 @@
 /**
- * @file ft.c The file transfer interface.
+ * @file ft.c File Transfer API
  *
  * gaim
  *
- * Copyright (C) 2003 Christian Hammond <chipx86@gnupdate.org>
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,9 +24,12 @@
  */
 #include "internal.h"
 #include "ft.h"
+#include "network.h"
 #include "notify.h"
 #include "prefs.h"
 #include "proxy.h"
+#include "request.h"
+#include "util.h"
 
 static GaimXferUiOps *xfer_ui_ops = NULL;
 
@@ -40,12 +45,12 @@ gaim_xfer_new(GaimAccount *account, GaimXferType type, const char *who)
 
 	xfer = g_new0(GaimXfer, 1);
 
+	xfer->ref = 1;
 	xfer->type    = type;
 	xfer->account = account;
 	xfer->who     = g_strdup(who);
-	xfer->ui_ops  = gaim_get_xfer_ui_ops();
-
-	xfer->local_ip = g_strdup(gaim_xfers_get_ip_for_account(account));
+	xfer->ui_ops  = gaim_xfers_get_ui_ops();
+	xfer->message = NULL;
 
 	ui_ops = gaim_xfer_get_ui_ops(xfer);
 
@@ -55,14 +60,17 @@ gaim_xfer_new(GaimAccount *account, GaimXferType type, const char *who)
 	return xfer;
 }
 
-void
+static void
 gaim_xfer_destroy(GaimXfer *xfer)
 {
 	GaimXferUiOps *ui_ops;
 
 	g_return_if_fail(xfer != NULL);
 
-	if (!xfer->completed)
+	/* Close the file browser, if it's open */
+	gaim_request_close_with_handle(xfer);
+
+	if (gaim_xfer_get_status(xfer) == GAIM_XFER_STATUS_STARTED)
 		gaim_xfer_cancel_local(xfer);
 
 	ui_ops = gaim_xfer_get_ui_ops(xfer);
@@ -72,90 +80,316 @@ gaim_xfer_destroy(GaimXfer *xfer)
 
 	g_free(xfer->who);
 	g_free(xfer->filename);
-
-	if (xfer->remote_ip != NULL) g_free(xfer->remote_ip);
-	if (xfer->local_ip  != NULL) g_free(xfer->local_ip);
-
-	if (xfer->local_filename != NULL)
-		g_free(xfer->local_filename);
+	g_free(xfer->remote_ip);
+	g_free(xfer->local_filename);
 
 	g_free(xfer);
 }
 
 void
-gaim_xfer_request(GaimXfer *xfer)
+gaim_xfer_ref(GaimXfer *xfer)
 {
-	GaimXferUiOps *ui_ops;
-
 	g_return_if_fail(xfer != NULL);
-	g_return_if_fail(xfer->ops.init != NULL);
 
-	ui_ops = gaim_get_xfer_ui_ops();
-
-	if (ui_ops == NULL || ui_ops->request_file == NULL)
-		return;
-
-	ui_ops->request_file(xfer);
+	xfer->ref++;
 }
 
 void
-gaim_xfer_request_accepted(GaimXfer *xfer, char *filename)
+gaim_xfer_unref(GaimXfer *xfer)
+{
+	g_return_if_fail(xfer != NULL);
+
+	xfer->ref--;
+
+	if (xfer->ref == 0)
+		gaim_xfer_destroy(xfer);
+}
+
+static void
+gaim_xfer_set_status(GaimXfer *xfer, GaimXferStatusType status)
+{
+	g_return_if_fail(xfer != NULL);
+
+	xfer->status = status;
+}
+
+static void
+gaim_xfer_conversation_write(GaimXfer *xfer, char *message, gboolean is_error)
+{
+	GaimConversation *conv = NULL;
+	GString *gs_message = NULL;
+	GaimMessageFlags flags = GAIM_MESSAGE_SYSTEM;
+
+	g_return_if_fail(xfer != NULL);
+	g_return_if_fail(message != NULL);
+
+	conv = gaim_find_conversation_with_account(xfer->who, gaim_xfer_get_account(xfer));
+
+	if (conv == NULL)
+		return;
+
+	gs_message = g_string_new(message);
+
+	if (is_error)
+		flags = GAIM_MESSAGE_ERROR;
+
+	gaim_conversation_write(conv, NULL, gs_message->str, flags, time(NULL));
+	g_string_free(gs_message, TRUE);
+}
+
+static void gaim_xfer_show_file_error(GaimXfer *xfer, const char *filename)
+{
+	gchar *msg = NULL;
+	GaimXferType xfer_type = gaim_xfer_get_type(xfer);
+
+	switch(xfer_type) {
+		case GAIM_XFER_SEND:
+			msg = g_strdup_printf(_("Error reading %s: \n%s.\n"),
+								  filename, strerror(errno));
+			break;
+		case GAIM_XFER_RECEIVE:
+			msg = g_strdup_printf(_("Error writing %s: \n%s.\n"),
+								  filename, strerror(errno));
+			break;
+		default:
+			msg = g_strdup_printf(_("Error accessing %s: \n%s.\n"),
+								  filename, strerror(errno));
+			break;
+		}
+
+	gaim_xfer_conversation_write(xfer, msg, TRUE);
+	gaim_xfer_error(xfer_type, xfer->who, msg);
+	g_free(msg);
+}
+
+static void
+gaim_xfer_choose_file_ok_cb(void *user_data, const char *filename)
+{
+	GaimXfer *xfer;
+	struct stat st;
+
+	xfer = (GaimXfer *)user_data;
+
+	if (stat(filename, &st) != 0) {
+		/* File not found. */
+		if (gaim_xfer_get_type(xfer) == GAIM_XFER_RECEIVE) {
+			gaim_xfer_request_accepted(xfer, filename);
+		}
+		else {
+		  	gaim_xfer_show_file_error(xfer, filename);
+			gaim_xfer_request_denied(xfer);
+		}
+	}
+	else if ((gaim_xfer_get_type(xfer) == GAIM_XFER_SEND) &&
+			 (st.st_size == 0)) {
+
+		gaim_notify_error(NULL, NULL,
+						  _("Cannot send a file of 0 bytes."), NULL);
+
+		gaim_xfer_request_denied(xfer);
+	}
+	else if ((gaim_xfer_get_type(xfer) == GAIM_XFER_SEND) &&
+			 S_ISDIR(st.st_mode)) {
+		/*
+		 * XXX - Sending a directory should be valid for some protocols.
+		 */
+		gaim_notify_error(NULL, NULL,
+						  _("Cannot send a directory."), NULL);
+
+		gaim_xfer_request_denied(xfer);
+	}
+	else if ((gaim_xfer_get_type(xfer) == GAIM_XFER_RECEIVE) &&
+			 S_ISDIR(st.st_mode)) {
+				char *msg = g_strdup_printf(
+					_("%s is not a regular file. Cowardly refusing to overwrite it.\n"), filename);
+		gaim_notify_error(NULL, NULL, msg, NULL);
+		g_free(msg);
+		gaim_xfer_request_denied(xfer);
+	}
+	else {
+		gaim_xfer_request_accepted(xfer, filename);
+	}
+
+	gaim_xfer_unref(xfer);
+}
+
+static void
+gaim_xfer_choose_file_cancel_cb(void *user_data, const char *filename)
+{
+	GaimXfer *xfer = (GaimXfer *)user_data;
+
+  	gaim_xfer_set_status(xfer, GAIM_XFER_STATUS_CANCEL_LOCAL);
+	gaim_xfer_request_denied(xfer);
+}
+
+static int
+gaim_xfer_choose_file(GaimXfer *xfer)
+{
+	gaim_request_file(xfer, NULL, gaim_xfer_get_filename(xfer),
+					  (gaim_xfer_get_type(xfer) == GAIM_XFER_RECEIVE),
+					  G_CALLBACK(gaim_xfer_choose_file_ok_cb),
+					  G_CALLBACK(gaim_xfer_choose_file_cancel_cb), xfer);
+
+	return 0;
+}
+
+static int
+cancel_recv_cb(GaimXfer *xfer)
+{
+  	gaim_xfer_set_status(xfer, GAIM_XFER_STATUS_CANCEL_LOCAL);
+	gaim_xfer_request_denied(xfer);
+	gaim_xfer_unref(xfer);
+
+	return 0;
+}
+
+static void
+gaim_xfer_ask_recv(GaimXfer *xfer)
+{
+	char *buf, *size_buf, *escaped;
+	size_t size;
+
+	/* If we have already accepted the request, ask the destination file
+	   name directly */
+	if (gaim_xfer_get_status(xfer) != GAIM_XFER_STATUS_ACCEPTED) {
+		if (gaim_xfer_get_filename(xfer) != NULL)
+		{
+			size = gaim_xfer_get_size(xfer);
+			size_buf = gaim_str_size_to_units(size);
+			escaped = g_markup_escape_text(gaim_xfer_get_filename(xfer), -1);
+			buf = g_strdup_printf(_("%s wants to send you %s (%s)"),
+					      xfer->who, escaped,
+					      size_buf);
+			g_free(size_buf);
+			g_free(escaped);
+		}
+		else
+		{
+			buf = g_strdup_printf(_("%s wants to send you a file"), xfer->who);
+		}
+
+		if (xfer->message != NULL)
+			serv_got_im(gaim_account_get_connection(xfer->account),
+								 xfer->who, xfer->message, 0, time(NULL));
+
+		gaim_request_accept_cancel(xfer, NULL, buf, NULL, GAIM_DEFAULT_ACTION_NONE, xfer,
+					   G_CALLBACK(gaim_xfer_choose_file),
+					   G_CALLBACK(cancel_recv_cb));
+
+		g_free(buf);
+	} else
+		gaim_xfer_choose_file(xfer);
+}
+
+static int
+ask_accept_ok(GaimXfer *xfer)
+{
+	gaim_xfer_request_accepted(xfer, NULL);
+
+	return 0;
+}
+
+static int
+ask_accept_cancel(GaimXfer *xfer)
+{
+	gaim_xfer_request_denied(xfer);
+	gaim_xfer_unref(xfer);
+
+	return 0;
+}
+
+static void
+gaim_xfer_ask_accept(GaimXfer *xfer)
+{
+	char *buf, *buf2 = NULL;
+
+	buf = g_strdup_printf(_("Accept file transfer request from %s?"),
+			      xfer->who);
+	if (gaim_xfer_get_remote_ip(xfer) &&
+	    gaim_xfer_get_remote_port(xfer))
+		buf2 = g_strdup_printf(_("A file is available for download from:\n"
+					 "Remote host: %s\nRemote port: %d"),
+				       gaim_xfer_get_remote_ip(xfer),
+				       gaim_xfer_get_remote_port(xfer));
+	gaim_request_accept_cancel(xfer, NULL, buf, buf2, GAIM_DEFAULT_ACTION_NONE, xfer,
+				   G_CALLBACK(ask_accept_ok),
+				   G_CALLBACK(ask_accept_cancel));
+	g_free(buf);
+	g_free(buf2);
+}
+
+void
+gaim_xfer_request(GaimXfer *xfer)
+{
+	g_return_if_fail(xfer != NULL);
+	g_return_if_fail(xfer->ops.init != NULL);
+
+	gaim_xfer_ref(xfer);
+
+	if (gaim_xfer_get_type(xfer) == GAIM_XFER_RECEIVE) {
+		if (gaim_xfer_get_filename(xfer) ||
+		    gaim_xfer_get_status(xfer) == GAIM_XFER_STATUS_ACCEPTED)
+			gaim_xfer_ask_recv(xfer);
+		else
+			gaim_xfer_ask_accept(xfer);
+	} else
+		gaim_xfer_choose_file(xfer);
+}
+
+void
+gaim_xfer_request_accepted(GaimXfer *xfer, const char *filename)
 {
 	GaimXferType type;
+	struct stat st;
 
-	if (xfer == NULL || filename == NULL) {
-
-		if (filename != NULL)
-			g_free(filename);
-
+	if (xfer == NULL)
 		return;
-	}
 
 	type = gaim_xfer_get_type(xfer);
 
+	if (!filename && type == GAIM_XFER_RECEIVE) {
+		xfer->status = GAIM_XFER_STATUS_ACCEPTED;
+		xfer->ops.init(xfer);
+		return;
+	}
+
 	if (type == GAIM_XFER_SEND) {
-		struct stat sb;
+		char *msg;
 
 		/* Check the filename. */
 		if (g_strrstr(filename, "..")) {
-			char *msg;
-
-			msg = g_strdup_printf(_("%s is not a valid filename.\n"),
-								  filename);
 
+			msg = g_strdup_printf(_("%s is not a valid filename.\n"), filename);
 			gaim_xfer_error(type, xfer->who, msg);
-
 			g_free(msg);
-			g_free(filename);
 
+			gaim_xfer_unref(xfer);
 			return;
 		}
 
-		if (stat(filename, &sb) == -1) {
-			char *msg;
-
-			msg = g_strdup_printf(_("%s was not found.\n"), filename);
-
-			gaim_xfer_error(type, xfer->who, msg);
-
-			g_free(msg);
-			g_free(filename);
-
+		if (stat(filename, &st) == -1) {
+		  	gaim_xfer_show_file_error(xfer, filename);
+			gaim_xfer_unref(xfer);
 			return;
 		}
 
 		gaim_xfer_set_local_filename(xfer, filename);
-		gaim_xfer_set_size(xfer, sb.st_size);
+		gaim_xfer_set_filename(xfer, g_basename(filename));
+		gaim_xfer_set_size(xfer, st.st_size);
+		
+		msg = g_strdup_printf(_("Offering to send %s to %s"),
+							  filename, xfer->who);
+		gaim_xfer_conversation_write(xfer, msg, FALSE);
+		g_free(msg);
 	}
 	else {
-		/* TODO: Sanity checks and file overwrite checks. */
-
+		xfer->status = GAIM_XFER_STATUS_ACCEPTED;
 		gaim_xfer_set_local_filename(xfer, filename);
 	}
 
-	g_free(filename);
-
+	gaim_xfer_add(xfer);
 	xfer->ops.init(xfer);
+
 }
 
 void
@@ -163,9 +397,10 @@ gaim_xfer_request_denied(GaimXfer *xfer)
 {
 	g_return_if_fail(xfer != NULL);
 
-	gaim_xfer_destroy(xfer);
+	if (xfer->ops.request_denied != NULL)
+		xfer->ops.request_denied(xfer);
 
-	/* TODO */
+	gaim_xfer_unref(xfer);
 }
 
 GaimXferType
@@ -184,12 +419,32 @@ gaim_xfer_get_account(const GaimXfer *xfer)
 	return xfer->account;
 }
 
+GaimXferStatusType
+gaim_xfer_get_status(const GaimXfer *xfer)
+{
+	g_return_val_if_fail(xfer != NULL, GAIM_XFER_STATUS_UNKNOWN);
+
+	return xfer->status;
+}
+
+gboolean
+gaim_xfer_is_canceled(const GaimXfer *xfer)
+{
+	g_return_val_if_fail(xfer != NULL, TRUE);
+
+	if ((gaim_xfer_get_status(xfer) == GAIM_XFER_STATUS_CANCEL_LOCAL) ||
+	    (gaim_xfer_get_status(xfer) == GAIM_XFER_STATUS_CANCEL_REMOTE))
+		return TRUE;
+	else
+		return FALSE;
+}
+
 gboolean
 gaim_xfer_is_completed(const GaimXfer *xfer)
 {
 	g_return_val_if_fail(xfer != NULL, TRUE);
 
-	return xfer->completed;
+	return (gaim_xfer_get_status(xfer) == GAIM_XFER_STATUS_DONE);
 }
 
 const char *
@@ -244,14 +499,6 @@ gaim_xfer_get_progress(const GaimXfer *xfer)
 			(double)gaim_xfer_get_size(xfer));
 }
 
-const char *
-gaim_xfer_get_local_ip(const GaimXfer *xfer)
-{
-	g_return_val_if_fail(xfer != NULL, NULL);
-
-	return xfer->local_ip;
-}
-
 unsigned int
 gaim_xfer_get_local_port(const GaimXfer *xfer)
 {
@@ -283,13 +530,26 @@ gaim_xfer_set_completed(GaimXfer *xfer, gboolean completed)
 
 	g_return_if_fail(xfer != NULL);
 
-	xfer->completed = completed;
+	if (completed == TRUE)
+		gaim_xfer_set_status(xfer, GAIM_XFER_STATUS_DONE);
 
 	ui_ops = gaim_xfer_get_ui_ops(xfer);
 
 	if (ui_ops != NULL && ui_ops->update_progress != NULL)
 		ui_ops->update_progress(xfer, gaim_xfer_get_progress(xfer));
+}
 
+void
+gaim_xfer_set_message(GaimXfer *xfer, const char *message)
+{
+	g_return_if_fail(xfer != NULL);
+
+	g_free(xfer->message);
+
+	if (message != NULL)
+		xfer->message = g_strdup(message);
+	else
+		xfer->message = NULL;
 }
 
 void
@@ -341,9 +601,15 @@ gaim_xfer_set_init_fnc(GaimXfer *xfer, void (*fnc)(GaimXfer *))
 	xfer->ops.init = fnc;
 }
 
+void gaim_xfer_set_request_denied_fnc(GaimXfer *xfer, void (*fnc)(GaimXfer *))
+{
+	g_return_if_fail(xfer != NULL);
+
+	xfer->ops.request_denied = fnc;
+}
 
 void
-gaim_xfer_set_read_fnc(GaimXfer *xfer, size_t (*fnc)(char **, GaimXfer *))
+gaim_xfer_set_read_fnc(GaimXfer *xfer, ssize_t (*fnc)(char **, GaimXfer *))
 {
 	g_return_if_fail(xfer != NULL);
 
@@ -352,7 +618,7 @@ gaim_xfer_set_read_fnc(GaimXfer *xfer, size_t (*fnc)(char **, GaimXfer *))
 
 void
 gaim_xfer_set_write_fnc(GaimXfer *xfer,
-						size_t (*fnc)(const char *, size_t, GaimXfer *))
+						ssize_t (*fnc)(const char *, size_t, GaimXfer *))
 {
 	g_return_if_fail(xfer != NULL);
 
@@ -400,10 +666,10 @@ gaim_xfer_set_cancel_recv_fnc(GaimXfer *xfer, void (*fnc)(GaimXfer *))
 	xfer->ops.cancel_recv = fnc;
 }
 
-size_t
+ssize_t
 gaim_xfer_read(GaimXfer *xfer, char **buffer)
 {
-	size_t s, r;
+	ssize_t s, r;
 
 	g_return_val_if_fail(xfer   != NULL, 0);
 	g_return_val_if_fail(buffer != NULL, 0);
@@ -422,15 +688,17 @@ gaim_xfer_read(GaimXfer *xfer, char **buffer)
 		if ((gaim_xfer_get_size(xfer) > 0) &&
 			((gaim_xfer_get_bytes_sent(xfer)+r) >= gaim_xfer_get_size(xfer)))
 			gaim_xfer_set_completed(xfer, TRUE);
+		else if(r <= 0)
+			r = -1;
 	}
 
 	return r;
 }
 
-size_t
+ssize_t
 gaim_xfer_write(GaimXfer *xfer, const char *buffer, size_t size)
 {
-	size_t r, s;
+	ssize_t r, s;
 
 	g_return_val_if_fail(xfer   != NULL, 0);
 	g_return_val_if_fail(buffer != NULL, 0);
@@ -438,10 +706,13 @@ gaim_xfer_write(GaimXfer *xfer, const char *buffer, size_t size)
 
 	s = MIN(gaim_xfer_get_bytes_remaining(xfer), size);
 
-	if (xfer->ops.write != NULL)
+	if (xfer->ops.write != NULL) {
 		r = (xfer->ops.write)(buffer, s, xfer);
-	else
+	} else {
 		r = write(xfer->fd, buffer, s);
+		if ((gaim_xfer_get_bytes_sent(xfer)+r) >= gaim_xfer_get_size(xfer))
+			gaim_xfer_set_completed(xfer, TRUE);
+	}
 
 	return r;
 }
@@ -452,16 +723,31 @@ transfer_cb(gpointer data, gint source, GaimInputCondition condition)
 	GaimXferUiOps *ui_ops;
 	GaimXfer *xfer = (GaimXfer *)data;
 	char *buffer = NULL;
-	size_t r;
+	ssize_t r = 0;
 
 	if (condition & GAIM_INPUT_READ) {
 		r = gaim_xfer_read(xfer, &buffer);
-		if (r > 0)
+		if (r > 0) {
 			fwrite(buffer, 1, r, xfer->dest_fp);
+		} else if(r < 0) {
+			gaim_xfer_cancel_remote(xfer);
+			return;
+		}
 	}
-	else {
+
+	if (condition & GAIM_INPUT_WRITE) {
 		size_t s = MIN(gaim_xfer_get_bytes_remaining(xfer), 4096);
 
+		/* this is so the prpl can keep the connection open
+		   if it needs to for some odd reason. */
+		if (s == 0) {
+			if (xfer->watcher) {
+				gaim_input_remove(xfer->watcher);
+				xfer->watcher = 0;
+			}
+			return;
+		}
+
 		buffer = g_malloc0(s);
 
 		fread(buffer, 1, s, xfer->dest_fp);
@@ -469,7 +755,11 @@ transfer_cb(gpointer data, gint source, GaimInputCondition condition)
 		/* Write as much as we're allowed to. */
 		r = gaim_xfer_write(xfer, buffer, s);
 
-		if (r < s) {
+		if (r == -1) {
+			gaim_xfer_cancel_remote(xfer);
+			g_free(buffer);
+			return;
+		} else if (r < s) {
 			/* We have to seek back in the file now. */
 			fseek(xfer->dest_fp, r - s, SEEK_CUR);
 		}
@@ -497,29 +787,19 @@ transfer_cb(gpointer data, gint source, GaimInputCondition condition)
 static void
 begin_transfer(GaimXfer *xfer, GaimInputCondition cond)
 {
-	GaimXferUiOps *ui_ops;
 	GaimXferType type = gaim_xfer_get_type(xfer);
 
-	ui_ops = gaim_xfer_get_ui_ops(xfer);
-
-	/*
-	 * We'll have already tried to open this earlier to make sure we can
-	 * read/write here. Should be safe.
-	 */
 	xfer->dest_fp = fopen(gaim_xfer_get_local_filename(xfer),
 						  type == GAIM_XFER_RECEIVE ? "wb" : "rb");
 
-	/* Just in case, though. */
 	if (xfer->dest_fp == NULL) {
-		gaim_xfer_cancel_local(xfer); /* ? */
+		gaim_xfer_show_file_error(xfer, gaim_xfer_get_local_filename(xfer));
+		gaim_xfer_cancel_local(xfer);
 		return;
 	}
 
 	xfer->watcher = gaim_input_add(xfer->fd, cond, transfer_cb, xfer);
 
-	if (ui_ops != NULL && ui_ops->add_xfer != NULL)
-		ui_ops->add_xfer(xfer);
-
 	if (xfer->ops.start != NULL)
 		xfer->ops.start(xfer);
 }
@@ -549,6 +829,8 @@ gaim_xfer_start(GaimXfer *xfer, int fd, const char *ip,
 	xfer->bytes_remaining = gaim_xfer_get_size(xfer);
 	xfer->bytes_sent      = 0;
 
+	gaim_xfer_set_status(xfer, GAIM_XFER_STATUS_STARTED);
+
 	if (type == GAIM_XFER_RECEIVE) {
 		cond = GAIM_INPUT_READ;
 
@@ -578,14 +860,24 @@ gaim_xfer_start(GaimXfer *xfer, int fd, const char *ip,
 void
 gaim_xfer_end(GaimXfer *xfer)
 {
+	char *msg = NULL;
+
 	g_return_if_fail(xfer != NULL);
 
 	/* See if we are actually trying to cancel this. */
-	if (!xfer->completed) {
+	if (gaim_xfer_get_status(xfer) != GAIM_XFER_STATUS_DONE) {
 		gaim_xfer_cancel_local(xfer);
 		return;
 	}
 
+	if (gaim_xfer_get_filename(xfer) != NULL)
+		msg = g_strdup_printf(_("Transfer of file %s complete"),
+							  gaim_xfer_get_filename(xfer));
+	else
+		msg = g_strdup_printf(_("File transfer complete"));
+	gaim_xfer_conversation_write(xfer, msg, FALSE);
+	g_free(msg);
+
 	if (xfer->ops.end != NULL)
 		xfer->ops.end(xfer);
 
@@ -601,15 +893,47 @@ gaim_xfer_end(GaimXfer *xfer)
 		fclose(xfer->dest_fp);
 		xfer->dest_fp = NULL;
 	}
+
+	gaim_xfer_unref(xfer);
+}
+
+void
+gaim_xfer_add(GaimXfer *xfer)
+{
+	GaimXferUiOps *ui_ops;
+
+	g_return_if_fail(xfer != NULL);
+
+	ui_ops = gaim_xfer_get_ui_ops(xfer);
+
+	if (ui_ops != NULL && ui_ops->add_xfer != NULL)
+		ui_ops->add_xfer(xfer);
 }
 
 void
 gaim_xfer_cancel_local(GaimXfer *xfer)
 {
 	GaimXferUiOps *ui_ops;
+	char *msg = NULL, *escaped;
 
 	g_return_if_fail(xfer != NULL);
 
+	gaim_xfer_set_status(xfer, GAIM_XFER_STATUS_CANCEL_LOCAL);
+
+	if (gaim_xfer_get_filename(xfer) != NULL)
+	{
+		escaped = g_markup_escape_text(gaim_xfer_get_filename(xfer), -1);
+		msg = g_strdup_printf(_("You canceled the transfer of %s"),
+							  escaped);
+		g_free(escaped);
+	}
+	else
+	{
+		msg = g_strdup_printf(_("File transfer cancelled"));
+	}
+	gaim_xfer_conversation_write(xfer, msg, FALSE);
+	g_free(msg);
+
 	if (gaim_xfer_get_type(xfer) == GAIM_XFER_SEND)
 	{
 		if (xfer->ops.cancel_send != NULL)
@@ -640,15 +964,36 @@ gaim_xfer_cancel_local(GaimXfer *xfer)
 		ui_ops->cancel_local(xfer);
 
 	xfer->bytes_remaining = 0;
+
+	gaim_xfer_unref(xfer);
 }
 
 void
 gaim_xfer_cancel_remote(GaimXfer *xfer)
 {
 	GaimXferUiOps *ui_ops;
+	gchar *msg, *escaped;
 
 	g_return_if_fail(xfer != NULL);
 
+	gaim_request_close_with_handle(xfer);
+	gaim_xfer_set_status(xfer, GAIM_XFER_STATUS_CANCEL_REMOTE);
+
+	if (gaim_xfer_get_filename(xfer) != NULL)
+	{
+		escaped = g_markup_escape_text(gaim_xfer_get_filename(xfer), -1);
+		msg = g_strdup_printf(_("%s canceled the transfer of %s"),
+							  xfer->who, escaped);
+		g_free(escaped);
+	}
+	else
+	{
+		msg = g_strdup_printf(_("%s canceled the file transfer"), xfer->who);
+	}
+	gaim_xfer_conversation_write(xfer, msg, TRUE);
+	gaim_xfer_error(gaim_xfer_get_type(xfer), xfer->who, msg);
+	g_free(msg);
+
 	if (gaim_xfer_get_type(xfer) == GAIM_XFER_SEND)
 	{
 		if (xfer->ops.cancel_send != NULL)
@@ -658,7 +1003,7 @@ gaim_xfer_cancel_remote(GaimXfer *xfer)
 	{
 		if (xfer->ops.cancel_recv != NULL)
 			xfer->ops.cancel_recv(xfer);
-	}	
+	}
 
 	if (xfer->watcher != 0) {
 		gaim_input_remove(xfer->watcher);
@@ -679,6 +1024,8 @@ gaim_xfer_cancel_remote(GaimXfer *xfer)
 		ui_ops->cancel_remote(xfer);
 
 	xfer->bytes_remaining = 0;
+
+	gaim_xfer_unref(xfer);
 }
 
 void
@@ -699,83 +1046,31 @@ gaim_xfer_error(GaimXferType type, const char *who, const char *msg)
 	g_free(title);
 }
 
+void
+gaim_xfer_update_progress(GaimXfer *xfer)
+{
+	GaimXferUiOps *ui_ops;
+
+	g_return_if_fail(xfer != NULL);
+
+	ui_ops = gaim_xfer_get_ui_ops(xfer);
+	if (ui_ops != NULL && ui_ops->update_progress != NULL)
+		ui_ops->update_progress(xfer, gaim_xfer_get_progress(xfer));
+}
+
+
 /**************************************************************************
  * File Transfer Subsystem API
  **************************************************************************/
-void
-gaim_xfers_set_local_ip(const char *ip)
-{
-	g_return_if_fail(ip != NULL);
-
-	gaim_prefs_set_string("/core/ft/public_ip", ip);
-}
-
-const char *
-gaim_xfers_get_local_ip(void)
-{
-	const char *ip;
-
-	ip = gaim_prefs_get_string("/core/ft/public_ip");
-
-	if (ip == NULL || *ip == '\0')
-		return NULL;
-
-	return ip;
-}
-
-const char *
-gaim_xfers_get_local_system_ip(void)
-{
-	struct hostent *host;
-	char localhost[129];
-	long unsigned add;
-	static char ip[46];
-
-	if (gethostname(localhost, 128) < 0)
-		return NULL;
-
-	if ((host = gethostbyname(localhost)) == NULL)
-		return NULL;
-
-	memcpy(&add, host->h_addr_list[0], 4);
-	add = htonl(add);
-
-	g_snprintf(ip, 11, "%lu", add);
-
-	return ip;
-}
-
-const char *
-gaim_xfers_get_ip_for_account(const GaimAccount *account)
-{
-	g_return_val_if_fail(account != NULL, NULL);
-
-	if (gaim_account_get_public_ip(account) != NULL)
-		return gaim_account_get_public_ip(account);
-	else if (gaim_xfers_get_local_ip() != NULL)
-		return gaim_xfers_get_local_ip();
-	else
-		return gaim_xfers_get_local_system_ip();
-}
 
 void
-gaim_xfers_init(void)
+gaim_xfers_set_ui_ops(GaimXferUiOps *ops)
 {
-	gaim_prefs_add_none("/core/ft");
-	gaim_prefs_add_string("/core/ft/public_ip", "");
-}
-
-void
-gaim_set_xfer_ui_ops(GaimXferUiOps *ops)
-{
-	g_return_if_fail(ops != NULL);
-
 	xfer_ui_ops = ops;
 }
 
 GaimXferUiOps *
-gaim_get_xfer_ui_ops(void)
+gaim_xfers_get_ui_ops(void)
 {
 	return xfer_ui_ops;
 }
-
