@@ -22,7 +22,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-#include "gtkinternal.h"
+
+#include "internal.h"
+#include "gtkgaim.h"
 
 #include "account.h"
 #include "accountopt.h"
@@ -31,6 +33,7 @@
 #include "notify.h"
 #include "plugin.h"
 #include "prefs.h"
+#include "prpl.h"
 #include "request.h"
 #include "signals.h"
 #include "util.h"
@@ -38,11 +41,10 @@
 #include "gaim-disclosure.h"
 #include "gtkaccount.h"
 #include "gtkblist.h"
+#include "gtkdialogs.h"
 #include "gtkutils.h"
 #include "stock.h"
 
-#include "ui.h"
-
 enum
 {
 	COLUMN_ICON,
@@ -115,11 +117,12 @@ typedef struct
 	/* User Options */
 	GtkWidget *user_frame;
 	GtkWidget *new_mail_check;
-	GtkWidget *buddy_icon_hbox;
-	GtkWidget *buddy_icon_entry;
-	GtkWidget *buddy_icon_filesel;
-	GtkWidget *buddy_icon_preview;
-	GtkWidget *buddy_icon_text;
+	GtkWidget *icon_hbox;
+	GtkWidget *icon_entry;
+	char *icon_path;
+	GtkWidget *icon_filesel;
+	GtkWidget *icon_preview;
+	GtkWidget *icon_text;
 
 	/* Protocol Options */
 	GtkWidget *protocol_frame;
@@ -155,14 +158,8 @@ static AccountsWindow *accounts_window = NULL;
 static void add_account(AccountsWindow *dialog, GaimAccount *account);
 static void set_account(GtkListStore *store, GtkTreeIter *iter,
 						  GaimAccount *account);
-
-static char *
-proto_name(const char *id)
-{
-	GaimPlugin *p = gaim_find_prpl(id);
-
-	return ((p && p->info->name) ? _(p->info->name) : _("Unknown"));
-}
+static char*
+convert_buddy_icon(GaimPlugin *plugin, const char *path);
 
 /**************************************************************************
  * Add/Modify Account dialog
@@ -201,7 +198,17 @@ static void
 set_account_protocol_cb(GtkWidget *item, const char *id,
 						AccountPrefsDialog *dialog)
 {
-	if ((dialog->plugin = gaim_find_prpl(id)) != NULL) {
+	GaimPlugin *new_plugin;
+
+	new_plugin = gaim_find_prpl(id);
+
+	if (new_plugin == dialog->plugin)
+		return;
+
+	dialog->plugin = new_plugin;
+
+	if (dialog->plugin != NULL)
+	{
 		dialog->prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(dialog->plugin);
 
 		if (dialog->protocol_id != NULL)
@@ -210,6 +217,9 @@ set_account_protocol_cb(GtkWidget *item, const char *id,
 		dialog->protocol_id = g_strdup(dialog->plugin->info->id);
 	}
 
+	if (dialog->account != NULL)
+		gaim_account_clear_settings(dialog->account);
+
 	add_login_options(dialog,    dialog->top_vbox);
 	add_user_options(dialog,     dialog->top_vbox);
 	add_protocol_options(dialog, dialog->bottom_vbox);
@@ -232,53 +242,85 @@ screenname_changed_cb(GtkEntry *entry, AccountPrefsDialog *dialog)
 
 }
 
+#if GTK_CHECK_VERSION(2,4,0) /* FILECHOOSER */
 static void
-buddy_icon_filesel_delete_cb(GtkWidget *w, AccountPrefsDialog *dialog)
+icon_filesel_choose_cb(GtkWidget *widget, gint response, AccountPrefsDialog *dialog)
 {
-	if (dialog->buddy_icon_filesel != NULL)
-		gtk_widget_destroy(dialog->buddy_icon_filesel);
+	char *filename;
 
-	dialog->buddy_icon_filesel = NULL;
-}
-
-static void
-buddy_icon_filesel_choose(GtkWidget *w, AccountPrefsDialog *dialog)
-{
-	const char *filename;
-
-	filename = gtk_file_selection_get_filename(
-			GTK_FILE_SELECTION(dialog->buddy_icon_filesel));
-
-	/* If they typed in a directory, change there */
-	if (gaim_gtk_check_if_dir(filename,
-			GTK_FILE_SELECTION(dialog->buddy_icon_filesel)))
-	{
+	if (response != GTK_RESPONSE_ACCEPT) {
+		if (response == GTK_RESPONSE_CANCEL)
+			gtk_widget_destroy(dialog->icon_filesel);
+		dialog->icon_filesel = NULL;
 		return;
 	}
 
-	gtk_entry_set_text(GTK_ENTRY(dialog->buddy_icon_entry), filename);
-	gtk_widget_destroy(dialog->buddy_icon_filesel);
-}
+	filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog->icon_filesel));
+#else /* FILECHOOSER */
+static void
+icon_filesel_choose_cb(GtkWidget *w, AccountPrefsDialog *dialog)
+{
+	char *filename;
+
+	filename = g_strdup(gtk_file_selection_get_filename(
+			GTK_FILE_SELECTION(dialog->icon_filesel)));
+
+	/* If they typed in a directory, change there */
+	if (gaim_gtk_check_if_dir(filename,
+			GTK_FILE_SELECTION(dialog->icon_filesel)))
+	{
+		g_free(filename);
+		return;
+	}
+#endif /* FILECHOOSER */
+
+	if (dialog->icon_path)
+		g_free(dialog->icon_path);
+	dialog->icon_path = convert_buddy_icon(dialog->plugin, filename);
+	gtk_image_set_from_file(GTK_IMAGE(dialog->icon_entry), dialog->icon_path);
+	gtk_widget_show(dialog->icon_entry);
+
+	gtk_widget_destroy(dialog->icon_filesel);
+	dialog->icon_filesel = NULL;
+	g_free(filename);
+ }
 
 static void
-buddy_icon_preview_change_cb(GtkTreeSelection *sel, AccountPrefsDialog *dialog)
+#if GTK_CHECK_VERSION(2,4,0) /* FILECHOOSER */
+icon_preview_change_cb(GtkFileChooser *widget, AccountPrefsDialog *dialog)
+#else /* FILECHOOSER */
+icon_preview_change_cb(GtkTreeSelection *sel, AccountPrefsDialog *dialog)
+#endif /* FILECHOOSER */
 {
 	GdkPixbuf *pixbuf, *scale;
 	int height, width;
 	char *basename, *markup, *size;
 	struct stat st;
-	const char *filename;
+	char *filename;
 
-	filename = gtk_file_selection_get_filename(
-		GTK_FILE_SELECTION(dialog->buddy_icon_filesel));
+#if GTK_CHECK_VERSION(2,4,0) /* FILECHOOSER */
+	filename = gtk_file_chooser_get_preview_filename(
+					GTK_FILE_CHOOSER(dialog->icon_filesel));
+#else /* FILECHOOSER */
+	filename = g_strdup(gtk_file_selection_get_filename(
+		GTK_FILE_SELECTION(dialog->icon_filesel)));
+#endif /* FILECHOOSER */
 
 	if (!filename || stat(filename, &st))
+	{
+		g_free(filename);
 		return;
+	}
 
 	pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
 	if (!pixbuf) {
-		gtk_image_set_from_pixbuf(GTK_IMAGE(dialog->buddy_icon_preview), NULL);
-		gtk_label_set_markup(GTK_LABEL(dialog->buddy_icon_text), "");
+		gtk_image_set_from_pixbuf(GTK_IMAGE(dialog->icon_preview), NULL);
+		gtk_label_set_markup(GTK_LABEL(dialog->icon_text), "");
+#if GTK_CHECK_VERSION(2,4,0) /* FILECHOOSER */
+		gtk_file_chooser_set_preview_widget_active(
+					GTK_FILE_CHOOSER(dialog->icon_filesel), FALSE);
+#endif /* FILECHOOSER */
+		g_free(filename);
 		return;
 	}
 
@@ -293,78 +335,290 @@ buddy_icon_preview_change_cb(GtkTreeSelection *sel, AccountPrefsDialog *dialog)
 
 	scale = gdk_pixbuf_scale_simple(pixbuf, width * 50 / height,
 									50, GDK_INTERP_BILINEAR);
-	gtk_image_set_from_pixbuf(GTK_IMAGE(dialog->buddy_icon_preview), scale);
-	gtk_label_set_markup(GTK_LABEL(dialog->buddy_icon_text), markup);
+	gtk_image_set_from_pixbuf(GTK_IMAGE(dialog->icon_preview), scale);
+#if GTK_CHECK_VERSION(2,4,0) /* FILECHOOSER */
+	gtk_file_chooser_set_preview_widget_active(
+					GTK_FILE_CHOOSER(dialog->icon_filesel), TRUE);
+#endif /* FILECHOOSER */
+	gtk_label_set_markup(GTK_LABEL(dialog->icon_text), markup);
 
 	g_object_unref(G_OBJECT(pixbuf));
 	g_object_unref(G_OBJECT(scale));
+	g_free(filename);
 	g_free(basename);
 	g_free(size);
 	g_free(markup);
 }
 
+#if !GTK_CHECK_VERSION(2,4,0) /* FILECHOOSER */
 static void
-buddy_icon_select_cb(GtkWidget *button, AccountPrefsDialog *dialog)
+icon_filesel_delete_cb(GtkWidget *w, AccountPrefsDialog *dialog)
 {
+	if (dialog->icon_filesel != NULL)
+		gtk_widget_destroy(dialog->icon_filesel);
+
+	dialog->icon_filesel = NULL;
+}
+#endif /* FILECHOOSER */
+
+static void
+icon_select_cb(GtkWidget *button, AccountPrefsDialog *dialog)
+{
+#if !GTK_CHECK_VERSION(2,4,0) /* FILECHOOSER */
 	GtkWidget *hbox;
 	GtkWidget *tv;
 	GtkTreeSelection *sel;
+#endif /* FILECHOOSER */
 
-	if (dialog->buddy_icon_filesel)
-	{
-		gtk_widget_show(GTK_WIDGET(dialog->buddy_icon_filesel));
-		gdk_window_raise(GDK_WINDOW(dialog->buddy_icon_filesel->window));
+	if (dialog->icon_filesel != NULL) {
+		gtk_window_present(GTK_WINDOW(dialog->icon_filesel));
 		return;
 	}
 
-	dialog->buddy_icon_filesel = gtk_file_selection_new(_("Buddy Icon"));
-	dialog->buddy_icon_preview = gtk_image_new();
-	dialog->buddy_icon_text = gtk_label_new(NULL);
+#if GTK_CHECK_VERSION(2,4,0) /* FILECHOOSER */
+	dialog->icon_filesel = gtk_file_chooser_dialog_new(_("Buddy Icon"),
+									GTK_WINDOW(dialog->window),
+									GTK_FILE_CHOOSER_ACTION_OPEN,
+									GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+									GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+									NULL);
+	gtk_dialog_set_default_response(GTK_DIALOG(dialog->icon_filesel), GTK_RESPONSE_ACCEPT);
 
-	gtk_widget_set_size_request(GTK_WIDGET(dialog->buddy_icon_preview), -1, 50);
+
+	/*  I removed code to set the current path to the current icon in the old file selector so I figure
+	 * it shouldn't be here either.  The reason is because the icon will potentially converted and won't
+	 * be anything near what the user selected last time (which is the advantage to doing it this way. 
+	 * The solution is to create a new pref to specify the last chosen buddy icon.  This would also have the
+	 * advantage (?) of not being account-specific. 
+	if (dialog->icon_path != NULL)
+		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog->icon_filesel),
+			dialog->icon_path);
+	*/
+
+	dialog->icon_preview = gtk_image_new();
+	dialog->icon_text = gtk_label_new(NULL);
+	gtk_widget_set_size_request(GTK_WIDGET(dialog->icon_preview), -1, 50);
+	gtk_file_chooser_set_preview_widget(GTK_FILE_CHOOSER(dialog->icon_filesel), 
+					    GTK_WIDGET(dialog->icon_preview));
+	g_signal_connect(G_OBJECT(dialog->icon_filesel), "update-preview",
+					 G_CALLBACK(icon_preview_change_cb), dialog);
+	g_signal_connect(G_OBJECT(dialog->icon_filesel), "response",
+					 G_CALLBACK(icon_filesel_choose_cb), dialog);
+	icon_preview_change_cb(NULL, dialog);
+#else /* FILECHOOSER */
+	dialog->icon_filesel = gtk_file_selection_new(_("Buddy Icon"));
+	dialog->icon_preview = gtk_image_new();
+	dialog->icon_text = gtk_label_new(NULL);
+
+	gtk_widget_set_size_request(GTK_WIDGET(dialog->icon_preview), -1, 50);
 	hbox = gtk_hbox_new(FALSE, 6);
 	gtk_box_pack_start(
-		GTK_BOX(GTK_FILE_SELECTION(dialog->buddy_icon_filesel)->main_vbox),
+		GTK_BOX(GTK_FILE_SELECTION(dialog->icon_filesel)->main_vbox),
 		hbox, FALSE, FALSE, 0);
-	gtk_box_pack_end(GTK_BOX(hbox), dialog->buddy_icon_preview,
+	gtk_box_pack_end(GTK_BOX(hbox), dialog->icon_preview,
 					 FALSE, FALSE, 0);
-	gtk_box_pack_end(GTK_BOX(hbox), dialog->buddy_icon_text, FALSE, FALSE, 0);
+	gtk_box_pack_end(GTK_BOX(hbox), dialog->icon_text, FALSE, FALSE, 0);
 
-	tv = GTK_FILE_SELECTION(dialog->buddy_icon_filesel)->file_list;
+	tv = GTK_FILE_SELECTION(dialog->icon_filesel)->file_list;
 	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv));
 
 	g_signal_connect(G_OBJECT(sel), "changed",
-					 G_CALLBACK(buddy_icon_preview_change_cb), dialog);
-
-	g_signal_connect(G_OBJECT(dialog->buddy_icon_filesel), "destroy",
-					 G_CALLBACK(buddy_icon_filesel_delete_cb), dialog);
+					 G_CALLBACK(icon_preview_change_cb), dialog);
 	g_signal_connect(
-		G_OBJECT(GTK_FILE_SELECTION(dialog->buddy_icon_filesel)->cancel_button),
+		G_OBJECT(GTK_FILE_SELECTION(dialog->icon_filesel)->ok_button),
 		"clicked",
-		G_CALLBACK(buddy_icon_filesel_delete_cb), dialog);
+		G_CALLBACK(icon_filesel_choose_cb), dialog);
 	g_signal_connect(
-		G_OBJECT(GTK_FILE_SELECTION(dialog->buddy_icon_filesel)->ok_button),
+		G_OBJECT(GTK_FILE_SELECTION(dialog->icon_filesel)->cancel_button),
 		"clicked",
-		G_CALLBACK(buddy_icon_filesel_choose), dialog);
-
-	gtk_widget_show_all(GTK_WIDGET(dialog->buddy_icon_filesel));
-
-	if (dialog->account &&
-		(gaim_account_get_buddy_icon(dialog->account) != NULL))
-	{
-		gtk_file_selection_set_filename(
-			GTK_FILE_SELECTION(dialog->buddy_icon_filesel),
-			gaim_account_get_buddy_icon(dialog->account));
-
-		buddy_icon_preview_change_cb(NULL, dialog);
-	}
+		G_CALLBACK(icon_filesel_delete_cb), dialog);
+	g_signal_connect(G_OBJECT(dialog->icon_filesel), "destroy",
+					 G_CALLBACK(icon_filesel_delete_cb), dialog);
+#endif /* FILECHOOSER */
 
+	gtk_widget_show_all(GTK_WIDGET(dialog->icon_filesel));
 }
 
 static void
-buddy_icon_reset_cb(GtkWidget *button, AccountPrefsDialog *dialog)
+icon_reset_cb(GtkWidget *button, AccountPrefsDialog *dialog)
 {
-	gtk_entry_set_text(GTK_ENTRY(dialog->buddy_icon_entry), "");
+	if (dialog->icon_path)
+		g_free(dialog->icon_path);
+	dialog->icon_path = NULL;
+
+	gtk_widget_hide(dialog->icon_entry);
+}
+
+
+static void
+account_dnd_recv(GtkWidget *widget, GdkDragContext *dc, gint x, gint y,
+		 GtkSelectionData *sd, guint info, guint t, AccountPrefsDialog *dialog)
+{
+	gchar *name = sd->data;
+	
+	if ((sd->length >= 0) && (sd->format == 8)) {
+		/* Well, it looks like the drag event was cool. 
+		 * Let's do something with it */
+		if (!g_ascii_strncasecmp(name, "file://", 7)) {
+			GError *converr = NULL;
+			gchar *tmp, *rtmp;
+			/* It looks like we're dealing with a local file. Let's 
+			 * just untar it in the right place */
+			if(!(tmp = g_filename_from_uri(name, NULL, &converr))) {
+				gaim_debug(GAIM_DEBUG_ERROR, "buddyicon", "%s\n",
+					   (converr ? converr->message :
+					    "g_filename_from_uri error"));
+				return;
+			}
+			if ((rtmp = strchr(tmp, '\r')) || (rtmp = strchr(tmp, '\n')))
+				*rtmp = '\0';
+			if (dialog->icon_path)
+				g_free(dialog->icon_path);
+			dialog->icon_path = convert_buddy_icon(dialog->plugin, tmp);
+			gtk_image_set_from_file(GTK_IMAGE(dialog->icon_entry), dialog->icon_path);
+			gtk_widget_show(dialog->icon_entry);
+			g_free(tmp);
+		} 
+		gtk_drag_finish(dc, TRUE, FALSE, t);
+	}
+	gtk_drag_finish(dc, FALSE, FALSE, t);
+}
+
+
+#if GTK_CHECK_VERSION(2,2,0)
+static gboolean
+str_array_match(char **a, char **b)
+{
+	int i, j;
+
+	if (!a || !b)
+		return FALSE;
+	for (i = 0; a[i] != NULL; i++)
+		for (j = 0; b[j] != NULL; j++)
+			if (!g_ascii_strcasecmp(a[i], b[j]))
+				return TRUE;
+	return FALSE;
+}
+#endif
+
+static char*
+convert_buddy_icon(GaimPlugin *plugin, const char *path)
+{
+#if GTK_CHECK_VERSION(2,2,0)
+	int width, height;
+	char **pixbuf_formats = NULL;
+	GdkPixbufFormat *format;
+	GdkPixbuf *pixbuf;
+	GaimPluginProtocolInfo *prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(plugin);
+	char **prpl_formats =  g_strsplit (prpl_info->icon_spec.format,",",0);
+#if !GTK_CHECK_VERSION(2,4,0)
+	GdkPixbufLoader *loader;
+	FILE *file;
+	struct stat st;
+	void *data = NULL;
+#endif
+
+#if GTK_CHECK_VERSION(2,4,0)
+	format = gdk_pixbuf_get_file_info (path, &width, &height);
+#else
+	loader = gdk_pixbuf_loader_new();
+	if (!stat(path, &st) && (file = fopen(path, "rb")) != NULL) {
+		data = g_malloc(st.st_size);
+		fread(data, 1, st.st_size, file);
+		fclose(file);
+		gdk_pixbuf_loader_write(loader, data, st.st_size, NULL);
+		g_free(data);
+	}
+	pixbuf = gdk_pixbuf_loader_get_pixbuf(loader);
+	width = gdk_pixbuf_get_width(pixbuf);
+	height = gdk_pixbuf_get_height(pixbuf);
+	format = gdk_pixbuf_loader_get_format(loader);
+	gdk_pixbuf_loader_close(loader, NULL);
+	g_object_unref(G_OBJECT(loader));
+#endif
+	pixbuf_formats =  gdk_pixbuf_format_get_extensions(format);
+
+	if (str_array_match(pixbuf_formats, prpl_formats) &&                                 /* This is an acceptable format AND */
+		 (!(prpl_info->icon_spec.scale_rules & GAIM_ICON_SCALE_SEND) ||                   /* The prpl doesn't scale before it sends OR */
+		  (prpl_info->icon_spec.min_width <= width &&
+		   prpl_info->icon_spec.max_width >= width &&
+		   prpl_info->icon_spec.min_height <= height &&
+		   prpl_info->icon_spec.max_height >= height))) {                                  /* The icon is the correct size */
+		g_strfreev(prpl_formats);
+		g_strfreev(pixbuf_formats);
+#endif
+		return g_strdup(path);
+#if GTK_CHECK_VERSION(2,2,0)
+	} else {
+		int i;
+		GError *error = NULL;
+		GdkPixbuf *scale;
+		char *random   = g_strdup_printf("%x", g_random_int());
+		const char *dirname = gaim_buddy_icons_get_cache_dir();
+		char *filename = g_build_filename(dirname, random, NULL);
+		pixbuf = gdk_pixbuf_new_from_file(path, &error);
+		g_strfreev(pixbuf_formats);
+		if (!error && prpl_info->icon_spec.scale_rules & GAIM_ICON_SCALE_SEND) {
+			int new_width = gdk_pixbuf_get_width(pixbuf);
+			int new_height = gdk_pixbuf_get_height(pixbuf);
+
+			if(new_width > prpl_info->icon_spec.max_width)
+				new_width = prpl_info->icon_spec.max_width;
+			else if(new_width < prpl_info->icon_spec.min_width)
+				new_width = prpl_info->icon_spec.min_width;
+			if(new_height > prpl_info->icon_spec.max_height)
+				new_height = prpl_info->icon_spec.max_height;
+			else if(new_height < prpl_info->icon_spec.min_height)
+				new_height = prpl_info->icon_spec.min_height;
+
+			scale = gdk_pixbuf_scale_simple (pixbuf, new_width, new_height,
+					GDK_INTERP_HYPER);
+			g_object_unref(G_OBJECT(pixbuf));
+			pixbuf = scale;
+		}
+		if (error) {
+			g_free(filename);
+			g_free(random);
+			gaim_debug_error("buddyicon", "Could not open icon for conversion: %s\n", error->message);
+			g_error_free(error);
+			g_strfreev(prpl_formats);
+			return NULL;
+		}
+
+		if (!g_file_test(dirname, G_FILE_TEST_IS_DIR)) {
+			gaim_debug_info("buddyicon", "Creating icon cache directory.\n");
+
+			if (mkdir(dirname, S_IRUSR | S_IWUSR | S_IXUSR) < 0) {
+				gaim_debug_error("buddyicon",
+								 "Unable to create directory %s: %s\n",
+								 dirname, strerror(errno));
+				g_strfreev(prpl_formats);
+				return NULL;
+			}
+		}
+
+		for (i = 0; prpl_formats[i]; i++) {
+			gaim_debug_info("buddyicon", "Converting buddy icon to %s as %s\n", prpl_formats[i], filename);
+			/* The gdk-pixbuf documentation is wrong. gdk_pixbuf_save returns TRUE if it was successful,
+			 * FALSE if an error was set. */
+			if (gdk_pixbuf_save (pixbuf, filename, prpl_formats[i], &error, NULL) == TRUE)
+					break;
+		}
+		g_strfreev(prpl_formats);
+		if (!error) {
+			g_free(random);
+			g_object_unref(G_OBJECT(pixbuf));
+			return filename;
+		} else {
+			gaim_debug_error("buddyicon", "Could not convert icon to usable format: %s\n", error->message);
+			g_error_free(error);
+		}
+		g_free(filename);
+		g_free(random);
+		g_object_unref(G_OBJECT(pixbuf));
+	}
+	return NULL;
+#endif
 }
 
 static void
@@ -531,7 +785,9 @@ add_user_options(AccountPrefsDialog *dialog, GtkWidget *parent)
 {
 	GtkWidget *frame;
 	GtkWidget *vbox;
+	GtkWidget *vbox2;
 	GtkWidget *hbox;
+	GtkWidget *hbox2;
 	GtkWidget *button;
 	GtkWidget *label;
 
@@ -557,52 +813,63 @@ add_user_options(AccountPrefsDialog *dialog, GtkWidget *parent)
 	gtk_widget_show(dialog->new_mail_check);
 
 	/* Buddy icon */
-	dialog->buddy_icon_hbox = hbox = gtk_hbox_new(FALSE, 6);
+	dialog->icon_hbox = hbox = gtk_hbox_new(FALSE, 6);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 	gtk_widget_show(hbox);
 
-	label = gtk_label_new(_("Buddy icon file:"));
+	label = gtk_label_new(_("Buddy icon:"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	gtk_widget_show(label);
 
-	dialog->buddy_icon_entry = gtk_entry_new();
-	gtk_editable_set_editable(GTK_EDITABLE(dialog->buddy_icon_entry), FALSE);
-	gtk_box_pack_start(GTK_BOX(hbox), dialog->buddy_icon_entry, TRUE, TRUE, 0);
-	gtk_widget_show(dialog->buddy_icon_entry);
-	gaim_set_accessible_label (dialog->buddy_icon_entry, label);
+	dialog->icon_entry = gtk_image_new();
+	gtk_box_pack_start(GTK_BOX(hbox), dialog->icon_entry,
+					   FALSE, FALSE, 0);
+	gtk_widget_show(dialog->icon_entry);
+	gaim_set_accessible_label (dialog->icon_entry, label);
+	dialog->icon_path = NULL;
 
-	button = gtk_button_new_with_mnemonic(_("_Browse"));
-	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+	vbox2 = gtk_vbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);
+	gtk_widget_show(vbox2);
+
+	hbox2 = gtk_hbox_new(FALSE, 6);
+	gtk_box_pack_start(GTK_BOX(vbox2), hbox2, FALSE, FALSE, 0);
+	gtk_widget_show(hbox2);
+
+	button = gtk_button_new_from_stock(GTK_STOCK_OPEN);
+	gtk_box_pack_start(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked",
-					 G_CALLBACK(buddy_icon_select_cb), dialog);
+					 G_CALLBACK(icon_select_cb), dialog);
 	gtk_widget_show(button);
 
-	button = gtk_button_new_with_mnemonic(_("_Reset"));
+	button = gtk_button_new_from_stock(GTK_STOCK_REMOVE);
 	g_signal_connect(G_OBJECT(button), "clicked",
-			 G_CALLBACK(buddy_icon_reset_cb), dialog);
-	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+			 G_CALLBACK(icon_reset_cb), dialog);
+	gtk_box_pack_start(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
 	if (dialog->prpl_info != NULL) {
 		if (!(dialog->prpl_info->options & OPT_PROTO_MAIL_CHECK))
 			gtk_widget_hide(dialog->new_mail_check);
 
-		if (!(dialog->prpl_info->options & OPT_PROTO_BUDDY_ICON))
-			gtk_widget_hide(dialog->buddy_icon_hbox);
+		if (!(dialog->prpl_info->icon_spec.format != NULL))
+			gtk_widget_hide(dialog->icon_hbox);
 	}
 
 	if (dialog->account != NULL) {
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->new_mail_check),
 				gaim_account_get_check_mail(dialog->account));
 
-		if (gaim_account_get_buddy_icon(dialog->account) != NULL)
-			gtk_entry_set_text(GTK_ENTRY(dialog->buddy_icon_entry),
-					gaim_account_get_buddy_icon(dialog->account));
+		if (gaim_account_get_buddy_icon(dialog->account) != NULL) {
+			dialog->icon_path = g_strdup(gaim_account_get_buddy_icon(dialog->account));
+			gtk_image_set_from_file(GTK_IMAGE(dialog->icon_entry),dialog->icon_path);
+		}
 	}
 
 	if (!dialog->prpl_info ||
 			(!(dialog->prpl_info->options & OPT_PROTO_MAIL_CHECK) &&
-		!(dialog->prpl_info->options & OPT_PROTO_BUDDY_ICON))) {
+			 (dialog->prpl_info->icon_spec.format ==  NULL))) {
 
 		/* Nothing to see :( aww. */
 		gtk_widget_hide(dialog->user_frame);
@@ -972,7 +1239,7 @@ add_proxy_options(AccountPrefsDialog *dialog, GtkWidget *parent)
 		dialog->new_proxy_type = type;
 #if GTK_CHECK_VERSION(2,4,0)
 		gtk_combo_box_set_active(GTK_COMBO_BOX(dialog->proxy_dropdown),
-				type - 1);
+				type + 1);
 #else
 		gtk_option_menu_set_history(GTK_OPTION_MENU(dialog->proxy_dropdown),
 				(int)type + 1);
@@ -1039,8 +1306,11 @@ account_win_destroy_cb(GtkWidget *w, GdkEvent *event,
 	if (dialog->protocol_id != NULL)
 		g_free(dialog->protocol_id);
 
-	if (dialog->buddy_icon_filesel)
-		gtk_widget_destroy(dialog->buddy_icon_filesel);
+	if (dialog->icon_path != NULL)
+		g_free(dialog->icon_path);
+
+	if (dialog->icon_filesel)
+		gtk_widget_destroy(dialog->icon_filesel);
 
 	g_free(dialog);
 }
@@ -1077,9 +1347,6 @@ ok_account_prefs_cb(GtkWidget *w, AccountPrefsDialog *dialog)
 		gaim_account_set_protocol_id(dialog->account, dialog->protocol_id);
 	}
 
-	/* Clear the existing settings. */
-	gaim_account_clear_settings(dialog->account);
-
 	/* Alias */
 	value = gtk_entry_get_text(GTK_ENTRY(dialog->alias_entry));
 
@@ -1089,15 +1356,9 @@ ok_account_prefs_cb(GtkWidget *w, AccountPrefsDialog *dialog)
 		gaim_account_set_alias(dialog->account, NULL);
 
 	/* Buddy Icon */
-	value = gtk_entry_get_text(GTK_ENTRY(dialog->buddy_icon_entry));
-
-	if (dialog->prpl_info &&
-			(dialog->prpl_info->options & OPT_PROTO_BUDDY_ICON) &&
-			*value != '\0')
-		gaim_account_set_buddy_icon(dialog->account, value);
-	else
-		gaim_account_set_buddy_icon(dialog->account, NULL);
-
+	value = dialog->icon_path;
+	gaim_account_set_buddy_icon(dialog->account, value);
+	
 	/* Remember Password */
 	gaim_account_set_remember_password(dialog->account,
 			gtk_toggle_button_get_active(
@@ -1267,6 +1528,8 @@ ok_account_prefs_cb(GtkWidget *w, AccountPrefsDialog *dialog)
 
 	account_win_destroy_cb(NULL, NULL, dialog);
 
+	gaim_signal_emit(gaim_gtk_account_get_handle(), "account-modified", ret);
+
 	return ret;
 }
 
@@ -1278,6 +1541,13 @@ register_account_prefs_cb(GtkWidget *w, AccountPrefsDialog *dialog)
 	gaim_account_register(account);
 }
 
+
+static const GtkTargetEntry dnd_targets[] = {
+       {"text/plain", 0, 0},
+	{"text/uri-list", 0, 1},
+	{"STRING", 0, 2}
+};
+
 void
 gaim_gtk_account_dialog_show(GaimGtkAccountDialogType type,
 							 GaimAccount *account)
@@ -1416,6 +1686,17 @@ gaim_gtk_account_dialog_show(GaimGtkAccountDialogType type,
 
 	dialog->ok_button = button;
 
+	/* Set up DND */
+	gtk_drag_dest_set(dialog->window,
+			  GTK_DEST_DEFAULT_MOTION |
+			  GTK_DEST_DEFAULT_DROP,
+			  dnd_targets,
+			  sizeof(dnd_targets) / sizeof(GtkTargetEntry),
+			  GDK_ACTION_COPY);
+
+	g_signal_connect(G_OBJECT(dialog->window), "drag_data_received",
+			 G_CALLBACK(account_dnd_recv), dialog);
+
 	g_signal_connect(G_OBJECT(button), "clicked",
 					 G_CALLBACK(ok_account_prefs_cb), dialog);
 
@@ -1739,7 +2020,8 @@ ask_delete_account_sel(GtkTreeModel *model, GtkTreePath *path,
 		buf = g_strdup_printf(_("Are you sure you want to delete %s?"),
 							  gaim_account_get_username(account));
 
-		gaim_request_action(NULL, NULL, buf, NULL, 1, account, 2,
+		gaim_request_close_with_handle(account);
+		gaim_request_action(account, NULL, buf, NULL, 0, account, 2,
 							_("Delete"), delete_account_cb,
 							_("Cancel"), NULL);
 		g_free(buf);
@@ -1856,6 +2138,7 @@ add_columns(GtkWidget *treeview, AccountsWindow *dialog)
 	column = gtk_tree_view_column_new();
 	gtk_tree_view_column_set_title(column, _("Screen Name"));
 	gtk_tree_view_insert_column(GTK_TREE_VIEW(treeview), column, -1);
+	gtk_tree_view_column_set_resizable(column, TRUE);
 
 	/* Icon */
 	renderer = gtk_cell_renderer_pixbuf_new();
@@ -1881,6 +2164,8 @@ add_columns(GtkWidget *treeview, AccountsWindow *dialog)
 						    renderer,
 						    "active", COLUMN_ONLINE,
 						    NULL);
+	column = gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), 1);
+	gtk_tree_view_column_set_resizable(column, TRUE);
 
 	/* Auto-login? */
 	renderer = gtk_cell_renderer_toggle_new();
@@ -1892,11 +2177,13 @@ add_columns(GtkWidget *treeview, AccountsWindow *dialog)
 			renderer, "active", COLUMN_AUTOLOGIN, NULL);
 
 	gtk_tree_view_insert_column(GTK_TREE_VIEW(treeview), column, -1);
+	gtk_tree_view_column_set_resizable(column, TRUE);
 
 	/* Protocol name */
 	column = gtk_tree_view_column_new();
 	gtk_tree_view_column_set_title(column, _("Protocol"));
 	gtk_tree_view_insert_column(GTK_TREE_VIEW(treeview), column, -1);
+	gtk_tree_view_column_set_resizable(column, TRUE);
 
 	renderer = gtk_cell_renderer_text_new();
 	gtk_tree_view_column_pack_start(column, renderer, TRUE);
@@ -1927,7 +2214,7 @@ set_account(GtkListStore *store, GtkTreeIter *iter, GaimAccount *account)
 			COLUMN_SCREENNAME, gaim_account_get_username(account),
 			COLUMN_ONLINE, gaim_account_is_connected(account),
 			COLUMN_AUTOLOGIN, gaim_account_get_auto_login(account, GAIM_GTK_UI),
-			COLUMN_PROTOCOL, proto_name(gaim_account_get_protocol_id(account)),
+			COLUMN_PROTOCOL, gaim_account_get_protocol_name(account),
 			COLUMN_DATA, account,
 			-1);
 
@@ -2052,7 +2339,7 @@ gaim_gtk_accounts_window_show(void)
 	int width, height;
 
 	if (accounts_window != NULL) {
-		gdk_window_raise(accounts_window->window->window);
+		gtk_window_present(GTK_WINDOW(accounts_window->window));
 		return;
 	}
 
@@ -2230,7 +2517,7 @@ gaim_gtk_accounts_notify_added(GaimAccount *account, const char *remote_user,
 	else
 	{
 		gaim_request_action(NULL, NULL, _("Add buddy to your list?"),
-							buffer, 0, data, 2,
+							buffer, GAIM_DEFAULT_ACTION_NONE, data, 2,
 							_("Add"),    G_CALLBACK(add_user_cb),
 							_("Cancel"), G_CALLBACK(free_add_user_data));
 	}
@@ -2248,3 +2535,23 @@ gaim_gtk_accounts_get_ui_ops(void)
 {
 	return &ui_ops;
 }
+
+void *
+gaim_gtk_account_get_handle() {
+	static int handle;
+
+	return &handle;
+}
+
+void
+gaim_gtk_account_init(void) {
+	gaim_signal_register(gaim_gtk_account_get_handle(), "account-modified",
+						 gaim_marshal_VOID__POINTER, NULL, 1,
+						 gaim_value_new(GAIM_TYPE_SUBTYPE,
+										GAIM_SUBTYPE_ACCOUNT));
+}
+
+void
+gaim_gtk_account_uninit(void) {
+	gaim_signals_unregister_by_instance(gaim_gtk_account_get_handle());
+}
