@@ -22,13 +22,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-#include "gtkinternal.h"
+#include "internal.h"
+#include "gtkgaim.h"
 
 #include "debug.h"
 #include "notify.h"
 #include "ft.h"
 #include "prpl.h"
-#include "request.h"
 #include "util.h"
 
 #include "gaim-disclosure.h"
@@ -58,8 +58,11 @@ struct _GaimGtkXferDialog
 
 	GtkWidget *table;
 
-	GtkWidget *user_desc_label;
-	GtkWidget *user_label;
+	GtkWidget *local_user_desc_label;
+	GtkWidget *local_user_label;
+	GtkWidget *remote_user_desc_label;
+	GtkWidget *remote_user_label;
+	GtkWidget *protocol_label;
 	GtkWidget *filename_label;
 	GtkWidget *status_label;
 	GtkWidget *speed_label;
@@ -79,7 +82,6 @@ struct _GaimGtkXferDialog
 
 typedef struct
 {
-	GtkWidget *filesel;
 	GtkTreeIter iter;
 	time_t start_time;
 	time_t end_time;
@@ -103,11 +105,6 @@ enum
 };
 
 
-/**************************************************************************
- * Prototype(s)
- **************************************************************************/
-static int choose_file(GaimXfer *xfer);
-
 /**************************************************************************
  * Utility Functions
  **************************************************************************/
@@ -218,14 +215,24 @@ update_detailed_info(GaimGtkXferDialog *dialog, GaimXfer *xfer)
 		g_object_unref(pixbuf);
 	}
 
-	if (gaim_xfer_get_type(xfer) == GAIM_XFER_RECEIVE)
-		gtk_label_set_markup(GTK_LABEL(dialog->user_desc_label),
+	if (gaim_xfer_get_type(xfer) == GAIM_XFER_RECEIVE) {
+		gtk_label_set_markup(GTK_LABEL(dialog->local_user_desc_label),
+							 _("<b>Receiving As:</b>"));
+		gtk_label_set_markup(GTK_LABEL(dialog->remote_user_desc_label),
 							 _("<b>Receiving From:</b>"));
-	else
-		gtk_label_set_markup(GTK_LABEL(dialog->user_desc_label),
+	}
+	else {
+		gtk_label_set_markup(GTK_LABEL(dialog->remote_user_desc_label),
 							 _("<b>Sending To:</b>"));
+		gtk_label_set_markup(GTK_LABEL(dialog->local_user_desc_label),
+							 _("<b>Sending As:</b>"));
+	}
 
-	gtk_label_set_text(GTK_LABEL(dialog->user_label), xfer->who);
+	gtk_label_set_text(GTK_LABEL(dialog->local_user_label), 
+								 gaim_account_get_username(xfer->account));
+	gtk_label_set_text(GTK_LABEL(dialog->remote_user_label), xfer->who);
+	gtk_label_set_text(GTK_LABEL(dialog->protocol_label), 
+								 gaim_account_get_protocol_name(xfer->account));
 
 	if (gaim_xfer_get_type(xfer) == GAIM_XFER_RECEIVE) {
 		gtk_label_set_text(GTK_LABEL(dialog->filename_label),
@@ -492,7 +499,7 @@ setup_tree(GaimGtkXferDialog *dialog)
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 
 	/* Progress bar column */
-	renderer = gtk_cell_renderer_progress_new();
+	renderer = gaim_gtk_cell_renderer_progress_new();
 	column = gtk_tree_view_column_new_with_attributes(_("Progress"), renderer,
 				"percentage", COLUMN_PROGRESS, NULL);
 	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
@@ -543,7 +550,9 @@ make_info_table(GaimGtkXferDialog *dialog)
 
 	} labels[] =
 	{
-		{ &dialog->user_desc_label, &dialog->user_label, NULL },
+		{ &dialog->local_user_desc_label, &dialog->local_user_label, NULL },
+		{ &dialog->remote_user_desc_label, &dialog->remote_user_label, NULL },
+		{ &label, &dialog->protocol_label,		 _("Protocol:") },
 		{ &label, &dialog->filename_label,       _("Filename:") },
 		{ &label, &dialog->status_label,         _("Status:") },
 		{ &label, &dialog->speed_label,          _("Speed:") },
@@ -552,7 +561,7 @@ make_info_table(GaimGtkXferDialog *dialog)
 	};
 
 	/* Setup the initial table */
-	dialog->table = table = gtk_table_new(8, 2, FALSE);
+	dialog->table = table = gtk_table_new(10, 2, FALSE);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 6);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 6);
 
@@ -581,12 +590,12 @@ make_info_table(GaimGtkXferDialog *dialog)
 
 	/* Setup the progress bar */
 	dialog->progress = gtk_progress_bar_new();
-	gtk_table_attach(GTK_TABLE(table), dialog->progress, 0, 2, 6, 7,
+	gtk_table_attach(GTK_TABLE(table), dialog->progress, 0, 2, 8, 9,
 					 GTK_FILL, GTK_FILL, 0, 0);
 	gtk_widget_show(dialog->progress);
 
 	sep = gtk_hseparator_new();
-	gtk_table_attach(GTK_TABLE(table), sep, 0, 2, 7, 8,
+	gtk_table_attach(GTK_TABLE(table), sep, 0, 2, 9, 10,
 					 GTK_FILL, GTK_FILL, 0, 0);
 	gtk_widget_show(sep);
 
@@ -617,13 +626,8 @@ gaim_gtkxfer_dialog_new(void)
 	dialog->window = window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_role(GTK_WINDOW(window), "file transfer");
 	gtk_window_set_title(GTK_WINDOW(window), _("File Transfers"));
-#if 0
-	gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, TRUE);
-	gtk_window_set_default_size(GTK_WINDOW(window), 390, 400);
-#endif
 	gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
 	gtk_container_set_border_width(GTK_CONTAINER(window), 12);
-	gtk_widget_realize(window);
 
 	g_signal_connect(G_OBJECT(window), "delete_event",
 					 G_CALLBACK(delete_win_cb), dialog);
@@ -841,7 +845,7 @@ gaim_gtkxfer_dialog_add_xfer(GaimGtkXferDialog *dialog, GaimXfer *xfer)
 					                     ? gaim_xfer_get_filename(xfer)
 							     : lfilename,
 					   COLUMN_SIZE, size_str,
-					   COLUMN_REMAINING, remaining_str,
+					   COLUMN_REMAINING, _("Waiting for transfer to begin"),
 					   COLUMN_DATA, xfer,
 					   -1);
 	g_free(lfilename);
@@ -975,7 +979,7 @@ gaim_gtkxfer_dialog_update_xfer(GaimGtkXferDialog *dialog,
 						   -1);
 
 		g_object_unref(pixbuf);
-	}
+	} 
 
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(xfer_dialog->tree));
 
@@ -1015,255 +1019,6 @@ gaim_gtkxfer_destroy(GaimXfer *xfer)
 	}
 }
 
-static gboolean
-choose_file_close_cb(GtkWidget *widget, GdkEvent *event, gpointer user_data)
-{
-	GaimXfer *xfer = (GaimXfer *)user_data;
-	GaimGtkXferUiData *data;
-
-	data = GAIM_GTKXFER(xfer);
-	data->filesel = NULL;
-	gaim_xfer_request_denied(xfer);
-
-	return FALSE;
-}
-
-static void
-choose_file_cancel_cb(GtkButton *button, gpointer user_data)
-{
-	GaimXfer *xfer = (GaimXfer *)user_data;
-	GaimGtkXferUiData *data;
-
-	data = GAIM_GTKXFER(xfer);
-	gtk_widget_destroy(data->filesel);
-	data->filesel = NULL;
-	gaim_xfer_request_denied(xfer);
-}
-
-static int
-do_overwrite_cb(GaimXfer *xfer)
-{
-	GaimGtkXferUiData *data;
-
-	data = GAIM_GTKXFER(xfer);
-
-	gaim_xfer_request_accepted(xfer, data->name);
-
-	g_free(data->name);
-	data->name = NULL;
-
-	gaim_xfer_unref(xfer);
-	return 0;
-}
-
-static int
-dont_overwrite_cb(GaimXfer *xfer)
-{
-	GaimGtkXferUiData *data;
-
-	data = GAIM_GTKXFER(xfer);
-
-	g_free(data->name);
-	data->name = NULL;
-
-	choose_file(xfer);
-
-	return 0;
-}
-
-static void
-choose_file_ok_cb(GtkButton *button, gpointer user_data)
-{
-	GaimXfer *xfer;
-	GaimGtkXferUiData *data;
-	struct stat st;
-	const char *name;
-
-	xfer = (GaimXfer *)user_data;
-	data = GAIM_GTKXFER(xfer);
-
-	name = gtk_file_selection_get_filename(GTK_FILE_SELECTION(data->filesel));
-
-	if (gaim_gtk_check_if_dir(name, GTK_FILE_SELECTION(data->filesel))) {
-		gaim_xfer_unref(xfer);
-		return;
-	}
-
-	if (stat(name, &st) != 0) {
-		/* File not found. */
-		if (gaim_xfer_get_type(xfer) == GAIM_XFER_RECEIVE) {
-			gaim_xfer_request_accepted(xfer, g_strdup(name));
-		}
-		else {
-			gaim_notify_error(NULL, NULL,
-							  _("That file does not exist."), NULL);
-
-			gaim_xfer_request_denied(xfer);
-		}
-	}
-	else if ((gaim_xfer_get_type(xfer) == GAIM_XFER_SEND) &&
-			 (st.st_size == 0)) {
-
-		gaim_notify_error(NULL, NULL,
-						  _("Cannot send a file of 0 bytes."), NULL);
-
-		gaim_xfer_request_denied(xfer);
-	}
-	else {
-		if (S_ISDIR(st.st_mode)) {
-			/* XXX */
-			gaim_xfer_request_denied(xfer);
-		}
-		else if (gaim_xfer_get_type(xfer) == GAIM_XFER_RECEIVE) {
-			data->name = g_strdup(name);
-			gaim_xfer_ref(xfer);
-
-			gaim_request_yes_no(NULL, NULL,
-								_("That file already exists."),
-								_("Would you like to overwrite it?"),
-								1, xfer,
-								G_CALLBACK(do_overwrite_cb),
-								G_CALLBACK(dont_overwrite_cb));
-		}
-		else {
-			gaim_xfer_request_accepted(xfer, name);
-		}
-	}
-
-	gtk_widget_destroy(data->filesel);
-	data->filesel = NULL;
-	gaim_xfer_unref(xfer);
-}
-
-static int
-choose_file(GaimXfer *xfer)
-{
-	char *cur_dir, *init_str;
-	GaimGtkXferUiData *data;
-
-	data = GAIM_GTKXFER(xfer);
-	cur_dir = g_get_current_dir();
-
-	if (gaim_xfer_get_type(xfer) == GAIM_XFER_SEND)
-		data->filesel = gtk_file_selection_new(_("Open..."));
-	else
-		data->filesel = gtk_file_selection_new(_("Save As..."));
-
-	if (gaim_xfer_get_filename(xfer) == NULL)
-		init_str = g_strdup_printf("%s" G_DIR_SEPARATOR_S, cur_dir);
-	else
-		init_str = g_build_filename(cur_dir, gaim_xfer_get_filename(xfer),
-									NULL);
-
-	g_free(cur_dir);
-
-	gtk_file_selection_set_filename(GTK_FILE_SELECTION(data->filesel),
-									init_str);
-
-	g_free(init_str);
-
-	g_signal_connect(G_OBJECT(data->filesel), "delete_event",
-					 G_CALLBACK(choose_file_close_cb), xfer);
-	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(data->filesel)->cancel_button),
-					 "clicked",
-					 G_CALLBACK(choose_file_cancel_cb), xfer);
-	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(data->filesel)->ok_button),
-					 "clicked",
-					 G_CALLBACK(choose_file_ok_cb), xfer);
-
-	gtk_widget_show(data->filesel);
-
-	return 0;
-}
-
-static int
-cancel_recv_cb(GaimXfer *xfer)
-{
-	gaim_xfer_request_denied(xfer);
-	gaim_xfer_unref(xfer);
-
-	return 0;
-}
-
-static void
-gaim_gtkxfer_ask_recv(GaimXfer *xfer)
-{
-	char *buf, *size_buf;
-	size_t size;
-
-	/* If we have already accepted the request, ask the destination file
-	   name directly */
-	if (gaim_xfer_get_status(xfer) != GAIM_XFER_STATUS_ACCEPTED) {
-		size = gaim_xfer_get_size(xfer);
-		size_buf = gaim_str_size_to_units(size);
-
-		buf = g_strdup_printf(_("%s wants to send you %s (%s)"),
-				      xfer->who, gaim_xfer_get_filename(xfer),
-				      size_buf);
-		g_free(size_buf);
-
-		gaim_request_accept_cancel(NULL, NULL, buf, NULL, 0, xfer,
-					   G_CALLBACK(choose_file),
-					   G_CALLBACK(cancel_recv_cb));
-		g_free(buf);
-	} else
-		choose_file(xfer);
-}
-
-static int
-ask_accept_ok(GaimXfer *xfer)
-{
-	gaim_xfer_request_accepted(xfer, NULL);
-	return 0;
-}
-
-static int
-ask_accept_cancel(GaimXfer *xfer)
-{
-	gaim_xfer_request_denied(xfer);
-	gaim_xfer_unref(xfer);
-	return 0;
-}
-
-static void
-gaim_gtkxfer_ask_accept(GaimXfer *xfer)
-{
-	char *buf, *buf2 = NULL;
-
-	buf = g_strdup_printf(_("Accept file transfer request from %s?"),
-			      xfer->who);
-	if (gaim_xfer_get_remote_ip(xfer) &&
-	    gaim_xfer_get_remote_port(xfer))
-		buf2 = g_strdup_printf(_("A file is available for download from:\n"
-					 "Remote host: %s\nRemote port: %d"),
-				       gaim_xfer_get_remote_ip(xfer),
-				       gaim_xfer_get_remote_port(xfer));
-	gaim_request_accept_cancel(NULL, NULL, buf, buf2, 0, xfer,
-				   G_CALLBACK(ask_accept_ok),
-				   G_CALLBACK(ask_accept_cancel));
-	g_free(buf);
-	g_free(buf2);
-}
-
-static void
-gaim_gtkxfer_request_file(GaimXfer *xfer)
-{
-	gaim_xfer_ref(xfer);
-	if (gaim_xfer_get_type(xfer) == GAIM_XFER_RECEIVE) {
-		if (gaim_xfer_get_filename(xfer) ||
-		    gaim_xfer_get_status(xfer) == GAIM_XFER_STATUS_ACCEPTED)
-			gaim_gtkxfer_ask_recv(xfer);
-		else
-			gaim_gtkxfer_ask_accept(xfer);
-	} else
-		choose_file(xfer);
-}
-
-static void
-gaim_gtkxfer_ask_cancel(GaimXfer *xfer)
-{
-}
-
 static void
 gaim_gtkxfer_add_xfer(GaimXfer *xfer)
 {
@@ -1297,8 +1052,6 @@ static GaimXferUiOps ops =
 {
 	gaim_gtkxfer_new_xfer,
 	gaim_gtkxfer_destroy,
-	gaim_gtkxfer_request_file,
-	gaim_gtkxfer_ask_cancel,
 	gaim_gtkxfer_add_xfer,
 	gaim_gtkxfer_update_progress,
 	gaim_gtkxfer_cancel_local,
