@@ -20,9 +20,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-
-#ifndef __GTK_IMHTML_H
-#define __GTK_IMHTML_H
+#ifndef _GAIM_GTKIMHTML_H_
+#define _GAIM_GTKIMHTML_H_
 
 #include <gdk/gdk.h>
 #include <gtk/gtktextview.h>
@@ -68,6 +67,7 @@ typedef enum {
 	GTK_IMHTML_IMAGE =     1 << 9,
 	GTK_IMHTML_SMILEY =    1 << 10,
 	GTK_IMHTML_LINKDESC =  1 << 11,
+	GTK_IMHTML_STRIKE =    1 << 12,
 	GTK_IMHTML_ALL =      -1
 } GtkIMHtmlButtons;
 
@@ -106,6 +106,7 @@ struct _GtkIMHtml {
 		gboolean bold:1;
 		gboolean italic:1;
 		gboolean underline:1;
+		gboolean strike:1;
 		gchar *forecolor;
 		gchar *backcolor;
 		gchar *fontface;
@@ -170,6 +171,7 @@ struct _GtkIMHtmlImage {
 	int width;
 	int height;
 	int id;
+	GtkWidget *filesel;
 };
 
 struct _GtkIMHtmlHr {
@@ -189,6 +191,28 @@ typedef enum {
 	GTK_IMHTML_USE_POINTSIZE = 1 << 8
 } GtkIMHtmlOptions;
 
+enum {
+	GTK_IMHTML_DRAG_URL = 0,
+	GTK_IMHTML_DRAG_HTML,
+	GTK_IMHTML_DRAG_UTF8_STRING,
+	GTK_IMHTML_DRAG_COMPOUND_TEXT,
+	GTK_IMHTML_DRAG_STRING,
+	GTK_IMHTML_DRAG_TEXT,
+	GTK_IMHTML_DRAG_NUM
+};
+
+#define GTK_IMHTML_DND_TARGETS	\
+	{ "text/uri-list", 0, GTK_IMHTML_DRAG_URL }, \
+	{ "_NETSCAPE_URL", 0, GTK_IMHTML_DRAG_URL }, \
+	{ "text/html", 0, GTK_IMHTML_DRAG_HTML }, \
+	{ "x-url/ftp", 0, GTK_IMHTML_DRAG_URL }, \
+	{ "x-url/http", 0, GTK_IMHTML_DRAG_URL }, \
+	{ "UTF8_STRING", 0, GTK_IMHTML_DRAG_UTF8_STRING }, \
+	{ "COMPOUND_TEXT", 0, GTK_IMHTML_DRAG_COMPOUND_TEXT }, \
+	{ "STRING", 0, GTK_IMHTML_DRAG_STRING }, \
+	{ "text/plain", 0, GTK_IMHTML_DRAG_TEXT }, \
+	{ "TEXT", 0, GTK_IMHTML_DRAG_TEXT }
+
 typedef gpointer    (*GtkIMHtmlGetImageFunc)        (int id);
 typedef gpointer    (*GtkIMHtmlGetImageDataFunc)    (gpointer i);
 typedef size_t      (*GtkIMHtmlGetImageSizeFunc)    (gpointer i);
@@ -289,8 +313,8 @@ void gtk_imhtml_set_protocol_name(GtkIMHtml *imhtml, const gchar *protocol_name)
  * @param text    The formatted text to append.
  * @param options A GtkIMHtmlOptions object indicating insert behavior.
  */
-#define gtk_imhtml_append_text(x, y, z) \
- gtk_imhtml_append_text_with_images(x, y, z, NULL)
+#define gtk_imhtml_append_text(imhtml, text, options) \
+ gtk_imhtml_append_text_with_images(imhtml, text, options, NULL)
 
 /**
  * Appends HTML formatted text to a GTK IM/HTML.
@@ -298,7 +322,7 @@ void gtk_imhtml_set_protocol_name(GtkIMHtml *imhtml, const gchar *protocol_name)
  * @param imhtml  The GTK IM/HTML.
  * @param text    The formatted text to append.
  * @param options A GtkIMHtmlOptions object indicating insert behavior.
- * @param unused  Use NULL value.
+ * @param unused  Use @c NULL value.
  */
 void gtk_imhtml_append_text_with_images(GtkIMHtml *imhtml,
                                          const gchar *text,
@@ -428,7 +452,7 @@ void gtk_imhtml_hr_scale(GtkIMHtmlScalable *scale, int width, int height);
  * @param imhtml The GTK IM/HTML.
  * @param iter   The GtkTextIter at which to add the scalable.
  */
-void gtk_imhtml_hr_add_to(GtkIMHtmlScalable *scale, GtkIMHtml *, GtkTextIter *);
+void gtk_imhtml_hr_add_to(GtkIMHtmlScalable *scale, GtkIMHtml *imhtml, GtkTextIter *iter);
 
 /**
  * Finds and highlights a given string in a GTK IM/HTML.
@@ -466,19 +490,19 @@ void gtk_imhtml_set_editable(GtkIMHtml *imhtml, gboolean editable);
 void gtk_imhtml_set_whole_buffer_formatting_only(GtkIMHtml *imhtml, gboolean wbfo);
 
 /**
- * Indicates which formatting fuctions to enable and disable in a GTK IM/HTML.
+ * Indicates which formatting functions to enable and disable in a GTK IM/HTML.
  *
  * @param imhtml  The GTK IM/HTML.
- * @param buttons A GtkIMHtmlButtons bitmask indicating which fuctions to use.
+ * @param buttons A GtkIMHtmlButtons bitmask indicating which functions to use.
  */
 void gtk_imhtml_set_format_functions(GtkIMHtml *imhtml, GtkIMHtmlButtons buttons);
 
 /**
- * Returns which formatting fuctions are enabled in a GTK IM/HTML.
+ * Returns which formatting functions are enabled in a GTK IM/HTML.
  *
  * @param imhtml The GTK IM/HTML.
  *
- * @return A GtkIMHtmlButtons bitmask indicating which fuctions to are enabled.
+ * @return A GtkIMHtmlButtons bitmask indicating which functions to are enabled.
  */
 GtkIMHtmlButtons gtk_imhtml_get_format_functions(GtkIMHtml *imhtml);
 
@@ -569,12 +593,21 @@ gboolean gtk_imhtml_toggle_italic(GtkIMHtml *imhtml);
  */
 gboolean gtk_imhtml_toggle_underline(GtkIMHtml *imhtml);
 
+/**
+ * Toggles strikethrough at the cursor location or selection in a GTK IM/HTML.
+ *
+ * @param imhtml The GTK IM/HTML.
+ *
+ * @return @c TRUE if strikethrough was turned on, or @c FALSE if it was turned off.
+ */
+gboolean gtk_imhtml_toggle_strike(GtkIMHtml *imhtml);
+
 /**
  * Toggles a foreground color at the current location or selection in a GTK
  * IM/HTML.
  *
  * @param imhtml The GTK IM/HTML.
- * @param color  The HTML-style color, or NULL or "" to clear the color.
+ * @param color  The HTML-style color, or @c NULL or "" to clear the color.
  *
  * @return @c TRUE if a color was set, or @c FALSE if it was cleared.
  */
@@ -585,7 +618,7 @@ gboolean gtk_imhtml_toggle_forecolor(GtkIMHtml *imhtml, const char *color);
  * IM/HTML.
  *
  * @param imhtml The GTK IM/HTML.
- * @param color  The HTML-style color, or NULL or "" to clear the color.
+ * @param color  The HTML-style color, or @c NULL or "" to clear the color.
  *
  * @return @c TRUE if a color was set, or @c FALSE if it was cleared.
  */
@@ -595,7 +628,7 @@ gboolean gtk_imhtml_toggle_backcolor(GtkIMHtml *imhtml, const char *color);
  * Toggles a font face at the current location or selection in a GTK IM/HTML.
  *
  * @param imhtml The GTK IM/HTML.
- * @param face   The font face name, or NULL or "" to clear the font.
+ * @param face   The font face name, or @c NULL or "" to clear the font.
  *
  * @return @c TRUE if a font name was set, or @c FALSE if it was cleared.
  */
@@ -606,7 +639,7 @@ gboolean gtk_imhtml_toggle_fontface(GtkIMHtml *imhtml, const char *face);
  * in a GTK IM/HTML.
  *
  * @param imhtml The GTK IM/HTML.
- * @param url    The URL for the link or NULL to terminate the link.
+ * @param url    The URL for the link or @c NULL to terminate the link.
  */
 void gtk_imhtml_toggle_link(GtkIMHtml *imhtml, const char *url);
 
@@ -708,7 +741,7 @@ char **gtk_imhtml_get_markup_lines(GtkIMHtml *imhtml);
  *
  * @param imhtml The GTK IM/HTML.
  * @param start  The GtkTextIter indicating the start point in the IM/HTML.
- * @param end    The GtkTextIter indicating the end point in the IM/HTML.
+ * @param stop   The GtkTextIter indicating the end point in the IM/HTML.
  *
  * @return A string containing the unformatted text.
  */
@@ -720,4 +753,4 @@ char *gtk_imhtml_get_text(GtkIMHtml *imhtml, GtkTextIter *start, GtkTextIter *st
 }
 #endif
 
-#endif
+#endif /* _GAIM_GTKIMHTML_H_ */
