@@ -1,6 +1,6 @@
 /*
  * gaim - Gadu-Gadu Protocol Plugin
- * $Id: gg.c,v 1.104 2004/05/23 17:27:42 thekingant Exp $
+ * $Id: gg.c,v 1.116.2.1 2004/09/03 21:34:18 faceprint Exp $
  *
  * Copyright (C) 2001 Arkadiusz Mikiewicz <misiek@pld.ORG.PL>
  *
@@ -33,6 +33,7 @@
 #include "prpl.h"
 #include "server.h"
 #include "util.h"
+#include "version.h"
 
 #define GG_CONNECT_STEPS 5
 
@@ -200,11 +201,51 @@ static void agg_set_away(GaimConnection *gc, const char *state, const char *msg)
 		if (gd->own_status & GG_STATUS_FRIENDS_MASK)
 			status |= GG_STATUS_FRIENDS_MASK;
 	}
+	
+	if (msg) {
+	    switch (status) {
+		case GG_STATUS_AVAIL:
+		    status = GG_STATUS_AVAIL_DESCR;
+		    break;
+		case GG_STATUS_BUSY:
+		    status = GG_STATUS_BUSY_DESCR;
+		    break;
+		case GG_STATUS_INVISIBLE:
+		    status = GG_STATUS_INVISIBLE_DESCR;
+		    break;
+		case GG_STATUS_NOT_AVAIL:
+		    status = GG_STATUS_NOT_AVAIL_DESCR;
+		    break;
+	    }
+	}
 
 	gd->own_status = status;
-	gg_change_status(gd->sess, status);
+	if (msg)
+	    gg_change_status_descr(gd->sess, status, msg);
+	else
+	    gg_change_status(gd->sess, status);
 }
 
+#if 0
+static void agg_get_away(GaimConnection *gc, const char *who)
+{
+    GaimBuddy *buddy;
+    char *dialog_msg, **splitmsg;
+
+    if (invalid_uin(who))
+		return;
+    
+    buddy = gaim_find_buddy(gaim_connection_get_account(gc), who);
+    if (buddy->proto_data) {
+		/* Split at (carriage return/newline)'s, then rejoin later with BRs between. */
+		splitmsg = g_strsplit(buddy->proto_data, "\r\n", 0);
+    
+		dialog_msg = g_strdup_printf(_("<B>UIN:</B> %s<BR><B>Status:</B> %s<HR>%s"), who, (char *)buddy->proto_data, g_strjoinv("<BR>", splitmsg));
+		gaim_notify_userinfo(gc, who, NULL, _("Buddy Information"), buddy->proto_data, dialog_msg, NULL, NULL);
+    }
+}
+#endif
+
 static gchar *get_away_text(int uc)
 {
 	if (uc == UC_UNAVAILABLE)
@@ -326,7 +367,6 @@ static void agg_load_buddy_list(GaimConnection *gc, char *buddylist)
 			}
 			b = gaim_buddy_new(gc->account, name, strlen(show) ? show : NULL);
 			gaim_blist_add_buddy(b,NULL,g,NULL);
-			gaim_blist_save();
 
 			userlist_size++;
 			userlist = g_renew(uin_t, userlist, userlist_size);
@@ -346,12 +386,12 @@ static void agg_load_buddy_list(GaimConnection *gc, char *buddylist)
 	}
 }
 
-static void agg_save_buddy_list (GaimConnection *gc, char *existlist)
+static void agg_save_buddy_list(GaimConnection *gc, char *existlist)
 {
+    struct agg_data *gd = (struct agg_data *)gc->proto_data;
     GaimBlistNode *gnode, *cnode, *bnode;
     char *buddylist = g_strdup(existlist ? existlist : "");
     char *ptr;
-    struct agg_data *gd = (struct agg_data *)gc->proto_data;
 
     for(gnode = gaim_get_blist()->root; gnode; gnode = gnode->next) {
 		GaimGroup *g = (GaimGroup *)gnode;
@@ -468,8 +508,8 @@ static void main_callback(gpointer data, gint source, GaimInputCondition cond)
 				}
 
 				g_snprintf(user, sizeof(user), "%lu", n->uin);
-				serv_got_update(gc, user, (status == UC_UNAVAILABLE) ? 0 : 1, 0, 0, 0,
-						status);
+				serv_got_update(gc, user, (status == UC_UNAVAILABLE) ? FALSE : TRUE, 0, 0,
+					       0, status);
 				n++;
 			}
 		}
@@ -477,28 +517,45 @@ static void main_callback(gpointer data, gint source, GaimInputCondition cond)
 	case GG_EVENT_NOTIFY60:
 		{
 			gchar user[20];
-			struct gg_notify_reply60 *n = (void *)e->event.notify60;
 			guint status;
+			guint i = 0;
 
-			while (n->uin) {
-				switch (n->status) {
+			for (i = 0; e->event.notify60[i].uin; i++) {
+				GaimBuddy *buddy;
+				
+				g_snprintf(user, sizeof(user), "%lu", (long unsigned int)e->event.notify60[i].uin);
+
+				buddy = gaim_find_buddy(gaim_connection_get_account(gc), user);
+				if (buddy && buddy->proto_data != NULL) {
+					g_free(buddy->proto_data);
+					buddy->proto_data = NULL;
+				}
+			
+				switch (e->event.notify60[i].status) {
 				case GG_STATUS_NOT_AVAIL:
+				case GG_STATUS_NOT_AVAIL_DESCR:
 					status = UC_UNAVAILABLE;
 					break;
 				case GG_STATUS_AVAIL:
+				case GG_STATUS_AVAIL_DESCR:
 				case GG_STATUS_BUSY:
+				case GG_STATUS_BUSY_DESCR:
 				case GG_STATUS_INVISIBLE:
-					status = UC_NORMAL | (n->status << 5);
+				case GG_STATUS_INVISIBLE_DESCR:
+					status = UC_NORMAL | (e->event.notify60[i].status << 5);
 					break;
 				default:
 					status = UC_NORMAL;
 					break;
 				}
+				
+				if (buddy && e->event.notify60[i].descr != NULL) {
+					buddy->proto_data = g_strdup(e->event.notify60[i].descr);
+				}
 
-				g_snprintf(user, sizeof(user), "%lu", (long unsigned int)n->uin);
-				serv_got_update(gc, user, (status == UC_UNAVAILABLE) ? 0 : 1, 0, 0, 0,
-						status);
-				n++;
+				serv_got_update(gc, user, (status == UC_UNAVAILABLE) ? FALSE : TRUE, 0, 0,
+					       0, status);
+				i++;
 			}
 		}
 		break;
@@ -522,8 +579,8 @@ static void main_callback(gpointer data, gint source, GaimInputCondition cond)
 			}
 
 			g_snprintf(user, sizeof(user), "%lu", e->event.status.uin);
-			serv_got_update(gc, user, (status == UC_UNAVAILABLE) ? 0 : 1, 0, 0, 0,
-					status);
+			serv_got_update(gc, user, (status == UC_UNAVAILABLE) ? FALSE : TRUE, 0, 0,
+					0, status);
 		}
 		break;
 	case GG_EVENT_STATUS60:
@@ -531,23 +588,40 @@ static void main_callback(gpointer data, gint source, GaimInputCondition cond)
 			gchar user[20];
 			guint status;
 
+			GaimBuddy *buddy;
+				
+			g_snprintf(user, sizeof(user), "%lu", e->event.status60.uin);
+
+			buddy = gaim_find_buddy(gaim_connection_get_account(gc), user);
+			if (buddy && buddy->proto_data != NULL) {
+				g_free(buddy->proto_data);
+				buddy->proto_data = NULL;
+			}
+
 			switch (e->event.status60.status) {
 			case GG_STATUS_NOT_AVAIL:
+			case GG_STATUS_NOT_AVAIL_DESCR:
 				status = UC_UNAVAILABLE;
 				break;
 			case GG_STATUS_AVAIL:
+			case GG_STATUS_AVAIL_DESCR:
 			case GG_STATUS_BUSY:
+			case GG_STATUS_BUSY_DESCR:
 			case GG_STATUS_INVISIBLE:
+			case GG_STATUS_INVISIBLE_DESCR:
 				status = UC_NORMAL | (e->event.status60.status << 5);
 				break;
 			default:
 				status = UC_NORMAL;
 				break;
 			}
-
-			g_snprintf(user, sizeof(user), "%lu", e->event.status60.uin);
-			serv_got_update(gc, user, (status == UC_UNAVAILABLE) ? 0 : 1, 0, 0, 0,
-					status);
+    
+			if (buddy && e->event.status60.descr != NULL) {
+				buddy->proto_data = g_strdup(e->event.status60.descr);
+			}
+			    
+			serv_got_update(gc, user, (status == UC_UNAVAILABLE) ? FALSE : TRUE, 0, 0,
+					0, status);
 		}
 		break;
 	case GG_EVENT_ACK:
@@ -781,38 +855,39 @@ static int agg_send_im(GaimConnection *gc, const char *who, const char *msg, Gai
 	return 1;
 }
 
-static void agg_add_buddy(GaimConnection *gc, const char *who, GaimGroup *group)
+static void agg_add_buddy(GaimConnection *gc, GaimBuddy *buddy, GaimGroup *group)
 {
 	struct agg_data *gd = (struct agg_data *)gc->proto_data;
-	if (invalid_uin(who))
+	if (invalid_uin(buddy->name))
 		return;
-	gg_add_notify(gd->sess, strtol(who, (char **)NULL, 10));
+	gg_add_notify(gd->sess, strtol(buddy->name, (char **)NULL, 10));
 	agg_save_buddy_list(gc, NULL);
 }
 
-static void agg_rem_buddy(GaimConnection *gc, const char *who, const char *group)
+static void agg_rem_buddy(GaimConnection *gc, GaimBuddy *buddy, GaimGroup *group)
 {
 	struct agg_data *gd = (struct agg_data *)gc->proto_data;
-	if (invalid_uin(who))
+	if (invalid_uin(buddy->name))
 		return;
-	gg_remove_notify(gd->sess, strtol(who, (char **)NULL, 10));
+	gg_remove_notify(gd->sess, strtol(buddy->name, (char **)NULL, 10));
 	agg_save_buddy_list(gc, NULL);
 }
 
-static void agg_add_buddies(GaimConnection *gc, GList *whos)
+static void agg_add_buddies(GaimConnection *gc, GList *buddies, GList *groups)
 {
 	struct agg_data *gd = (struct agg_data *)gc->proto_data;
 	uin_t *userlist = NULL;
 	int userlist_size = 0;
 
-	while (whos) {
-		if (!invalid_uin(whos->data)) {
+	while (buddies) {
+		GaimBuddy *buddy = buddies->data;
+		if (!invalid_uin(buddy->name)) {
 			userlist_size++;
 			userlist = g_renew(uin_t, userlist, userlist_size);
 			userlist[userlist_size - 1] =
-			    (uin_t) strtol((char *)whos->data, (char **)NULL, 10);
+			    (uin_t) strtol(buddy->name, (char **)NULL, 10);
 		}
-		whos = g_list_next(whos);
+		buddies = g_list_next(buddies);
 	}
 
 	if (userlist) {
@@ -823,6 +898,14 @@ static void agg_add_buddies(GaimConnection *gc, GList *whos)
 	agg_save_buddy_list(gc, NULL);
 }
 
+static void agg_buddy_free (GaimBuddy *buddy)
+{
+    if (buddy->proto_data) {
+	g_free(buddy->proto_data);
+	buddy->proto_data = NULL;
+    }
+}
+
 static void search_results(GaimConnection *gc, gchar *webdata)
 {
 	gchar **webdata_tbl;
@@ -844,6 +927,7 @@ static void search_results(GaimConnection *gc, gchar *webdata)
 	j = 0;
 
 	/* Parse array */
+	/* XXX - Make this use a GString */
 	for (i = 0; webdata_tbl[i] != NULL; i++) {
 		gchar *p, *oldibuf;
 		static gchar *ibuf;
@@ -1004,8 +1088,7 @@ static void import_buddies_server_results(GaimConnection *gc, gchar *webdata)
 				gaim_blist_add_group(g, NULL);
 			}
 			b = gaim_buddy_new(gc->account, name, strlen(show) ? show : NULL);
-			gaim_blist_add_buddy(b,NULL,g,NULL);
-			gaim_blist_save();
+			gaim_blist_add_buddy(b, NULL, g, NULL);
 			g_free(group);
 		}
 		g_free(show);
@@ -1519,8 +1602,8 @@ static void agg_group_buddy (GaimConnection *gc, const char *who,
     g_free(newdata);
 }
 
-static void agg_rename_group (GaimConnection *gc, const char *old_group,
-		     const char *new_group, GList *members)
+static void agg_rename_group (GaimConnection *gc, const char *old_name,
+		     GaimGroup *group, GList *moved_buddies)
 {
     agg_save_buddy_list(gc, NULL);
 }
@@ -1529,65 +1612,71 @@ static GaimPlugin *my_protocol = NULL;
 
 static GaimPluginProtocolInfo prpl_info =
 {
-	GAIM_PRPL_API_VERSION,
 	0,
-	NULL,
-	NULL,
-	agg_list_icon,
-	agg_list_emblems,
-	NULL,
-	NULL,
-	agg_away_states,
-	agg_blist_node_menu,
-	NULL,
-	agg_login,
-	agg_close,
-	agg_send_im,
-	NULL,
-	NULL,
-	agg_get_info,
-	agg_set_away,
-	NULL,
-	agg_change_passwd,
-	agg_add_buddy,
-	agg_add_buddies,
-	agg_rem_buddy,
-	NULL,
-	agg_permit_deny_dummy,
-	agg_permit_deny_dummy,
-	agg_permit_deny_dummy,
-	agg_permit_deny_dummy,
-	agg_set_permit_deny_dummy,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	agg_keepalive,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	agg_group_buddy,
-	agg_rename_group,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
+	NULL,						/* user_splits */
+	NULL,						/* protocol_options */
+	NO_BUDDY_ICONS,				/* icon_spec */
+	agg_list_icon,				/* list_icon */
+	agg_list_emblems,			/* list_emblems */
+	NULL,						/* status_text */
+	NULL,						/* tooltip_text */
+	agg_away_states,			/* away_states */
+	agg_blist_node_menu,		/* blist_node_menu */
+	NULL,						/* chat_info */
+	NULL,						/* chat_info_defaults */
+	agg_login,					/* login */
+	agg_close,					/* close */
+	agg_send_im,				/* send_im */
+	NULL,						/* set_info */
+	NULL,						/* send_typing */
+	agg_get_info,				/* get_info */
+	agg_set_away,				/* set_away */
+	NULL,						/* set_idle */
+	agg_change_passwd,			/* change_passwd */
+	agg_add_buddy,				/* add_buddy */
+	agg_add_buddies,			/* add_buddies */
+	agg_rem_buddy,				/* remove_buddy */
+	NULL,						/* remove_buddies */
+	agg_permit_deny_dummy,		/* add_permit */
+	agg_permit_deny_dummy,		/* add_deny */
+	agg_permit_deny_dummy,		/* rem_permit */
+	agg_permit_deny_dummy,		/* rem_deny */
+	agg_set_permit_deny_dummy,	/* set_permit_deny */
+	NULL,						/* warn */
+	NULL,						/* join_chat */
+	NULL,						/* reject_chat */
+	NULL,						/* get_chat_name */
+	NULL,						/* chat_invite */
+	NULL,						/* chat_leave */
+	NULL,						/* chat_whisper */
+	NULL,						/* chat_send */
+	agg_keepalive,				/* keepalive */
+	NULL,						/* register_user */
+	NULL,						/* get_cb_info */
+	NULL,						/* get_cb_away */
+	NULL,						/* alias_buddy */
+	agg_group_buddy,			/* group_buddy */
+	agg_rename_group,			/* rename_group */
+	agg_buddy_free,				/* buddy_free */
+	NULL,						/* convo_closed */
+	NULL,						/* normalize */
+	NULL,						/* set_buddy_icon */
+	NULL,						/* remove_group */
+	NULL,						/* get_cb_real_name */
+	NULL,						/* set_chat_topic */
+	NULL,						/* find_blist_chat */
+	NULL,						/* roomlist_get_list */
+	NULL,						/* roomlist_cancel */
+	NULL,						/* roomlist_expand_category */
+	NULL,						/* can_receive_file */
+	NULL						/* send_file */
 };
 
 static GaimPluginInfo info =
 {
-	GAIM_PLUGIN_API_VERSION,                          /**< api_version    */
+	GAIM_PLUGIN_MAGIC,
+	GAIM_MAJOR_VERSION,
+	GAIM_MINOR_VERSION,
 	GAIM_PLUGIN_PROTOCOL,                             /**< type           */
 	NULL,                                             /**< ui_requirement */
 	0,                                                /**< flags          */
@@ -1627,4 +1716,4 @@ init_plugin(GaimPlugin *plugin)
 	my_protocol = plugin;
 }
 
-GAIM_INIT_PLUGIN(gg, init_plugin, info);
+GAIM_INIT_PLUGIN(gg, init_plugin, info)
