@@ -1,4 +1,4 @@
-/* $Id: libgg.c,v 1.16 2004/04/23 17:24:19 lschiere Exp $ */
+/* $Id: libgg.c,v 1.21 2004/08/25 03:09:08 thekingant Exp $ */
 
 /*
  *  (C) Copyright 2001 Wojtek Kaniewski <wojtekka@irc.pl>,
@@ -49,9 +49,9 @@
 #if G_BYTE_ORDER == G_BIG_ENDIAN
 #  define WORDS_BIGENDIAN 1
 #endif
+#include "internal.h"
 #include "libgg.h"
 
-#include "internal.h"
 #include "proxy.h"
 #include "debug.h"
 
@@ -64,16 +64,13 @@ int gg_http_use_proxy = 0;
 int gg_http_proxy_port = 0;
 char *gg_http_proxy_host = NULL;
 
-/* temp -Herman */
-static int ping_outstanding = 0;
-
 #ifndef lint 
 
 static char rcsid[]
 #ifdef __GNUC__
 __attribute__ ((unused))
 #endif
-= "$Id: libgg.c,v 1.16 2004/04/23 17:24:19 lschiere Exp $";
+= "$Id: libgg.c,v 1.21 2004/08/25 03:09:08 thekingant Exp $";
 
 #endif 
 
@@ -591,6 +588,38 @@ int gg_change_status(struct gg_session *sess, int status)
 	return gg_send_packet(sess, GG_NEW_STATUS, &p, sizeof(p), NULL, 0);
 }
 
+/*
+ * gg_change_status_descr()
+ *
+ * zmienia status uøytkownika na opisowy.
+ *
+ *  - sess - opis sesji
+ *  - status - nowy status uøytkownika
+ *  - descr - opis statusu
+ *
+ * 0, -1.
+ */
+int gg_change_status_descr(struct gg_session *sess, int status, const char *descr)
+{
+	struct gg_new_status p;
+
+	gg_debug(GG_DEBUG_FUNCTION, "** gg_change_status_descr(%p, %d, \"%s\");\n", sess, status, descr);
+
+	if (!sess || !descr) {
+		errno = EFAULT;
+		return -1;
+	}
+
+	if (sess->state != GG_STATE_CONNECTED) {
+		errno = ENOTCONN;
+		return -1;
+	}
+
+	p.status = fix32(status);
+
+	return gg_send_packet(sess, GG_NEW_STATUS, &p, sizeof(p), descr, (strlen(descr) > GG_STATUS_DESCR_MAXSIZE) ? GG_STATUS_DESCR_MAXSIZE : strlen(descr), NULL);
+}
+
 /*
  * gg_logoff()
  *
@@ -661,7 +690,7 @@ int gg_send_message_richtext(struct gg_session *sess, int msgclass, uin_t recipi
 		errno = EFAULT;
 		return -1;
 	}
-	
+
 	if (sess->state != GG_STATE_CONNECTED) {
 		errno = ENOTCONN;
 		return -1;
@@ -673,7 +702,7 @@ int gg_send_message_richtext(struct gg_session *sess, int msgclass, uin_t recipi
 	s.seq = fix32(sess->seq);
 	s.msgclass = fix32(msgclass);
 	sess->seq += (rand() % 0x300) + 0x300;
-	
+
 	if (gg_send_packet(sess, GG_SEND_MSG, &s, sizeof(s), message, strlen(message) + 1, format, formatlen, NULL) == -1)
 		return -1;
 
@@ -702,7 +731,7 @@ int gg_ping(struct gg_session *sess)
 	}
 
 	gg_debug(GG_DEBUG_FUNCTION, "** gg_ping(...);\n");
-	
+
 	return gg_send_packet(sess, GG_PING, NULL);
 }
 
@@ -912,8 +941,8 @@ static int gg_watch_fd_connected(struct gg_session *sess, struct gg_event *e)
 	}
 
 	p = (void *)h + sizeof(struct gg_header);
-	
-	
+
+
 	switch (h->type) {
 	    case GG_RECV_MSG:
 	    {
@@ -1094,7 +1123,6 @@ static int gg_watch_fd_connected(struct gg_session *sess, struct gg_event *e)
 	    case GG_PONG:
 	    {
 		gg_debug(GG_DEBUG_MISC, "-- received a pong\n");
-		ping_outstanding = 0;
 		sess->last_pong = time(NULL);
 		break;
 	    }
@@ -1294,18 +1322,18 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 
 			if (gg_http_use_proxy) {
 				g_snprintf(buf, sizeof(buf) - 1,
-					"GET http://" GG_APPMSG_HOST "/appsvc/appmsg.asp?fmnumber=%lu HTTP/1.0\r\n"
+					"GET http://" GG_APPMSG_HOST "/appsvc/appmsg2.asp?fmnumber=%lu&version=%s&lastmsg=0 HTTP/1.0\r\n"
 					"Host: " GG_APPMSG_HOST "\r\n"
 					"User-Agent: " GG_HTTP_USERAGENT "\r\n"
 					"Pragma: no-cache\r\n"
-					"\r\n", sess->uin);
+					"\r\n", sess->uin, gg_urlencode(GG_DEFAULT_CLIENT_VERSION));
 			} else {
 				g_snprintf(buf, sizeof(buf) - 1,
-					"GET /appsvc/appmsg.asp?fmnumber=%lu HTTP/1.0\r\n"
+					"GET /appsvc/appmsg2.asp?fmnumber=%lu&version=%s&lastmsg=0 HTTP/1.0\r\n"
 					"Host: " GG_APPMSG_HOST "\r\n"
 					"User-Agent: " GG_HTTP_USERAGENT "\r\n"
 					"Pragma: no-cache\r\n"
-					"\r\n", sess->uin);
+					"\r\n", sess->uin, gg_urlencode(GG_DEFAULT_CLIENT_VERSION));
 			};
 
     			gg_debug(GG_DEBUG_MISC, "=> -----BEGIN-HTTP-QUERY-----\n%s\n=> -----END-HTTP-QUERY-----\n", buf);
@@ -1360,21 +1388,13 @@ struct gg_event *gg_watch_fd(struct gg_session *sess)
 			tmp = buf;
 			
 			while (*tmp && *tmp != ' ')
-				tmp++;
-			while (*tmp && *tmp == ' ')
-				tmp++;
-			while (*tmp && *tmp != ' ')
-				tmp++;
-			while (*tmp && *tmp == ' ')
-				tmp++;
-			while (*tmp && *tmp != ' ')
-				tmp++;
-			while (*tmp && *tmp == ' ')
-				tmp++;
-			host = tmp;
-			while (*tmp && *tmp != ' ')
-				tmp++;
-			*tmp = 0;
+                                tmp++;
+                        while (*tmp && *tmp == ' ')
+                                tmp++;
+                        host = tmp;
+                        while (*tmp && *tmp != ' ')
+                                tmp++;
+                        *tmp = 0;
 
 			if ((tmp = strchr(host, ':'))) {
 				*tmp = 0;
