@@ -111,48 +111,23 @@ int irc_cmd_ctcp_action(struct irc_conn *irc, const char *cmd, const char *targe
 	g_free(newargs);
 
 	convo = gaim_find_conversation_with_account(target, irc->account);
-	if (convo && gaim_conversation_get_type(convo) == GAIM_CONV_CHAT) {
+	if (convo) {
 		action = g_strdup_printf("/me %s", args[0]);
 		if (action[strlen(action) - 1] == '\n')
 			action[strlen(action) - 1] = '\0';
-		serv_got_chat_in(gc, gaim_conv_chat_get_id(GAIM_CONV_CHAT(convo)),
-				 gaim_connection_get_display_name(gc),
-				 0, action, time(NULL));
+		if (gaim_conversation_get_type(convo) == GAIM_CONV_CHAT)
+			serv_got_chat_in(gc, gaim_conv_chat_get_id(GAIM_CONV_CHAT(convo)),
+			         	 gaim_connection_get_display_name(gc),
+				         0, action, time(NULL));
+		else
+			gaim_conv_im_write(GAIM_CONV_IM(convo), gaim_connection_get_display_name(gc),
+			                  action, 0, time(NULL));
 		g_free(action);
 	}
 
 	return 1;
 }
 
-int irc_cmd_help(struct irc_conn *irc, const char *cmd, const char *target, const char **args)
-{
-	GaimConversation *convo = gaim_find_conversation_with_account(target, irc->account);
-
-	/* XXX we should eventually have per-command help */
-
-	if (!convo)
-		return 0;
-
-	if (gaim_conversation_get_type(convo) == GAIM_CONV_CHAT) {
-		gaim_conv_chat_write(GAIM_CONV_CHAT(convo), "", _("<B>Supported IRC Commands:</B><BR>"
-							"AWAY INVITE JOIN KICK<BR>"
-							"ME MODE MSG NAMES<BR>"
-							"NICK OP DEOP OPERWALL<BR>"
-							"PART PING QUERY QUIT<BR>"
-							"QUOTE REMOVE TOPIC UMODE<BR>"
-							"VOICE DEVOICE WALLOPS WHOIS<BR>"),
-				GAIM_MESSAGE_NO_LOG, time(NULL));
-	} else {
-		gaim_conv_im_write(GAIM_CONV_IM(convo), "", _("<B>Supported IRC Commands:</B><BR>"
-						    "AWAY JOIN ME MODE<BR>"
-						    "MSG NICK OPERWALL PING<BR>"
-						    "QUERY QUIT QUOTE UMODE<BR>"
-						    "WALLOPS WHOIS"), GAIM_MESSAGE_NO_LOG, time(NULL));
-	}
-
-	return 0;
-}
-
 int irc_cmd_invite(struct irc_conn *irc, const char *cmd, const char *target, const char **args)
 {
 	char *buf;
@@ -222,7 +197,7 @@ int irc_cmd_mode(struct irc_conn *irc, const char *cmd, const char *target, cons
 		return 0;
 
 	if (!strcmp(cmd, "mode")) {
-		if (!args[0] && (*target == '#' || *target == '&'))
+		if (!args[0] && irc_ischannel(target))
 			buf = irc_format(irc, "vc", "MODE", target);
 		else if (args[0] && (*args[0] == '+' || *args[0] == '-'))
 			buf = irc_format(irc, "vcv", "MODE", target, args[0]);
@@ -249,7 +224,7 @@ int irc_cmd_names(struct irc_conn *irc, const char *cmd, const char *target, con
 {
 	char *buf;
 
-	if (!args)
+	if (!args || (!args[0] && !irc_ischannel(target)))
 		return 0;
 
 	buf = irc_format(irc, "vc", "NAMES", args[0] ? args[0] : target);
@@ -342,7 +317,7 @@ int irc_cmd_ping(struct irc_conn *irc, const char *cmd, const char *target, cons
 	char *buf;
 
 	if (args && args[0]) {
-		if (*args[0] == '#' || *args[0] == '&')
+		if (irc_ischannel(args[0]))
 			return 0;
 		stamp = g_strdup_printf("\001PING %lu\001", time(NULL));
 		buf = irc_format(irc, "vn:", "PRIVMSG", args[0], stamp);
@@ -387,9 +362,13 @@ int irc_cmd_quit(struct irc_conn *irc, const char *cmd, const char *target, cons
 {
 	char *buf;
 
-	buf = irc_format(irc, "v:", "QUIT", (args && args[0]) ? args[0] : "Download Gaim: " GAIM_WEBSITE);
-	irc_send(irc, buf);
-	g_free(buf);
+	if (!irc->quitting) {
+		buf = irc_format(irc, "v:", "QUIT", (args && args[0]) ? args[0] : "Download Gaim: " GAIM_WEBSITE);
+		irc_send(irc, buf);
+		g_free(buf);
+
+		irc->quitting = TRUE;
+	}
 
 	return 0;
 }
@@ -435,7 +414,7 @@ int irc_cmd_remove(struct irc_conn *irc, const char *cmd, const char *target, co
 	if (!args || !args[0])
 		return 0;
 
-	if (*target != '#' && *target != '&') /* not a channel, punt */
+	if (!irc_ischannel(target)) /* not a channel, punt */
 		return 0;
 
 	if (args[1])
@@ -465,9 +444,12 @@ int irc_cmd_topic(struct irc_conn *irc, const char *cmd, const char *target, con
 		topic = gaim_conv_chat_get_topic (GAIM_CONV_CHAT(convo));
 
 		if (topic) {
-			char *tmp = gaim_escape_html(topic);
-			buf = g_strdup_printf(_("current topic is: %s"), tmp);
+			char *tmp, *tmp2;
+			tmp = gaim_escape_html(topic);
+			tmp2 = gaim_markup_linkify(tmp);
+			buf = g_strdup_printf(_("current topic is: %s"), tmp2);
 			g_free(tmp);
+			g_free(tmp2);
 		} else
 			buf = g_strdup(_("No topic is set"));
 		gaim_conv_chat_write(GAIM_CONV_CHAT(convo), target, buf, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
