@@ -29,11 +29,11 @@
 #include "blist.h"
 #include "conversation.h"
 #include "debug.h"
-#include "multi.h"
 #include "notify.h"
 #include "prpl.h"
 #include "plugin.h"
 #include "util.h"
+#include "version.h"
 
 #include "irc.h"
 
@@ -82,11 +82,17 @@ static void irc_view_motd(GaimPluginAction *action)
 
 int irc_send(struct irc_conn *irc, const char *buf)
 {
+	int ret;
+
 	if (irc->fd < 0)
 		return -1;
 
 	/* gaim_debug(GAIM_DEBUG_MISC, "irc", "sent: %s", buf); */
-	return write(irc->fd, buf, strlen(buf));
+	if ((ret = write(irc->fd, buf, strlen(buf))) < 0)
+		gaim_connection_error(gaim_account_get_connection(irc->account),
+				      _("Server has disconnected"));
+
+	return ret;
 }
 
 /* XXX I don't like messing directly with these buddies */
@@ -117,6 +123,17 @@ static void irc_buddy_append(char *name, struct irc_buddy *ib, GString *string)
 	g_string_append_printf(string, "%s ", name);
 }
 
+static void irc_ison_one(struct irc_conn *irc, struct irc_buddy *ib)
+{
+	char *buf;
+
+	ib->flag = FALSE;
+	buf = irc_format(irc, "vn", "ISON", ib->name);
+	irc_send(irc, buf);
+	g_free(buf);
+}
+
+
 static const char *irc_blist_icon(GaimAccount *a, GaimBuddy *b)
 {
 	return "irc";
@@ -144,34 +161,15 @@ static GList *irc_actions(GaimPlugin *plugin, gpointer context)
 	return list;
 }
 
-
-static void irc_dccsend_send_ask_menu(GaimBlistNode *node, gpointer data)
-{
-	GaimBuddy *buddy;
-	GaimConnection *gc;
-
-	g_return_if_fail(GAIM_BLIST_NODE_IS_BUDDY(node));
-
-	buddy = (GaimBuddy *) node;
-	gc =gaim_account_get_connection(buddy->account);
-
-	irc_dccsend_send_ask(gc, buddy->name);
-}
-
-
+#if 0
 static GList *irc_blist_node_menu(GaimBlistNode *node)
 {
 	GList *m = NULL;
 	GaimBlistNodeAction *act;
 
-	if(GAIM_BLIST_NODE_IS_BUDDY(node)) {
-		act = gaim_blist_node_action_new(_("Send File"),
-				irc_dccsend_send_ask_menu, NULL);
-		m = g_list_append(m, act);
-	}
-	
 	return m;
 }
+#endif
 
 static GList *irc_chat_join_info(GaimConnection *gc)
 {
@@ -192,11 +190,23 @@ static GList *irc_chat_join_info(GaimConnection *gc)
 	return m;
 }
 
+GHashTable *irc_chat_info_defaults(GaimConnection *gc, const char *chat_name)
+{
+	GHashTable *defaults;
+
+	defaults = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, g_free);
+
+	if (chat_name != NULL)
+		g_hash_table_insert(defaults, "channel", g_strdup(chat_name));
+
+	return defaults;
+}
+
 static void irc_login(GaimAccount *account)
 {
 	GaimConnection *gc;
 	struct irc_conn *irc;
-	char *buf, **userparts;
+	char **userparts;
 	const char *username = gaim_account_get_username(account);
 	int err;
 
@@ -223,9 +233,7 @@ static void irc_login(GaimAccount *account)
 	irc->msgs = g_hash_table_new(g_str_hash, g_str_equal);
 	irc_msg_table_build(irc);
 
-	buf = g_strdup_printf(_("Signon: %s"), username);
-	gaim_connection_update_progress(gc, buf, 1, 2);
-	g_free(buf);
+	gaim_connection_update_progress(gc, _("Connecting"), 1, 2);
 
 	err = gaim_proxy_connect(account, irc->server, 
 				 gaim_account_get_int(account, "port", IRC_DEFAULT_PORT),
@@ -243,7 +251,7 @@ static void irc_login_cb(gpointer data, gint source, GaimInputCondition cond)
 	struct irc_conn *irc = gc->proto_data;
 	char hostname[256];
 	char *buf;
-	const char *username;
+	const char *username, *realname;
 	GList *connections = gaim_connections_get_all();
 
 	if (source < 0) {
@@ -270,8 +278,9 @@ static void irc_login_cb(gpointer data, gint source, GaimInputCondition cond)
 	gethostname(hostname, sizeof(hostname));
 	hostname[sizeof(hostname) - 1] = '\0';
 	username = gaim_account_get_string(irc->account, "username", "");
+	realname = gaim_account_get_string(irc->account, "realname", "");
 	buf = irc_format(irc, "vvvv:", "USER", strlen(username) ? username : g_get_user_name(), hostname, irc->server,
-			      gc->account->alias && *gc->account->alias ? gc->account->alias : IRC_DEFAULT_ALIAS);
+			      strlen(realname) ? realname : IRC_DEFAULT_ALIAS);
 	if (irc_send(irc, buf) < 0) {
 		gaim_connection_error(gc, "Error registering with server");
 		return;
@@ -297,7 +306,7 @@ static void irc_close(GaimConnection *gc)
 	irc_cmd_quit(irc, "quit", NULL, NULL);
 
 	if (gc->inpa)
-		gaim_timeout_remove(gc->inpa);
+		gaim_input_remove(gc->inpa);
 
 	g_free(irc->inbuf);
 	close(irc->fd);
@@ -305,6 +314,7 @@ static void irc_close(GaimConnection *gc)
 		gaim_timeout_remove(irc->timer);
 	g_hash_table_destroy(irc->cmds);
 	g_hash_table_destroy(irc->msgs);
+	g_hash_table_destroy(irc->buddies);
 	if (irc->motd)
 		g_string_free(irc->motd, TRUE);
 	g_free(irc->server);
@@ -322,10 +332,6 @@ static int irc_im_send(GaimConnection *gc, const char *who, const char *what, Ga
 		args[0] = who;
 	args[1] = what;
 
-	if (*what == '/') {
-		return irc_parse_cmd(irc, who, what + 1);
-	}
-
 	irc_cmd_privmsg(irc, "msg", NULL, args);
 	return 1;
 }
@@ -355,21 +361,26 @@ static void irc_set_away(GaimConnection *gc, const char *state, const char *msg)
 	irc_cmd_away(irc, "away", NULL, args);
 }
 
-static void irc_add_buddy(GaimConnection *gc, const char *who, GaimGroup *group)
+static void irc_add_buddy(GaimConnection *gc, GaimBuddy *buddy, GaimGroup *group)
 {
 	struct irc_conn *irc = (struct irc_conn *)gc->proto_data;
 	struct irc_buddy *ib = g_new0(struct irc_buddy, 1);
-	ib->name = g_strdup(who);
+	ib->name = g_strdup(buddy->name);
 	g_hash_table_insert(irc->buddies, ib->name, ib);
+
+	/* if the timer isn't set, this is during signon, so we don't want to flood
+	 * ourself off with ISON's, so we don't, but after that we want to know when
+	 * someone's online asap */
+	if (irc->timer)
+		irc_ison_one(irc, ib);
 }
 
-static void irc_remove_buddy(GaimConnection *gc, const char *who, const char *group)
+static void irc_remove_buddy(GaimConnection *gc, GaimBuddy *buddy, GaimGroup *group)
 {
 	struct irc_conn *irc = (struct irc_conn *)gc->proto_data;
-	g_hash_table_remove(irc->buddies, who);
+	g_hash_table_remove(irc->buddies, buddy->name);
 }
 
-
 static void irc_input_cb(gpointer data, gint source, GaimInputCondition cond)
 {
 	GaimConnection *gc = data;
@@ -386,7 +397,7 @@ static void irc_input_cb(gpointer data, gint source, GaimInputCondition cond)
 		gaim_connection_error(gc, _("Read error"));
 		return;
 	} else if (len == 0) {
-		/* Remote closed the connection, probably */
+		gaim_connection_error(gc, _("Server has disconnected"));
 		return;
 	}
 
@@ -419,6 +430,10 @@ static void irc_chat_join (GaimConnection *gc, GHashTable *data)
 	irc_cmd_join(irc, "join", NULL, args);
 }
 
+static char *irc_get_chat_name(GHashTable *data) {
+	return g_strdup(g_hash_table_lookup(data, "channel"));
+}
+
 static void irc_chat_invite(GaimConnection *gc, int id, const char *message, const char *name) 
 {
 	struct irc_conn *irc = gc->proto_data;
@@ -461,11 +476,11 @@ static int irc_chat_send(GaimConnection *gc, int id, const char *what)
 		gaim_debug(GAIM_DEBUG_ERROR, "irc", "chat send on nonexistent chat\n");
 		return -EINVAL;
 	}
-
+#if 0
 	if (*what == '/') {
 		return irc_parse_cmd(irc, convo->name, what + 1);
 	}
-
+#endif
 	args[0] = convo->name;
 	args[1] = what;
 
@@ -500,6 +515,23 @@ static void irc_buddy_free(struct irc_buddy *ib)
 	g_free(ib);
 }
 
+static void irc_chat_set_topic(GaimConnection *gc, int id, const char *topic)
+{
+	char *buf;
+	const char *name = NULL;
+	struct irc_conn *irc;
+
+	irc = gc->proto_data;
+	name = gaim_conversation_get_name(gaim_find_chat(gc, id));
+
+	if (name == NULL)
+		return;
+
+	buf = irc_format(irc, "vt:", "TOPIC", name, topic);
+	irc_send(irc, buf);
+	g_free(buf);
+}
+
 static GaimRoomlist *irc_roomlist_get_list(GaimConnection *gc)
 {
 	struct irc_conn *irc;
@@ -552,66 +584,72 @@ static void irc_roomlist_cancel(GaimRoomlist *list)
 
 static GaimPluginProtocolInfo prpl_info =
 {
-	GAIM_PRPL_API_VERSION,
 	OPT_PROTO_CHAT_TOPIC | OPT_PROTO_PASSWORD_OPTIONAL,
-	NULL,
-	NULL,
-	irc_blist_icon,
-	irc_blist_emblems,
-	NULL,
-	NULL,
-	irc_away_states,
-	irc_blist_node_menu,
-	irc_chat_join_info,
-	irc_login,
-	irc_close,
-	irc_im_send,
-	NULL,
-	NULL,
-	irc_get_info,
-	irc_set_away,
-	NULL,
-	NULL,
-	irc_add_buddy,
-	NULL,
-	irc_remove_buddy,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	irc_chat_join,
-	NULL,  /* reject chat invite */
-	irc_chat_invite,
-	irc_chat_leave,
-	NULL,
-	irc_chat_send,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL, /*irc_convo_closed,*/
-	NULL, /* normalize */
-	NULL, /* set buddy icon */
-	NULL, /* remove group */
-	NULL, /* get_cb_real_name */
-	NULL,
-	NULL,
-	irc_roomlist_get_list,
-	irc_roomlist_cancel,
-	NULL
+	NULL,					/* user_splits */
+	NULL,					/* protocol_options */
+	NO_BUDDY_ICONS,			/* icon_spec */
+	irc_blist_icon,			/* list_icon */
+	irc_blist_emblems,		/* list_emblems */
+	NULL,					/* status_text */
+	NULL,					/* tooltip_text */
+	irc_away_states,		/* away_states */
+	NULL,					/* blist_node_menu */
+	irc_chat_join_info,		/* chat_info */
+	irc_chat_info_defaults,	/* chat_info_defaults */
+	irc_login,				/* login */
+	irc_close,				/* close */
+	irc_im_send,			/* send_im */
+	NULL,					/* set_info */
+	NULL,					/* send_typing */
+	irc_get_info,			/* get_info */
+	irc_set_away,			/* set_away */
+	NULL,					/* set_idle */
+	NULL,					/* change_passwd */
+	irc_add_buddy,			/* add_buddy */
+	NULL,					/* add_buddies */
+	irc_remove_buddy,		/* remove_buddy */
+	NULL,					/* remove_buddies */
+	NULL,					/* add_permit */
+	NULL,					/* add_deny */
+	NULL,					/* rem_permit */
+	NULL,					/* rem_deny */
+	NULL,					/* set_permit_deny */
+	NULL,					/* warn */
+	irc_chat_join,			/* join_chat */
+	NULL,					/* reject_chat */
+	irc_get_chat_name,		/* get_chat_name */
+	irc_chat_invite,		/* chat_invite */
+	irc_chat_leave,			/* chat_leave */
+	NULL,					/* chat_whisper */
+	irc_chat_send,			/* chat_send */
+	NULL,					/* keepalive */
+	NULL,					/* register_user */
+	NULL,					/* get_cb_info */
+	NULL,					/* get_cb_away */
+	NULL,					/* alias_buddy */
+	NULL,					/* group_buddy */
+	NULL,					/* rename_group */
+	NULL,					/* buddy_free */
+	NULL,					/* convo_closed */
+	NULL,					/* normalize */
+	NULL,					/* set_buddy_icon */
+	NULL,					/* remove_group */
+	NULL,					/* get_cb_real_name */
+	irc_chat_set_topic,		/* set_chat_topic */
+	NULL,					/* find_blist_chat */
+	irc_roomlist_get_list,	/* roomlist_get_list */
+	irc_roomlist_cancel,	/* roomlist_cancel */
+	NULL,					/* roomlist_expand_category */
+	NULL,					/* can_receive_file */
+	irc_dccsend_send_file	/* send_file */
 };
 
 
 static GaimPluginInfo info =
 {
-	GAIM_PLUGIN_API_VERSION,                          /**< api_version    */
+	GAIM_PLUGIN_MAGIC,
+	GAIM_MAJOR_VERSION,
+	GAIM_MINOR_VERSION,
 	GAIM_PLUGIN_PROTOCOL,                             /**< type           */
 	NULL,                                             /**< ui_requirement */
 	0,                                                /**< flags          */
@@ -647,13 +685,18 @@ static void _init_plugin(GaimPlugin *plugin)
 	option = gaim_account_option_int_new(_("Port"), "port", IRC_DEFAULT_PORT);
 	prpl_info.protocol_options = g_list_append(prpl_info.protocol_options, option);
 
-	option = gaim_account_option_string_new(_("Encoding"), "encoding", IRC_DEFAULT_CHARSET);
+	option = gaim_account_option_string_new(_("Encodings"), "encoding", IRC_DEFAULT_CHARSET);
 	prpl_info.protocol_options = g_list_append(prpl_info.protocol_options, option);
 
 	option = gaim_account_option_string_new(_("Username"), "username", "");
 	prpl_info.protocol_options = g_list_append(prpl_info.protocol_options, option);
 
+	option = gaim_account_option_string_new(_("Real name"), "realname", "");
+	prpl_info.protocol_options = g_list_append(prpl_info.protocol_options, option);
+
 	_irc_plugin = plugin;
+
+	irc_register_commands();
 }
 
 GAIM_INIT_PLUGIN(irc, _init_plugin, info);
