@@ -69,6 +69,8 @@ struct irc_conn {
 		time_t signon;
 	} whois;
 	GaimRoomlist *roomlist;
+
+	gboolean quitting;
 };
 
 struct irc_buddy {
@@ -85,9 +87,11 @@ gboolean irc_blist_timeout(struct irc_conn *irc);
 char *irc_mirc2html(const char *string);
 char *irc_mirc2txt(const char *string);
 
+gboolean irc_ischannel(const char *string);
+
+void irc_register_commands(void);
 void irc_msg_table_build(struct irc_conn *irc);
 void irc_parse_msg(struct irc_conn *irc, char *input);
-int irc_parse_cmd(struct irc_conn *irc, const char *target, const char *cmdstr);
 char *irc_parse_ctcp(struct irc_conn *irc, const char *from, const char *to, const char *msg, int notice);
 char *irc_format(struct irc_conn *irc, const char *format, ...);
 
@@ -134,7 +138,6 @@ void irc_cmd_table_build(struct irc_conn *irc);
 int irc_cmd_default(struct irc_conn *irc, const char *cmd, const char *target, const char **args);
 int irc_cmd_away(struct irc_conn *irc, const char *cmd, const char *target, const char **args);
 int irc_cmd_ctcp_action(struct irc_conn *irc, const char *cmd, const char *target, const char **args);
-int irc_cmd_help(struct irc_conn *irc, const char *cmd, const char *target, const char **args);
 int irc_cmd_invite(struct irc_conn *irc, const char *cmd, const char *target, const char **args);
 int irc_cmd_join(struct irc_conn *irc, const char *cmd, const char *target, const char **args);
 int irc_cmd_kick(struct irc_conn *irc, const char *cmd, const char *target, const char **args);
@@ -154,6 +157,6 @@ int irc_cmd_topic(struct irc_conn *irc, const char *cmd, const char *target, con
 int irc_cmd_wallops(struct irc_conn *irc, const char *cmd, const char *target, const char **args);
 int irc_cmd_whois(struct irc_conn *irc, const char *cmd, const char *target, const char **args);
 
-void irc_dccsend_send_ask(GaimConnection *gc, const char *dst);
+void irc_dccsend_send_file(GaimConnection *gc, const char *who, const char *file);
 void irc_dccsend_recv(struct irc_conn *irc, const char *from, const char *msg);
 #endif /* _GAIM_IRC_H */
