@@ -56,10 +56,9 @@ static char *irc_mask_userhost(const char *mask)
 
 static void irc_chat_remove_buddy(GaimConversation *convo, char *data[2])
 {
-	GList *users = gaim_conv_chat_get_users(GAIM_CONV_CHAT(convo));
 	char *message = g_strdup_printf("quit: %s", data[1]);
 
-	if (g_list_find_custom(users, data[0], (GCompareFunc)(strcmp)))
+	if (gaim_conv_chat_find_user(GAIM_CONV_CHAT(convo), data[0]))
 		gaim_conv_chat_remove_user(GAIM_CONV_CHAT(convo), data[0], message);
 
 	g_free(message);
@@ -180,33 +179,39 @@ void irc_msg_endwhois(struct irc_conn *irc, const char *name, const char *from,
 	}
 
 	info = g_string_new("");
-	g_string_append_printf(info, "<b>%s:</b> %s%s%s<br>", _("Nick"), args[1],
+	g_string_append_printf(info, _("<b>%s:</b> %s"), _("Nick"), args[1]);
+	g_string_append_printf(info, "%s%s<br>",
 			       irc->whois.ircop ? _(" <i>(ircop)</i>") : "",
 			       irc->whois.identified ? _(" <i>(identified)</i>") : "");
 	if (irc->whois.away) {
-		g_string_append_printf(info, "<b>%s:</b> %s<br>", _("Away"), irc->whois.away);
+		char *tmp = g_markup_escape_text(irc->whois.away, strlen(irc->whois.away));
 		g_free(irc->whois.away);
+		g_string_append_printf(info, _("<b>%s:</b> %s<br>"), _("Away"), tmp);
+		g_free(tmp);
 	}
 	if (irc->whois.userhost) {
-		g_string_append_printf(info, "<b>%s:</b> %s<br>", _("Username"), irc->whois.userhost);
-		g_string_append_printf(info, "<b>%s:</b> %s<br>", _("Realname"), irc->whois.name);
-		g_free(irc->whois.userhost);
+		char *tmp = g_markup_escape_text(irc->whois.name, strlen(irc->whois.name));
 		g_free(irc->whois.name);
+		g_string_append_printf(info, _("<b>%s:</b> %s<br>"), _("Username"), irc->whois.userhost);
+		g_string_append_printf(info, _("<b>%s:</b> %s<br>"), _("Realname"), tmp);
+		g_free(irc->whois.userhost);
+		g_free(tmp);
 	}
 	if (irc->whois.server) {
-		g_string_append_printf(info, "<b>%s:</b> %s (%s)<br>", _("Server"), irc->whois.server, irc->whois.serverinfo);
+		g_string_append_printf(info, _("<b>%s:</b> %s"), _("Server"), irc->whois.server);
+		g_string_append_printf(info, " (%s)<br>", irc->whois.serverinfo);
 		g_free(irc->whois.server);
 		g_free(irc->whois.serverinfo);
 	}
 	if (irc->whois.channels) {
-		g_string_append_printf(info, "<b>%s:</b> %s<br>", _("Currently on"), irc->whois.channels);
+		g_string_append_printf(info, _("<b>%s:</b> %s<br>"), _("Currently on"), irc->whois.channels);
 		g_free(irc->whois.channels);
 	}
 	if (irc->whois.idle) {
 		gchar *timex = gaim_str_seconds_to_string(irc->whois.idle);
 		g_string_append_printf(info, _("<b>Idle for:</b> %s<br>"), timex);
 		g_free(timex);
-		g_string_append_printf(info, "<b>%s:</b> %s", _("Online since"), ctime(&irc->whois.signon));
+		g_string_append_printf(info, _("<b>%s:</b> %s"), _("Online since"), ctime(&irc->whois.signon));
 	}
 	if (!strcmp(irc->whois.nick, "Paco-Paco")) {
 		g_string_append_printf(info, _("<br><b>Defining adjective:</b> Glorious<br>"));
@@ -217,8 +222,9 @@ void irc_msg_endwhois(struct irc_conn *irc, const char *name, const char *from,
 
 	g_snprintf(buffer, sizeof(buffer),
 			   _("Buddy Information for %s"), irc->whois.nick);
-	gaim_notify_formatted(gc, NULL, buffer, NULL, str, NULL, NULL);
+	gaim_notify_userinfo(gc, irc->whois.nick, NULL, buffer, NULL, str, NULL, NULL);
 
+	g_free(irc->whois.nick);
 	g_free(str);
 	memset(&irc->whois, 0, sizeof(irc->whois));
 }
@@ -255,7 +261,7 @@ void irc_msg_list(struct irc_conn *irc, const char *name, const char *from, char
 
 void irc_msg_topic(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
-	char *chan, *topic, *msg, *nick, *tmp;
+	char *chan, *topic, *msg, *nick, *tmp, *tmp2;
 	GaimConversation *convo;
 
 	if (!strcmp(name, "topic")) {
@@ -270,21 +276,25 @@ void irc_msg_topic(struct irc_conn *irc, const char *name, const char *from, cha
 	if (!convo) {
 		gaim_debug(GAIM_DEBUG_ERROR, "irc", "Got a topic for %s, which doesn't exist\n", chan);
 	}
-	gaim_conv_chat_set_topic(GAIM_CONV_CHAT(convo), NULL, topic);
+
 	/* If this is an interactive update, print it out */
 	tmp = gaim_escape_html(topic);
+	tmp2 = gaim_markup_linkify(tmp);
+	g_free(tmp);
 	if (!strcmp(name, "topic")) {
 		nick = irc_mask_nick(from);
-		msg = g_strdup_printf(_("%s has changed the topic to: %s"), nick, tmp);
+		gaim_conv_chat_set_topic(GAIM_CONV_CHAT(convo), nick, topic);
+		msg = g_strdup_printf(_("%s has changed the topic to: %s"), nick, tmp2);
 		g_free(nick);
 		gaim_conv_chat_write(GAIM_CONV_CHAT(convo), from, msg, GAIM_MESSAGE_SYSTEM, time(NULL));
 		g_free(msg);
 	} else {
-		msg = g_strdup_printf(_("The topic for %s is: %s"), chan, tmp);
+		msg = g_strdup_printf(_("The topic for %s is: %s"), chan, tmp2);
+		gaim_conv_chat_set_topic(GAIM_CONV_CHAT(convo), NULL, topic);
 		gaim_conv_chat_write(GAIM_CONV_CHAT(convo), "", msg, GAIM_MESSAGE_SYSTEM, time(NULL));
 		g_free(msg);
 	}
-	g_free(tmp);
+	g_free(tmp2);
 	g_free(topic);
 }
 
@@ -320,7 +330,7 @@ void irc_msg_names(struct irc_conn *irc, const char *name, const char *from, cha
 		names = cur = g_string_free(irc->names, FALSE);
 		irc->names = NULL;
 		if (irc->nameconv) {
-			msg = g_strdup_printf("Users on %s: %s", args[1], names);
+			msg = g_strdup_printf(_("Users on %s: %s"), args[1], names);
 			if (gaim_conversation_get_type(convo) == GAIM_CONV_CHAT)
 				gaim_conv_chat_write(GAIM_CONV_CHAT(convo), "", msg, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
 			else
@@ -330,15 +340,26 @@ void irc_msg_names(struct irc_conn *irc, const char *name, const char *from, cha
 			irc->nameconv = NULL;
 		} else {
 			GList *users = NULL;
+			GList *flags = NULL;
 
 			while (*cur) {
+				GaimConvChatBuddyFlags f = GAIM_CBFLAGS_NONE;
 				end = strchr(cur, ' ');
 				if (!end)
 					end = cur + strlen(cur);
-				if (*cur == '@' || *cur == '%' || *cur == '+')
+				if (*cur == '@') {
+					f = GAIM_CBFLAGS_OP;
 					cur++;
+				} else if (*cur == '%') {
+					f = GAIM_CBFLAGS_HALFOP;
+					cur++;
+				} else if(*cur == '+') {
+					f = GAIM_CBFLAGS_VOICE;
+					cur++;
+				}
 				tmp = g_strndup(cur, end - cur);
 				users = g_list_append(users, tmp);
+				flags = g_list_append(flags, GINT_TO_POINTER(f));
 				cur = end;
 				if (*cur)
 					cur++;
@@ -347,12 +368,13 @@ void irc_msg_names(struct irc_conn *irc, const char *name, const char *from, cha
 			if (users != NULL) {
 				GList *l;
 
-				gaim_conv_chat_add_users(GAIM_CONV_CHAT(convo), users);
+				gaim_conv_chat_add_users(GAIM_CONV_CHAT(convo), users, flags);
 
 				for (l = users; l != NULL; l = l->next)
 					g_free(l->data);
 
 				g_list_free(users);
+				g_list_free(flags);
 			}
 		}
 		g_free(names);
@@ -534,12 +556,12 @@ static void irc_buddy_status(char *name, struct irc_buddy *ib, struct irc_conn *
 		return;
 
 	if (ib->online && !ib->flag) {
-		serv_got_update(gc, buddy->name, 0, 0, 0, 0, 0);
+		serv_got_update(gc, buddy->name, FALSE, 0, 0, 0, 0);
 		ib->online = FALSE;
 	}
 
 	if (!ib->online && ib->flag) {
-		serv_got_update(gc, buddy->name, 1, 0, 0, 0, 0);
+		serv_got_update(gc, buddy->name, TRUE, 0, 0, 0, 0);
 		ib->online = TRUE;
 	}
 }
@@ -549,6 +571,7 @@ void irc_msg_join(struct irc_conn *irc, const char *name, const char *from, char
 	GaimConnection *gc = gaim_account_get_connection(irc->account);
 	GaimConversation *convo;
 	char *nick = irc_mask_nick(from), *userhost;
+	struct irc_buddy *ib;
 	static int id = 1;
 
 	if (!gc) {
@@ -571,7 +594,13 @@ void irc_msg_join(struct irc_conn *irc, const char *name, const char *from, char
 	}
 
 	userhost = irc_mask_userhost(from);
-	gaim_conv_chat_add_user(GAIM_CONV_CHAT(convo), nick, userhost);
+	gaim_conv_chat_add_user(GAIM_CONV_CHAT(convo), nick, userhost, GAIM_CBFLAGS_NONE, TRUE);
+
+	if ((ib = g_hash_table_lookup(irc->buddies, nick)) != NULL) {
+		ib->flag = TRUE;
+		irc_buddy_status(nick, ib, irc);
+	}
+
 	g_free(userhost);
 	g_free(nick);
 }
@@ -623,6 +652,45 @@ void irc_msg_mode(struct irc_conn *irc, const char *name, const char *from, char
 		buf = g_strdup_printf(_("mode (%s %s) by %s"), args[1], args[2] ? args[2] : "", nick);
 		gaim_conv_chat_write(GAIM_CONV_CHAT(convo), args[0], buf, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
 		g_free(buf);
+		if(args[2]) {
+			GaimConvChatBuddyFlags newflag, flags;
+			char *mcur, *cur, *end, *user;
+			gboolean add = FALSE;
+			mcur = args[1];
+			cur = args[2];
+			while (*cur && *mcur) {
+				if ((*mcur == '+') || (*mcur == '-')) {
+					add = (*mcur == '+') ? TRUE : FALSE;
+					mcur++;
+					continue;
+				}
+				end = strchr(cur, ' ');
+				if (!end)
+					end = cur + strlen(cur);
+				user = g_strndup(cur, end - cur);
+				flags = gaim_conv_chat_user_get_flags(GAIM_CONV_CHAT(convo), user);
+				newflag = GAIM_CBFLAGS_NONE;
+				if (*mcur == 'o')
+					newflag = GAIM_CBFLAGS_OP;
+				else if (*mcur =='h')
+					newflag = GAIM_CBFLAGS_HALFOP;
+				else if (*mcur == 'v')
+					newflag = GAIM_CBFLAGS_VOICE;
+				if (newflag) {
+					if (add)
+						flags |= newflag;
+					else
+						flags &= ~newflag;
+					gaim_conv_chat_user_set_flags(GAIM_CONV_CHAT(convo), user, flags);
+				}
+				g_free(user);
+				cur = end;
+				if (*cur)
+					cur++;
+				if (*mcur)
+					mcur++;
+			}
+		}
 	} else {					/* User		*/
 	}
 	g_free(nick);
@@ -646,18 +714,9 @@ void irc_msg_nick(struct irc_conn *irc, const char *name, const char *from, char
 
 	while (chats) {
 		GaimConvChat *chat = GAIM_CONV_CHAT(chats->data);
-		GList *users = gaim_conv_chat_get_users(chat);
-
-		while (users) {
-			char *user = users->data;
-
-			if (!strcmp(nick, user)) {
-				gaim_conv_chat_rename_user(chat, user, args[0]);
-				users = gaim_conv_chat_get_users(chat);
-				break;
-			}
-			users = users->next;
-		}
+		/* This is ugly ... */
+		if (gaim_conv_chat_find_user(chat, nick))
+			gaim_conv_chat_rename_user(chat, nick, args[0]);
 		chats = chats->next;
 	}
 	g_free(nick);
@@ -680,6 +739,7 @@ void irc_msg_nickused(struct irc_conn *irc, const char *name, const char *from,
 	buf = irc_format(irc, "vn", "NICK", newnick);
 	irc_send(irc, buf);
 	g_free(buf);
+	g_free(newnick);
 }
 
 void irc_msg_notice(struct irc_conn *irc, const char *name, const char *from, char **args)
@@ -846,6 +906,7 @@ void irc_msg_regonly(struct irc_conn *irc, const char *name, const char *from, c
 void irc_msg_quit(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
 	GaimConnection *gc = gaim_account_get_connection(irc->account);
+	struct irc_buddy *ib;
 	char *data[2];
 
 	if (!args || !args[0] || !gc)
@@ -855,6 +916,11 @@ void irc_msg_quit(struct irc_conn *irc, const char *name, const char *from, char
 	data[1] = args[0];
 	/* XXX this should have an API, I shouldn't grab this directly */
 	g_slist_foreach(gc->buddy_chats, (GFunc)irc_chat_remove_buddy, data);
+
+	if ((ib = g_hash_table_lookup(irc->buddies, data[0])) != NULL) {
+		ib->flag = FALSE;
+		irc_buddy_status(data[0], ib, irc);
+	}
 	g_free(data[0]);
 
 	return;
