@@ -553,6 +553,8 @@ faim_export int aim_clientready(aim_session_t *sess, aim_conn_t *conn);
 faim_export int aim_sendflapver(aim_session_t *sess, aim_conn_t *conn);
 faim_export int aim_request_login(aim_session_t *sess, aim_conn_t *conn, const char *sn);
 faim_export int aim_send_login(aim_session_t *, aim_conn_t *, const char *, const char *, struct client_info_s *, const char *key);
+/* 0x000b */ faim_export int aim_auth_securid_send(aim_session_t *sess, const char *securid);
+
 faim_export void aim_purge_rxqueue(aim_session_t *);
 faim_export void aim_cleansnacs(aim_session_t *, int maxage);
 
@@ -681,19 +683,21 @@ struct aim_chat_roominfo {
 	fu16_t instance;
 };
 
-#define AIM_IMFLAGS_AWAY		0x0001 /* mark as an autoreply */
-#define AIM_IMFLAGS_ACK			0x0002 /* request a receipt notice */
-#define AIM_IMFLAGS_UNICODE		0x0004
-#define AIM_IMFLAGS_ISO_8859_1		0x0008
-#define AIM_IMFLAGS_BUDDYREQ		0x0010 /* buddy icon requested */
-#define AIM_IMFLAGS_HASICON		0x0020 /* already has icon */
+#define AIM_IMFLAGS_AWAY				0x0001 /* mark as an autoreply */
+#define AIM_IMFLAGS_ACK					0x0002 /* request a receipt notice */
+#define AIM_IMFLAGS_BUDDYREQ			0x0010 /* buddy icon requested */
+#define AIM_IMFLAGS_HASICON				0x0020 /* already has icon */
 #define AIM_IMFLAGS_SUBENC_MACINTOSH	0x0040 /* damn that Steve Jobs! */
-#define AIM_IMFLAGS_CUSTOMFEATURES 	0x0080 /* features field present */
-#define AIM_IMFLAGS_EXTDATA		0x0100
-#define AIM_IMFLAGS_X			0x0200
-#define AIM_IMFLAGS_MULTIPART		0x0400 /* ->mpmsg section valid */
-#define AIM_IMFLAGS_OFFLINE		0x0800 /* send to offline user */
-#define AIM_IMFLAGS_TYPINGNOT		0x1000 /* typing notification */
+#define AIM_IMFLAGS_CUSTOMFEATURES 		0x0080 /* features field present */
+#define AIM_IMFLAGS_EXTDATA				0x0100
+#define AIM_IMFLAGS_X					0x0200
+#define AIM_IMFLAGS_MULTIPART			0x0400 /* ->mpmsg section valid */
+#define AIM_IMFLAGS_OFFLINE				0x0800 /* send to offline user */
+#define AIM_IMFLAGS_TYPINGNOT			0x1000 /* typing notification */
+
+#define AIM_CHARSET_ASCII		0x0000
+#define AIM_CHARSET_UNICODE	0x0002 /* UCS-2BE */
+#define AIM_CHARSET_CUSTOM	0x0003
 
 /*
  * Multipart message structures.
@@ -816,6 +820,7 @@ struct aim_incomingim_ch2_args {
 	fu16_t port;
 	fu16_t errorcode;
 	const char *msg; /* invite message or file description */
+	fu16_t msglen;
 	const char *encoding;
 	const char *language;
 	union {
@@ -1378,6 +1383,7 @@ typedef struct aim_tlvlist_s {
 
 /* TLV handling functions */
 faim_internal aim_tlv_t *aim_tlv_gettlv(aim_tlvlist_t *list, fu16_t type, const int nth);
+faim_internal int aim_tlv_getlength(aim_tlvlist_t *list, fu16_t type, const int nth);
 faim_internal char *aim_tlv_getstr(aim_tlvlist_t *list, const fu16_t type, const int nth);
 faim_internal fu8_t aim_tlv_get8(aim_tlvlist_t *list, const fu16_t type, const int nth);
 faim_internal fu16_t aim_tlv_get16(aim_tlvlist_t *list, const fu16_t type, const int nth);
