@@ -30,7 +30,7 @@ faim_internal int aim_bstream_curpos(aim_bstream_t *bs)
 	return bs->offset;
 }
 
-faim_internal int aim_bstream_setpos(aim_bstream_t *bs, int off)
+faim_internal int aim_bstream_setpos(aim_bstream_t *bs, unsigned int off)
 {
 
 	if (off > bs->len)
@@ -231,11 +231,6 @@ faim_internal char *aimbs_getstr(aim_bstream_t *bs, int len)
 		return NULL;
 	}
 
-	if (ob[0] == '\0') {
-		free(ob);
-		return NULL;
-	}
-	
 	ob[len] = '\0';
 
 	return ob;
