@@ -51,7 +51,7 @@ silc_channel_message(SilcClient client, SilcClientConnection conn,
 	GaimConnection *gc = client->application;
 	SilcGaim sg = gc->proto_data;
 	GaimConversation *convo = NULL;
-	char *msg;
+	char *msg, *tmp;
 
 	if (!message)
 		return;
@@ -85,16 +85,18 @@ silc_channel_message(SilcClient client, SilcClientConnection conn,
 	}
 
 	if (flags & SILC_MESSAGE_FLAG_ACTION) {
-		msg = g_strdup_printf("<I>%s</I> %s",
-				      sender->nickname ?
-				      sender->nickname : "<unknown>",
+		msg = g_strdup_printf("/me %s",
 				      (const char *)message);
 		if (!msg)
 			return;
 
+		tmp = gaim_escape_html(msg);
 		/* Send to Gaim */
-		gaim_conversation_write(convo, NULL, (const char *)msg,
-					GAIM_MESSAGE_SYSTEM, time(NULL));
+		serv_got_chat_in(gc, gaim_conv_chat_get_id(GAIM_CONV_CHAT(convo)),
+				 sender->nickname ?
+				 sender->nickname : "<unknown>", 0,
+				 tmp, time(NULL));
+		g_free(tmp);
 		g_free(msg);
 		return;
 	}
@@ -114,12 +116,15 @@ silc_channel_message(SilcClient client, SilcClientConnection conn,
 		return;
 	}
 
-	if (flags & SILC_MESSAGE_FLAG_UTF8)
+	if (flags & SILC_MESSAGE_FLAG_UTF8) {
+		tmp = gaim_escape_html((const char *)message);
 		/* Send to Gaim */
 		serv_got_chat_in(gc, gaim_conv_chat_get_id(GAIM_CONV_CHAT(convo)),
 				 sender->nickname ?
 				 sender->nickname : "<unknown>", 0,
-				 (const char *)message, time(NULL));
+				 tmp, time(NULL));
+		g_free(tmp);
+	}
 }
 
 
@@ -138,7 +143,7 @@ silc_private_message(SilcClient client, SilcClientConnection conn,
 	GaimConnection *gc = client->application;
 	SilcGaim sg = gc->proto_data;
 	GaimConversation *convo = NULL;
-	char *msg;
+	char *msg, *tmp;
 
 	if (!message)
 		return;
@@ -157,17 +162,18 @@ silc_private_message(SilcClient client, SilcClientConnection conn,
 	}
 
 	if (flags & SILC_MESSAGE_FLAG_ACTION && convo) {
-		msg = g_strdup_printf("<I>%s</I> %s",
-				      sender->nickname ?
-				      sender->nickname : "<unknown>",
+		msg = g_strdup_printf("/me %s",
 				      (const char *)message);
 		if (!msg)
 			return;
 
+		tmp = gaim_escape_html(msg);
 		/* Send to Gaim */
-		gaim_conversation_write(convo, NULL, (const char *)msg,
-					GAIM_MESSAGE_SYSTEM, time(NULL));
+		serv_got_im(gc, sender->nickname ?
+			    sender->nickname : "<unknown>",
+			    tmp, 0, time(NULL));
 		g_free(msg);
+		g_free(tmp);
 		return;
 	}
 
@@ -186,11 +192,14 @@ silc_private_message(SilcClient client, SilcClientConnection conn,
 		return;
 	}
 
-	if (flags & SILC_MESSAGE_FLAG_UTF8)
+	if (flags & SILC_MESSAGE_FLAG_UTF8) {
+		tmp = gaim_escape_html((const char *)message);
 		/* Send to Gaim */
 		serv_got_im(gc, sender->nickname ?
 			    sender->nickname : "<unknown>",
-			    (const char *)message, 0, time(NULL));
+			    tmp, 0, time(NULL));
+		g_free(tmp);
+	}
 }
 
 
@@ -241,8 +250,8 @@ silc_notify(SilcClient client, SilcClientConnection conn,
 			client_entry = va_arg(va, SilcClientEntry);
 
 			components = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
-			g_hash_table_insert(components, strdup("channel"), name);
-			serv_got_chat_invite(gc, name, client_entry->nickname, NULL, NULL);
+			g_hash_table_insert(components, strdup("channel"), strdup(name));
+			serv_got_chat_invite(gc, name, client_entry->nickname, NULL, components);
 		}
 		break;
 
@@ -263,7 +272,7 @@ silc_notify(SilcClient client, SilcClientConnection conn,
 		g_snprintf(buf, sizeof(buf), "%s@%s",
 			   client_entry->username, client_entry->hostname);
 		gaim_conv_chat_add_user(GAIM_CONV_CHAT(convo),
-					g_strdup(client_entry->nickname), buf);
+					g_strdup(client_entry->nickname), buf, GAIM_CBFLAGS_NONE, TRUE);
 
 		break;
 
@@ -305,46 +314,61 @@ silc_notify(SilcClient client, SilcClientConnection conn,
 		break;
 
 	case SILC_NOTIFY_TYPE_TOPIC_SET:
-		idtype = va_arg(va, int);
-		entry = va_arg(va, void *);
-		tmp = va_arg(va, char *);
-		channel = va_arg(va, SilcChannelEntry);
+		{
+			char *esc, *tmp2;
+			idtype = va_arg(va, int);
+			entry = va_arg(va, void *);
+			tmp = va_arg(va, char *);
+			channel = va_arg(va, SilcChannelEntry);
+
+			convo = gaim_find_conversation_with_account(channel->channel_name,
+					sg->account);
+			if (!convo)
+				break;
+
+			if (!tmp)
+				break;
+
+			esc = gaim_escape_html(tmp);
+			tmp2 = gaim_markup_linkify(esc);
+			g_free(esc);
+
+			if (idtype == SILC_ID_CLIENT) {
+				client_entry = (SilcClientEntry)entry;
+				g_snprintf(buf, sizeof(buf),
+						_("%s has changed the topic of <I>%s</I> to: %s"),
+						client_entry->nickname, channel->channel_name, tmp2);
+				gaim_conv_chat_write(GAIM_CONV_CHAT(convo), client_entry->nickname,
+						buf, GAIM_MESSAGE_SYSTEM, time(NULL));
+				gaim_conv_chat_set_topic(GAIM_CONV_CHAT(convo),
+						client_entry->nickname, tmp);
+			} else if (idtype == SILC_ID_SERVER) {
+				server_entry = (SilcServerEntry)entry;
+				g_snprintf(buf, sizeof(buf),
+						_("%s has changed the topic of <I>%s</I> to: %s"),
+						server_entry->server_name, channel->channel_name, tmp2);
+				gaim_conv_chat_write(GAIM_CONV_CHAT(convo), server_entry->server_name,
+						buf, GAIM_MESSAGE_SYSTEM, time(NULL));
+				gaim_conv_chat_set_topic(GAIM_CONV_CHAT(convo),
+						server_entry->server_name, tmp);
+			} else if (idtype == SILC_ID_CHANNEL) {
+				channel = (SilcChannelEntry)entry;
+				g_snprintf(buf, sizeof(buf),
+						_("%s has changed the topic of <I>%s</I> to: %s"),
+						channel->channel_name, channel->channel_name, tmp2);
+				gaim_conv_chat_write(GAIM_CONV_CHAT(convo), channel->channel_name,
+						buf, GAIM_MESSAGE_SYSTEM, time(NULL));
+				gaim_conv_chat_set_topic(GAIM_CONV_CHAT(convo),
+						channel->channel_name, tmp);
+			} else {
+				gaim_conv_chat_set_topic(GAIM_CONV_CHAT(convo), NULL, tmp);
+			}
+
+			g_free(tmp2);
 
-		convo = gaim_find_conversation_with_account(channel->channel_name,
-							    sg->account);
-		if (!convo)
 			break;
 
-		if (!tmp)
-			break;
-
-		if (idtype == SILC_ID_CLIENT) {
-			client_entry = (SilcClientEntry)entry;
-			g_snprintf(buf, sizeof(buf),
-				   _("%s has changed the topic of <I>%s</I> to: %s"),
-				   client_entry->nickname, channel->channel_name, tmp);
-			gaim_conv_chat_write(GAIM_CONV_CHAT(convo), client_entry->nickname,
-					     buf, GAIM_MESSAGE_SYSTEM, time(NULL));
-		} else if (idtype == SILC_ID_SERVER) {
-			server_entry = (SilcServerEntry)entry;
-			g_snprintf(buf, sizeof(buf),
-				   _("%s has changed the topic of <I>%s</I> to: %s"),
-				   server_entry->server_name, channel->channel_name, tmp);
-			gaim_conv_chat_write(GAIM_CONV_CHAT(convo), server_entry->server_name,
-					     buf, GAIM_MESSAGE_SYSTEM, time(NULL));
-		} else if (idtype == SILC_ID_CHANNEL) {
-			channel = (SilcChannelEntry)entry;
-			g_snprintf(buf, sizeof(buf),
-				   _("%s has changed the topic of <I>%s</I> to: %s"),
-				   channel->channel_name, channel->channel_name, tmp);
-			gaim_conv_chat_write(GAIM_CONV_CHAT(convo), channel->channel_name,
-					     buf, GAIM_MESSAGE_SYSTEM, time(NULL));
 		}
-
-		gaim_conv_chat_set_topic(GAIM_CONV_CHAT(convo), NULL, tmp);
-
-		break;
-
 	case SILC_NOTIFY_TYPE_NICK_CHANGE:
 		client_entry = va_arg(va, SilcClientEntry);
 		client_entry2 = va_arg(va, SilcClientEntry);
@@ -359,9 +383,10 @@ silc_notify(SilcClient client, SilcClientConnection conn,
 								    sg->account);
 			if (!convo)
 				continue;
-			gaim_conv_chat_rename_user(GAIM_CONV_CHAT(convo),
-						   client_entry->nickname,
-						   client_entry2->nickname);
+			if (gaim_conv_chat_find_user(GAIM_CONV_CHAT(convo), client_entry->nickname))
+				gaim_conv_chat_rename_user(GAIM_CONV_CHAT(convo),
+										   client_entry->nickname,
+										   client_entry2->nickname);
 		}
 		silc_hash_table_list_reset(&htl);
 
@@ -407,39 +432,47 @@ silc_notify(SilcClient client, SilcClientConnection conn,
 		break;
 
 	case SILC_NOTIFY_TYPE_CUMODE_CHANGE:
-		idtype = va_arg(va, int);
-		entry = va_arg(va, void *);
-		mode = va_arg(va, SilcUInt32);
-		client_entry2 = va_arg(va, SilcClientEntry);
-		channel = va_arg(va, SilcChannelEntry);
+		{
+			GaimConvChatBuddyFlags flags = GAIM_CBFLAGS_NONE;
+			idtype = va_arg(va, int);
+			entry = va_arg(va, void *);
+			mode = va_arg(va, SilcUInt32);
+			client_entry2 = va_arg(va, SilcClientEntry);
+			channel = va_arg(va, SilcChannelEntry);
 
-		convo = gaim_find_conversation_with_account(channel->channel_name,
-							    sg->account);
-		if (!convo)
+			convo = gaim_find_conversation_with_account(channel->channel_name,
+					sg->account);
+			if (!convo)
+				break;
+
+			if (idtype == SILC_ID_CLIENT)
+				name = ((SilcClientEntry)entry)->nickname;
+			else if (idtype == SILC_ID_SERVER)
+				name = ((SilcServerEntry)entry)->server_name;
+			else
+				name = ((SilcChannelEntry)entry)->channel_name;
+			if (!name)
+				break;
+
+			if (mode) {
+				silcgaim_get_chumode_string(mode, buf2, sizeof(buf2));
+				g_snprintf(buf, sizeof(buf),
+						_("<I>%s</I> set <I>%s's</I> modes to: %s"), name,
+						client_entry2->nickname, buf2);
+				if (mode & SILC_CHANNEL_UMODE_CHANFO)
+					flags |= GAIM_CBFLAGS_FOUNDER;
+				if (mode & SILC_CHANNEL_UMODE_CHANOP)
+					flags |= GAIM_CBFLAGS_OP;
+			} else {
+				g_snprintf(buf, sizeof(buf),
+						_("<I>%s</I> removed all <I>%s's</I> modes"), name,
+						client_entry2->nickname);
+			}
+			gaim_conv_chat_write(GAIM_CONV_CHAT(convo), channel->channel_name,
+					buf, GAIM_MESSAGE_SYSTEM, time(NULL));
+			gaim_conv_chat_user_set_flags(GAIM_CONV_CHAT(convo), client_entry2->nickname, flags);
 			break;
-
-		if (idtype == SILC_ID_CLIENT)
-			name = ((SilcClientEntry)entry)->nickname;
-		else if (idtype == SILC_ID_SERVER)
-			name = ((SilcServerEntry)entry)->server_name;
-		else
-			name = ((SilcChannelEntry)entry)->channel_name;
-		if (!name)
-			break;
-
-		if (mode) {
-			silcgaim_get_chumode_string(mode, buf2, sizeof(buf2));
-			g_snprintf(buf, sizeof(buf),
-				   _("<I>%s</I> set <I>%s's</I> modes to: %s"), name,
-				   client_entry2->nickname, buf2);
-		} else {
-			g_snprintf(buf, sizeof(buf),
-				   _("<I>%s</I> removed all <I>%s's</I> modes"), name,
-				   client_entry2->nickname);
 		}
-		gaim_conv_chat_write(GAIM_CONV_CHAT(convo), channel->channel_name,
-				     buf, GAIM_MESSAGE_SYSTEM, time(NULL));
-		break;
 
 	case SILC_NOTIFY_TYPE_MOTD:
 		tmp = va_arg(va, char *);
@@ -471,7 +504,7 @@ silc_notify(SilcClient client, SilcClientConnection conn,
 			/* Remove user from channel */
 			g_snprintf(buf, sizeof(buf), ("Kicked by %s (%s)"),
 				   client_entry2->nickname, tmp ? tmp : "");
-			gaim_conv_chat_rename_user(GAIM_CONV_CHAT(convo),
+			gaim_conv_chat_remove_user(GAIM_CONV_CHAT(convo),
 						   client_entry->nickname,
 						   buf);
 		}
@@ -653,7 +686,7 @@ silc_notify(SilcClient client, SilcClientConnection conn,
 				/* Find buddy by nickname */
 				b = gaim_find_buddy(sg->account, client_entry->nickname);
 				if (!b) {
-					fprintf(stderr, "WATCH for %s, unknown buddy",
+					gaim_debug_warning("silc", "WATCH for %s, unknown buddy",
 						client_entry->nickname);
 					break;
 				}
@@ -675,7 +708,7 @@ silc_notify(SilcClient client, SilcClientConnection conn,
 				     client_entry->mode & SILC_UMODE_PAGE ||
 				     client_entry->mode & SILC_UMODE_DETACHED)) {
 					client_entry->mode = mode;
-					gaim_blist_update_buddy_presence(b, GAIM_BUDDY_ONLINE);
+					gaim_blist_update_buddy_presence(b, TRUE);
 				}
 				else if ((mode & SILC_UMODE_GONE) ||
 					 (mode & SILC_UMODE_INDISPOSED) ||
@@ -683,21 +716,22 @@ silc_notify(SilcClient client, SilcClientConnection conn,
 					 (mode & SILC_UMODE_PAGE) ||
 					 (mode & SILC_UMODE_DETACHED)) {
 					client_entry->mode = mode;
-					gaim_blist_update_buddy_presence(b, GAIM_BUDDY_OFFLINE);
+					gaim_blist_update_buddy_presence(b, FALSE);
 				}
 			} else if (notify == SILC_NOTIFY_TYPE_SIGNOFF ||
 				   notify == SILC_NOTIFY_TYPE_SERVER_SIGNOFF ||
 				   notify == SILC_NOTIFY_TYPE_KILLED) {
 				client_entry->mode = mode;
-				gaim_blist_update_buddy_presence(b, GAIM_BUDDY_OFFLINE);
+				gaim_blist_update_buddy_presence(b, FALSE);
 			} else if (notify == SILC_NOTIFY_TYPE_NONE) {
 				client_entry->mode = mode;
-				gaim_blist_update_buddy_presence(b, GAIM_BUDDY_ONLINE);
+				gaim_blist_update_buddy_presence(b, TRUE);
 			}
 		}
 		break;
 
 	default:
+		gaim_debug_info("silc", "Unhandled notification: %d\n", type);
 		break;
 	}
 
@@ -901,13 +935,14 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
 			SilcUInt32 idle, mode;
 			SilcBuffer channels, user_modes;
 			SilcClientEntry client_entry;
-			char *buf, tmp[1024];
+			char *buf, tmp[1024], *tmp2;
+			char *moodstr, *statusstr, *contactstr, *langstr, *devicestr, *tzstr, *geostr;
 			GString *s;
 
 			if (!success) {
 				gaim_notify_error(gc, _("User Information"),
-						  _("Cannot get user information"),
-						  silc_get_status_message(status));
+						_("Cannot get user information"),
+						silc_get_status_message(status));
 				break;
 			}
 
@@ -924,38 +959,88 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
 			user_modes = va_arg(vp, SilcBuffer);
 
 			s = g_string_new("");
-			g_string_append_printf(s, "%s:\t\t%s\n", _("Nickname"), client_entry->nickname);
-			if (client_entry->realname)
-				g_string_append_printf(s, "%s:\t%s\n", _("Realname"), client_entry->realname);
-			if (client_entry->username)
-				g_string_append_printf(s, "%s:\t\t%s\n", _("Username"), client_entry->username);
-			if (client_entry->hostname)
-				g_string_append_printf(s, "%s:\t\t%s\n", _("Hostname"), client_entry->hostname);
-			if (client_entry->server)
-				g_string_append_printf(s, "%s:\t\t%s\n", _("Server"), client_entry->server);
-
-			if (mode) {
-				memset(tmp, 0, sizeof(tmp));
-				silcgaim_get_umode_string(mode, tmp, sizeof(tmp) - 1);
-				g_string_append_printf(s, "%s:\t%s\n", _("User Mode"), tmp);
+			tmp2 = gaim_escape_html(client_entry->nickname);
+			g_string_append_printf(s, "<b>%s:</b> %s", _("Nickname"), tmp2);
+			g_free(tmp2);
+			if (client_entry->realname) {
+				tmp2 = gaim_escape_html(client_entry->realname);
+				g_string_append_printf(s, "<br><b>%s:</b> %s", _("Realname"), tmp2);
+				g_free(tmp2);
 			}
+			if (client_entry->username) {
+				tmp2 = gaim_escape_html(client_entry->username);
+				if (client_entry->hostname)
+					g_string_append_printf(s, "<br><b>%s:</b> %s@%s", _("Username"), tmp2, client_entry->hostname);
+				else
+					g_string_append_printf(s, "<br><b>%s:</b> %s", _("Username"), tmp2);
+				g_free(tmp2);
+			}
+
+			if (client_entry->mode) {
+				g_string_append_printf(s, "<br><b>%s:</b> ", _("User Modes"));
+				memset(tmp, 0, sizeof(tmp));
+				silcgaim_get_umode_string(client_entry->mode,
+						tmp, sizeof(tmp) - strlen(tmp));
+				g_string_append_printf(s, "%s", tmp);
+			}
+
+			silcgaim_parse_attrs(client_entry->attrs, &moodstr, &statusstr, &contactstr, &langstr, &devicestr, &tzstr, &geostr);
+			if (moodstr) {
+				g_string_append_printf(s, "<br><b>%s:</b> %s", _("Mood"), moodstr);
+				g_free(moodstr);
+			}
+
+			if (statusstr) {
+				tmp2 = gaim_escape_html(statusstr);
+				g_string_append_printf(s, "<br><b>%s:</b> %s", _("Status Text"), tmp2);
+				g_free(statusstr);
+				g_free(tmp2);
+			}
+
+			if (contactstr) {
+				g_string_append_printf(s, "<br><b>%s:</b> %s", _("Preferred Contact"), contactstr);
+				g_free(contactstr);
+			}
+
+			if (langstr) {
+				g_string_append_printf(s, "<br><b>%s:</b> %s", _("Preferred Language"), langstr);
+				g_free(langstr);
+			}
+
+			if (devicestr) {
+				g_string_append_printf(s, "<br><b>%s:</b> %s", _("Device"), devicestr);
+				g_free(devicestr);
+			}
+
+			if (tzstr) {
+				g_string_append_printf(s, "<br><b>%s:</b> %s", _("Timezone"), tzstr);
+				g_free(tzstr);
+			}
+
+			if (geostr) {
+				g_string_append_printf(s, "<br><b>%s:</b> %s", _("Geolocation"), geostr);
+				g_free(geostr);
+			}
+
+			if (client_entry->server)
+				g_string_append_printf(s, "<br><b>%s:</b> %s", _("Server"), client_entry->server);
 
 			if (channels && user_modes) {
 				SilcUInt32 *umodes;
 				SilcDList list =
 					silc_channel_payload_parse_list(channels->data,
-									channels->len);
+							channels->len);
 				if (list && silc_get_mode_list(user_modes,
-							       silc_dlist_count(list),
-							       &umodes)) {
+							silc_dlist_count(list),
+							&umodes)) {
 					SilcChannelPayload entry;
 					int i = 0;
 
-					g_string_append_printf(s, "\n%s:\n", _("Channels"));
+					g_string_append_printf(s, "<br><b>%s:</b> ", _("Currently on"));
 					memset(tmp, 0, sizeof(tmp));
 					silc_dlist_start(list);
 					while ((entry = silc_dlist_get(list))
-					       != SILC_LIST_END) {
+							!= SILC_LIST_END) {
 						SilcUInt32 name_len;
 						char *m = silc_client_chumode_char(umodes[i++]);
 						char *name = silc_channel_get_name(entry, &name_len);
@@ -966,7 +1051,9 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
 						silc_free(m);
 
 					}
-					g_string_append_printf(s, "%s\n", tmp);
+					tmp2 = gaim_escape_html(tmp);
+					g_string_append_printf(s, "%s", tmp2);
+					g_free(tmp2);
 					silc_free(umodes);
 				}
 			}
@@ -978,8 +1065,8 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
 				pk = silc_pkcs_public_key_encode(client_entry->public_key, &pk_len);
 				fingerprint = silc_hash_fingerprint(NULL, pk, pk_len);
 				babbleprint = silc_hash_babbleprint(NULL, pk, pk_len);
-				g_string_append_printf(s, "\n%s:\n%s\n\n", _("Public Key Fingerprint"), fingerprint);
-				g_string_append_printf(s, "%s:\n%s", _("Public Key Babbleprint"), babbleprint);
+				g_string_append_printf(s, "<br><b>%s:</b><br>%s", _("Public Key Fingerprint"), fingerprint);
+				g_string_append_printf(s, "<br><b>%s:</b><br>%s", _("Public Key Babbleprint"), babbleprint);
 				silc_free(fingerprint);
 				silc_free(babbleprint);
 				silc_free(pk);
@@ -989,14 +1076,76 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
 #if 0 /* XXX for now, let's not show attrs here */
 			if (client_entry->attrs)
 				gaim_request_action(NULL, _("User Information"),
-						    _("User Information"),
-						    buf, 1, client_entry, 2,
-						    _("OK"), G_CALLBACK(silcgaim_whois_more),
-						    _("More..."), G_CALLBACK(silcgaim_whois_more));
+						_("User Information"),
+						buf, 1, client_entry, 2,
+						_("OK"), G_CALLBACK(silcgaim_whois_more),
+						_("More..."), G_CALLBACK(silcgaim_whois_more));
 			else
 #endif
-				gaim_notify_info(NULL, _("User Information"),
-						 _("User Information"), buf);
+				/* XXX this should use gaim_notify_userinfo, but it is unclear to me what should be passed for 'who' */
+				gaim_notify_formatted(gc, NULL, _("Buddy Information"), NULL, buf, NULL, NULL);
+			g_free(buf);
+		}
+		break;
+
+	case SILC_COMMAND_WHOWAS:
+		{
+			SilcClientEntry client_entry;
+			char *buf, *nickname, *realname, *username, *tmp;
+			GString *s;
+
+			if (!success) {
+				gaim_notify_error(gc, _("User Information"),
+						  _("Cannot get user information"),
+						  silc_get_status_message(status));
+				break;
+			}
+
+			client_entry = va_arg(vp, SilcClientEntry);
+			nickname = va_arg(vp, char *);
+			username = va_arg(vp, char *);
+			realname = va_arg(vp, char *);
+			if (!nickname)
+				break;
+
+			s = g_string_new("");
+			tmp = gaim_escape_html(nickname);
+			g_string_append_printf(s, "<b>%s:</b> %s", _("Nickname"), tmp);
+			g_free(tmp);
+			if (realname) {
+				tmp = gaim_escape_html(realname);
+				g_string_append_printf(s, "<br><b>%s:</b> %s", _("Realname"), tmp);
+				g_free(tmp);
+			}
+			if (username) {
+				tmp = gaim_escape_html(username);
+				if (client_entry && client_entry->hostname)
+					g_string_append_printf(s, "<br><b>%s:</b> %s@%s", _("Username"), tmp, client_entry->hostname);
+				else
+					g_string_append_printf(s, "<br><b>%s:</b> %s", _("Username"), tmp);
+				g_free(tmp);
+			}
+			if (client_entry && client_entry->server)
+				g_string_append_printf(s, "<br><b>%s:</b> %s", _("Server"), client_entry->server);
+
+
+			if (client_entry && client_entry->public_key) {
+				char *fingerprint, *babbleprint;
+				unsigned char *pk;
+				SilcUInt32 pk_len;
+				pk = silc_pkcs_public_key_encode(client_entry->public_key, &pk_len);
+				fingerprint = silc_hash_fingerprint(NULL, pk, pk_len);
+				babbleprint = silc_hash_babbleprint(NULL, pk, pk_len);
+				g_string_append_printf(s, "<br><b>%s:</b><br>%s", _("Public Key Fingerprint"), fingerprint);
+				g_string_append_printf(s, "<br><b>%s:</b><br>%s", _("Public Key Babbleprint"), babbleprint);
+				silc_free(fingerprint);
+				silc_free(babbleprint);
+				silc_free(pk);
+			}
+
+			buf = g_string_free(s, FALSE);
+			/* XXX this should use gaim_notify_userinfo, but it is unclear to me what should be passed for 'who' */
+			gaim_notify_formatted(gc, NULL, _("Buddy Information"), NULL, buf, NULL, NULL);
 			g_free(buf);
 		}
 		break;
@@ -1023,8 +1172,17 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
 
 			convo = gaim_find_conversation_with_account(channel->channel_name,
 								    sg->account);
-			if (!convo)
+			if (!convo) {
+				gaim_debug_error("silc", "Got a topic for %s, which doesn't exist\n",
+								 channel->channel_name);
 				break;
+			}
+
+			if (gaim_conversation_get_type(convo) != GAIM_CONV_CHAT) {
+				gaim_debug_error("silc", "Got a topic for %s, which isn't a chat\n",
+								 channel->channel_name);
+				break;
+			}
 
 			/* Set topic */
 			if (channel->topic)
@@ -1032,6 +1190,44 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
 		}
 		break;
 
+	case SILC_COMMAND_NICK:
+		{
+			/* I don't think we should need to do this because the server should
+			 * be sending a SILC_NOTIFY_TYPE_NICK_CHANGE when we change our own
+			 * nick, but it isn't, so we deal with it here instead. Stu. */
+			SilcClientEntry local_entry;
+			SilcHashTableList htl;
+			SilcChannelUser chu;
+			const char *oldnick;
+
+			if (!success) {
+				gaim_notify_error(gc, _("Nick"), _("Failed to change nickname"),
+						silc_get_status_message(status));
+				return;
+			}
+
+			local_entry = va_arg(vp, SilcClientEntry);
+
+			/* Change nick on all channels */
+			silc_hash_table_list(local_entry->channels, &htl);
+			while (silc_hash_table_get(&htl, NULL, (void *)&chu)) {
+				convo = gaim_find_conversation_with_account(chu->channel->channel_name,
+						sg->account);
+				if (!convo || (gaim_conversation_get_type(convo) != GAIM_CONV_CHAT))
+					continue;
+				oldnick = gaim_conv_chat_get_nick(GAIM_CONV_CHAT(convo));
+				if (strcmp(oldnick, local_entry->nickname)) {
+					gaim_conv_chat_rename_user(GAIM_CONV_CHAT(convo),
+							oldnick, local_entry->nickname);
+					gaim_conv_chat_set_nick(GAIM_CONV_CHAT(convo), local_entry->nickname);
+				}
+			}
+			silc_hash_table_list_reset(&htl);
+
+			gaim_connection_set_display_name(gc, local_entry->nickname);
+		}
+		break;
+
 	case SILC_COMMAND_LIST:
 		{
 			char *topic, *name;
@@ -1100,7 +1296,7 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
 			SilcServerEntry server_entry;
 			char *server_name;
 			char *server_info;
-			char tmp[256];
+			char tmp[256], *msg;
 
 			if (!success) {
 				gaim_notify_error(gc, _("Server Information"),
@@ -1116,12 +1312,98 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
 			if (server_name && server_info) {
 				g_snprintf(tmp, sizeof(tmp), "Server: %s\n%s",
 					   server_name, server_info);
-				gaim_notify_info(NULL, _("Server Information"),
-						 _("Server Information"), tmp);
+				msg = g_markup_escape_text(tmp, strlen(tmp));
+				gaim_notify_info(gc, NULL, _("Server Information"), msg);
+				g_free(msg);
 			}
 		}
 		break;
 
+	case SILC_COMMAND_STATS:
+		{
+			SilcUInt32 starttime, uptime, my_clients, my_channels, my_server_ops,
+			my_router_ops, cell_clients, cell_channels, cell_servers,
+			clients, channels, servers, routers, server_ops, router_ops;
+			SilcUInt32 buffer_length;
+			SilcBufferStruct buf;
+
+			unsigned char *server_stats;
+			char *msg;
+
+			if (!success) {
+				gaim_notify_error(gc, _("Server Statistics"),
+						_("Cannot get server statistics"),
+						silc_get_status_message(status));
+				return;
+			}
+
+			server_stats = va_arg(vp, unsigned char *);
+			buffer_length = va_arg(vp, SilcUInt32);
+			if (!server_stats || !buffer_length) {
+				gaim_notify_error(gc, _("Server Statistics"),
+						_("No server statistics available"), NULL);
+				break;
+			}
+			silc_buffer_set(&buf, server_stats, buffer_length);
+			silc_buffer_unformat(&buf,
+					SILC_STR_UI_INT(&starttime),
+					SILC_STR_UI_INT(&uptime),
+					SILC_STR_UI_INT(&my_clients),
+					SILC_STR_UI_INT(&my_channels),
+					SILC_STR_UI_INT(&my_server_ops),
+					SILC_STR_UI_INT(&my_router_ops),
+					SILC_STR_UI_INT(&cell_clients),
+					SILC_STR_UI_INT(&cell_channels),
+					SILC_STR_UI_INT(&cell_servers),
+					SILC_STR_UI_INT(&clients),
+					SILC_STR_UI_INT(&channels),
+					SILC_STR_UI_INT(&servers),
+					SILC_STR_UI_INT(&routers),
+					SILC_STR_UI_INT(&server_ops),
+					SILC_STR_UI_INT(&router_ops),
+					SILC_STR_END);
+
+			msg = g_strdup_printf(_("Local server start time: %s\n"
+					"Local server uptime: %s\n"
+					"Local server clients: %d\n"
+					"Local server channels: %d\n"
+					"Local server operators: %d\n"
+					"Local router operators: %d\n"
+					"Local cell clients: %d\n"
+					"Local cell channels: %d\n"
+					"Local cell servers: %d\n"
+					"Total clients: %d\n"
+					"Total channels: %d\n"
+					"Total servers: %d\n"
+					"Total routers: %d\n"
+					"Total server operators: %d\n"
+					"Total router operators: %d\n"),
+					silc_get_time(starttime),
+					gaim_str_seconds_to_string((int)uptime),
+					(int)my_clients, (int)my_channels, (int)my_server_ops, (int)my_router_ops,
+					(int)cell_clients, (int)cell_channels, (int)cell_servers,
+					(int)clients, (int)channels, (int)servers, (int)routers,
+					(int)server_ops, (int)router_ops);
+
+			gaim_notify_info(gc, NULL,
+					_("Network Statistics"), msg);
+			g_free(msg);
+		}
+		break;
+
+	case SILC_COMMAND_PING:
+		{
+			if (!success) {
+				gaim_notify_error(gc, _("Ping"), _("Ping failed"),
+								  silc_get_status_message(status));
+				return;
+			}
+
+			gaim_notify_info(gc, _("Ping"), _("Ping reply received from server"),
+							 NULL);
+		}
+		break;
+
 	case SILC_COMMAND_KILL:
 		if (!success) {
 			gaim_notify_error(gc, _("Kill User"),
@@ -1150,6 +1432,11 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
 		break;
 
 	default:
+		if (success)
+			gaim_debug_info("silc", "Unhandled command: %d (succeeded)\n", command);
+		else
+			gaim_debug_info("silc", "Unhandled command: %d (failed: %s)\n", command,
+							silc_get_status_message(status));
 		break;
 	}
 
@@ -1159,7 +1446,7 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
 
 /* Called to indicate that connection was either successfully established
    or connecting failed.  This is also the first time application receives
-   the SilcClientConnection objecet which it should save somewhere.
+   the SilcClientConnection object which it should save somewhere.
    If the `success' is FALSE the application must always call the function
    silc_client_close_connection. */
 
@@ -1191,16 +1478,15 @@ silc_connected(SilcClient client, SilcClientConnection conn,
 		if (reject_watch || block_invites || block_ims) {
 			char m[5];
 			g_snprintf(m, sizeof(m), "+%s%s%s",
-				   reject_watch ? "w" : "",
-				   block_invites ? "I" : "",
-				   block_ims ? "P" : "");
+					   reject_watch ? "w" : "",
+					   block_invites ? "I" : "",
+					   block_ims ? "P" : "");
 			silc_client_command_call(sg->client, sg->conn, NULL,
-						 "UMODE", m, NULL);
+					"UMODE", m, NULL);
 		}
 
 		return;
 		break;
-
 	case SILC_CLIENT_CONN_ERROR:
 		gaim_connection_error(gc, _("Error during connecting to SILC Server"));
 		unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
@@ -1317,15 +1603,15 @@ silc_get_auth_method(SilcClient client, SilcClientConnection conn,
 
 	/* Check configuration if we have this connection configured.  If we
 	   have then return that data immediately, as it's faster way. */
+	if (gaim_account_get_bool(sg->account, "pubkey-auth", FALSE)) {
+		completion(TRUE, SILC_AUTH_PUBLIC_KEY, NULL, 0, context);
+		return;
+	}
 	if (gc->account->password && *gc->account->password) {
 		completion(TRUE, SILC_AUTH_PASSWORD, gc->account->password,
 			   strlen(gc->account->password), context);
 		return;
 	}
-	if (gaim_account_get_bool(sg->account, "pubkey-auth", FALSE)) {
-		completion(TRUE, SILC_AUTH_PUBLIC_KEY, NULL, 0, context);
-		return;
-	}
 
 	/* Resolve the authentication method from server, as we may not know it. */
 	internal = silc_calloc(1, sizeof(*internal));
