@@ -24,19 +24,12 @@
 #include "accountopt.h"
 #include "conversation.h"
 #include "debug.h"
-#include "prpl.h"
-#include "multi.h"
 #include "notify.h"
 #include "proxy.h"
+#include "prpl.h"
 #include "request.h"
 #include "util.h"
-#include "html.h"
-
-/* XXX */
-#include "gtkinternal.h"
-#include "gaim.h"
-#include "ui.h"
-#include "gtkutils.h"
+#include "version.h"
 
 static GaimPlugin *my_protocol = NULL;
 
@@ -122,41 +115,10 @@ struct signon {
 #define USEROPT_AUTH      0
 #define USEROPT_AUTHPORT  1
 
-#define TOC_CONNECT_STEPS 4
+#define TOC_CONNECT_STEPS 3
 
 static void toc_login_callback(gpointer, gint, GaimInputCondition);
 static void toc_callback(gpointer, gint, GaimInputCondition);
-static void accept_file_dialog(struct ft_request *);
-
-/* The following were added for win32 port - Herman */
-
-int toc_write(int fd, const void *buffer, int len)
-{
-#ifndef _WIN32
-	return write(fd, buffer, len);
-#else
-	return send(fd, buffer, len, 0);
-#endif
-}
-
-int toc_read(int fd, void *buffer, int size)
-{
-#ifndef _WIN32
-	return read(fd, buffer, size);
-#else
-	return recv(fd, buffer, size, 0);
-#endif
-}
-
-int toc_soc_close( int fd )
-{
-#ifndef _WIN32
-	return close(fd);
-#else
-	return closesocket(fd);
-#endif
-}
-
 
 /* ok. this function used to take username/password, and return 0 on success.
  * now, it takes username/password, and returns NULL on error or a new gaim_connection
@@ -174,7 +136,7 @@ static void toc_login(GaimAccount *account)
 
 	g_snprintf(buf, sizeof buf, _("Looking up %s"),
 			gaim_account_get_string(account, "server", TOC_HOST));
-	gaim_connection_update_progress(gc, buf, 1, TOC_CONNECT_STEPS);
+	gaim_connection_update_progress(gc, buf, 0, TOC_CONNECT_STEPS);
 
 	gaim_debug(GAIM_DEBUG_INFO, "toc", "Client connects to TOC\n");
 	if (gaim_proxy_connect(account,
@@ -197,7 +159,7 @@ static void toc_login_callback(gpointer data, gint source, GaimInputCondition co
 	socklen_t namelen;
 
 	if (!g_list_find(gaim_connections_get_all(), data)) {
-		toc_soc_close(source);
+		close(source);
 		return;
 	}
 
@@ -222,7 +184,7 @@ static void toc_login_callback(gpointer data, gint source, GaimInputCondition co
 
 	gaim_debug(GAIM_DEBUG_INFO, "toc",
 			   "Client sends \"FLAPON\\r\\n\\r\\n\"\n");
-	if (toc_write(tdt->toc_fd, FLAPON, strlen(FLAPON)) < 0) {
+	if (write(tdt->toc_fd, FLAPON, strlen(FLAPON)) < 0) {
 		gaim_connection_error(gc, _("Disconnected."));
 		return;
 	}
@@ -234,7 +196,7 @@ static void toc_login_callback(gpointer data, gint source, GaimInputCondition co
 	gc->inpa = gaim_input_add(tdt->toc_fd, GAIM_INPUT_READ, toc_callback, gc);
 
 	g_snprintf(buf, sizeof(buf), _("Signon: %s"), gaim_account_get_username(gc->account));
-	gaim_connection_update_progress(gc, buf, 2, TOC_CONNECT_STEPS);
+	gaim_connection_update_progress(gc, buf, 1, TOC_CONNECT_STEPS);
 }
 
 static void toc_close(GaimConnection *gc)
@@ -242,15 +204,15 @@ static void toc_close(GaimConnection *gc)
 	if (gc->inpa > 0)
 		gaim_input_remove(gc->inpa);
 	gc->inpa = 0;
-	toc_soc_close(((struct toc_data *)gc->proto_data)->toc_fd);
+	close(((struct toc_data *)gc->proto_data)->toc_fd);
 	g_free(gc->proto_data);
 }
 
 static void toc_build_config(GaimAccount *account, char *s, int len, gboolean show)
 {
-	GaimBlistNode *gnode,*bnode;
-	struct group *g;
-	struct buddy *b;
+	GaimBlistNode *gnode, *cnode, *bnode;
+	GaimGroup *g;
+	GaimBuddy *b;
 	GSList *plist = account->permit;
 	GSList *dlist = account->deny;
 
@@ -261,19 +223,24 @@ static void toc_build_config(GaimAccount *account, char *s, int len, gboolean sh
 
 	pos += g_snprintf(&s[pos], len - pos, "m %d\n", account->perm_deny);
 	for(gnode = gaim_get_blist()->root; gnode && len > pos; gnode = gnode->next) {
-		g = (struct group *)gnode;
+		g = (GaimGroup *)gnode;
 		if(!GAIM_BLIST_NODE_IS_GROUP(gnode))
 			continue;
 		if(gaim_group_on_account(g, account)) {
 			pos += g_snprintf(&s[pos], len - pos, "g %s\n", g->name);
-			for(bnode = gnode->child; bnode && len > pos; bnode = bnode->next) {
-				b = (struct buddy *)bnode;
-				if(!GAIM_BLIST_NODE_IS_BUDDY(bnode))
+			for(cnode = gnode->child; cnode; cnode = cnode->next) {
+				if(!GAIM_BLIST_NODE_IS_CONTACT(cnode))
 					continue;
-				if(b->account == account) {
-					pos += g_snprintf(&s[pos], len - pos, "b %s%s%s\n", b->name,
-							(show && b->alias) ? ":" : "",
-							(show && b->alias) ? b->alias : "");
+				for(bnode = gnode->child; bnode && len > pos; bnode = bnode->next) {
+					b = (GaimBuddy *)bnode;
+					if(!GAIM_BLIST_NODE_IS_BUDDY(bnode))
+						continue;
+					if(b->account == account) {
+						pos += g_snprintf(&s[pos], len - pos, "b %s%s%s\n",
+								b->name,
+								(show && b->alias) ? ":" : "",
+								(show && b->alias) ? b->alias : "");
+					}
 				}
 			}
 		}
@@ -312,7 +279,7 @@ char *escape_message(const char *msg)
 		}
 
 	/* Allocate a string */
-	ret = (char *)malloc((j+1) * sizeof(char));
+	ret = (char *)g_malloc((j+1) * sizeof(char));
 
 	/* Copy the string */
 	for (i=0, j=0; msg[i]; i++)
@@ -437,9 +404,9 @@ static int sflap_send(GaimConnection *gc, const char *buf, int olen, int type)
 		slen += 1;
 	}
 
-	ret = toc_write(tdt->toc_fd, obuf, slen);
+	ret = write(tdt->toc_fd, obuf, slen);
 	free(obuf);
-	free(escaped);
+	g_free(escaped);
 
 	return ret;
 }
@@ -450,7 +417,7 @@ static int wait_reply(GaimConnection *gc, char *buffer, size_t buflen)
 	struct sflap_hdr *hdr;
 	int ret;
 
-	if (toc_read(tdt->toc_fd, buffer, sizeof(struct sflap_hdr)) < 0) {
+	if (read(tdt->toc_fd, buffer, sizeof(struct sflap_hdr)) < 0) {
 		gaim_debug(GAIM_DEBUG_ERROR, "toc", "Couldn't read flap header\n");
 		return -1;
 	}
@@ -470,7 +437,7 @@ static int wait_reply(GaimConnection *gc, char *buffer, size_t buflen)
 		ret = 0;
 		do {
 			count += ret;
-			ret = toc_read(tdt->toc_fd,
+			ret = read(tdt->toc_fd,
 				   buffer + sizeof(struct sflap_hdr) + count, ntohs(hdr->len) - count);
 		} while (count + ret < ntohs(hdr->len) && ret > 0);
 		buffer[sizeof(struct sflap_hdr) + count + ret] = '\0';
@@ -493,12 +460,13 @@ static unsigned char *roast_password(const char *pass)
 	return rp;
 }
 
-static void toc_got_info(gpointer data, char *url_text, unsigned long len)
+static void toc_got_info(void *data, const char *url_text, size_t len)
 {
 	if (!url_text)
 		return;
 
-	g_show_info_text(NULL, NULL, 2, url_text, NULL);
+	gaim_notify_formatted(data, NULL, _("Buddy Information"), NULL,
+						  url_text, NULL, NULL);
 }
 
 static char *show_error_message()
@@ -593,14 +561,15 @@ static char *show_error_message()
 static void toc_callback(gpointer data, gint source, GaimInputCondition condition)
 {
 	GaimConnection *gc = (GaimConnection *)data;
+	GaimAccount *account = gaim_connection_get_account(gc);
 	struct toc_data *tdt = (struct toc_data *)gc->proto_data;
 	struct sflap_hdr *hdr;
 	struct signon so;
 	char buf[8 * 1024], *c;
 	char snd[BUF_LEN * 2];
-
-	const char *username = gaim_account_get_username(gc->account);
+	const char *username = gaim_account_get_username(account);
 	char *password;
+	GaimBuddy *buddy;
 
 	/* there's data waiting to be read, so read it. */
 	if (wait_reply(gc, buf, 8 * 1024) <= 0) {
@@ -636,7 +605,7 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 		else
 			password = g_strdup(gaim_account_get_password(gc->account));
 		g_snprintf(snd, sizeof snd, "toc_signon %s %d  %s %s %s \"%s\"",
-			   AUTH_HOST, AUTH_PORT, normalize(username),
+			   AUTH_HOST, AUTH_PORT, gaim_normalize(gc->account, username),
 			   roast_password(password), LANGUAGE, REVISION);
 		g_free(password);
 		if (sflap_send(gc, snd, -1, TYPE_DATA) < 0) {
@@ -644,7 +613,7 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 			return;
 		}
 
-		gaim_connection_update_progress(gc, _("Waiting for reply..."), 3, TOC_CONNECT_STEPS);
+		gaim_connection_update_progress(gc, _("Waiting for reply..."), 2, TOC_CONNECT_STEPS);
 		return;
 	}
 
@@ -674,6 +643,15 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 		gaim_connection_set_state(gc, GAIM_CONNECTED);
 		serv_finish_login(gc);
 
+		/*
+		 * Add me to my buddy list so that we know the time when
+		 * the server thinks I signed on.
+		 */
+		buddy = gaim_buddy_new(account, username, NULL);
+		/* XXX - Pick a group to add to */
+		/* gaim_blist_add(buddy, NULL, g, NULL); */
+		serv_add_buddy(gc, buddy);
+
 		/* Client sends TOC toc_init_done message */
 		gaim_debug(GAIM_DEBUG_INFO, "toc",
 				   "Client sends TOC toc_init_done message\n");
@@ -704,7 +682,7 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 			tdt->state = STATE_ONLINE;
 			g_snprintf(snd, sizeof snd, "toc_signon %s %d %s %s %s \"%s\"",
 				   AUTH_HOST, AUTH_PORT,
-				   normalize(gaim_account_get_username(gc->account)),
+				   gaim_normalize(gc->account, gaim_account_get_username(gc->account)),
 				   roast_password(gaim_account_get_password(gc->account)),
 				   LANGUAGE, REVISION);
 			if (sflap_send(gc, snd, -1, TYPE_DATA) < 0) {
@@ -719,7 +697,7 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 		}
 	} else if (!g_ascii_strcasecmp(c, "CONFIG")) {
 		c = strtok(NULL, ":");
-		parse_toc_buddy_list(gc->account, c);
+		gaim_blist_parse_toc_buddy_list(gc->account, c);
 	} else if (!g_ascii_strcasecmp(c, "NICK")) {
 		/* ignore NICK so that things get imported/exported properly
 		c = strtok(NULL, ":");
@@ -737,12 +715,13 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 			message++;
 		message++;
 
-		a = (away && (*away == 'T')) ? GAIM_IM_AUTO_RESP : 0;
+		a = (away && (*away == 'T')) ? GAIM_CONV_IM_AUTO_RESP : 0;
 
-		serv_got_im(gc, c, message, a, time(NULL), -1);
+		serv_got_im(gc, c, message, a, time(NULL));
 	} else if (!g_ascii_strcasecmp(c, "UPDATE_BUDDY")) {
 		char *l, *uc, *tmp;
-		int logged, evil, idle, type = 0;
+		gboolean logged_in;
+		int evil, idle, type = 0;
 		time_t signon, time_idle;
 
 		c = strtok(NULL, ":");	/* name */
@@ -752,7 +731,7 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 		sscanf(strtok(NULL, ":"), "%d", &idle);
 		uc = strtok(NULL, ":");
 
-		logged = (l && (*l == 'T')) ? 1 : 0;
+		logged_in = (l && (*l == 'T')) ? TRUE : FALSE;
 
 		if (uc[0] == 'A')
 			type |= UC_AOL;
@@ -781,12 +760,19 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 		} else
 			time_idle = 0;
 
-		tmp = g_strdup(normalize(gaim_account_get_username(gc->account)));
-		if (!strcmp(tmp, normalize(c)))
+		/*
+		 * If we have info for ourselves then set our display name, warning
+		 * level and official time of login.
+		 */
+		tmp = g_strdup(gaim_normalize(gc->account, gaim_account_get_username(gc->account)));
+		if (!strcmp(tmp, gaim_normalize(gc->account, c))) {
 			gaim_connection_set_display_name(gc, c);
+			gc->evil = evil;
+			gc->login_time_official = signon;
+		}
 		g_free(tmp);
 
-		serv_got_update(gc, c, logged, evil, signon, time_idle, type);
+		serv_got_update(gc, c, logged_in, evil, signon, time_idle, type);
 	} else if (!g_ascii_strcasecmp(c, "ERROR")) {
 		gaim_notify_error(gc, NULL, show_error_message(), NULL);
 	} else if (!g_ascii_strcasecmp(c, "EVILED")) {
@@ -806,7 +792,8 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 
 		serv_got_joined_chat(gc, id, name);
 	} else if (!g_ascii_strcasecmp(c, "CHAT_IN")) {
-		int id, w;
+		int id;
+		GaimConvChatFlags flags;
 		char *m, *who, *whisper;
 
 		sscanf(strtok(NULL, ":"), "%d", &id);
@@ -817,24 +804,24 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 			m++;
 		m++;
 
-		w = (whisper && (*whisper == 'T')) ? 1 : 0;
+		flags = (whisper && (*whisper == 'T')) ? GAIM_CONV_CHAT_WHISPER : 0;
 
-		serv_got_chat_in(gc, id, who, w, m, time((time_t)NULL));
+		serv_got_chat_in(gc, id, who, flags, m, time((time_t)NULL));
 	} else if (!g_ascii_strcasecmp(c, "CHAT_UPDATE_BUDDY")) {
 		int id;
 		char *in, *buddy;
 		GSList *bcs = gc->buddy_chats;
 		GaimConversation *b = NULL;
-		GaimChat *chat;
+		GaimConvChat *chat;
 
 		sscanf(strtok(NULL, ":"), "%d", &id);
 		in = strtok(NULL, ":");
 
-		chat = GAIM_CHAT(b);
+		chat = GAIM_CONV_CHAT(b);
 
 		while (bcs) {
 			b = (GaimConversation *)bcs->data;
-			if (id == gaim_chat_get_id(chat))
+			if (id == gaim_conv_chat_get_id(chat))
 				break;
 			bcs = bcs->next;
 			b = NULL;
@@ -845,10 +832,10 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 
 		if (in && (*in == 'T'))
 			while ((buddy = strtok(NULL, ":")) != NULL)
-				gaim_chat_add_user(chat, buddy, NULL);
+				gaim_conv_chat_add_user(chat, buddy, NULL, GAIM_CBFLAGS_NONE, TRUE);
 		else
 			while ((buddy = strtok(NULL, ":")) != NULL)
-				gaim_chat_remove_user(chat, buddy, NULL);
+				gaim_conv_chat_remove_user(chat, buddy, NULL);
 	} else if (!g_ascii_strcasecmp(c, "CHAT_INVITE")) {
 		char *name, *who, *message;
 		int id;
@@ -872,7 +859,7 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 
 		while (bcs) {
 			b = (GaimConversation *)bcs->data;
-			if (id == gaim_chat_get_id(GAIM_CHAT(b)))
+			if (id == gaim_conv_chat_get_id(GAIM_CONV_CHAT(b)))
 				break;
 			b = NULL;
 			bcs = bcs->next;
@@ -898,7 +885,7 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 		g_snprintf(tmp, sizeof(tmp), "http://%s:%d/%s", tdt->toc_ip,
 				gaim_account_get_int(gc->account, "port", TOC_PORT),
 				url);
-		grab_url(tmp, FALSE, toc_got_info, NULL, NULL, 0);
+		gaim_url_fetch(tmp, FALSE, NULL, FALSE, toc_got_info, gc);
 	} else if (!g_ascii_strcasecmp(c, "DIR_STATUS")) {
 	} else if (!g_ascii_strcasecmp(c, "ADMIN_NICK_STATUS")) {
 	} else if (!g_ascii_strcasecmp(c, "ADMIN_PASSWD_STATUS")) {
@@ -913,6 +900,7 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 							  "going through. This is only temporary, please "
 							  "be patient."));
 	} else if (!g_ascii_strcasecmp(c, "RVOUS_PROPOSE")) {
+#if 0
 		char *user, *uuid, *cookie;
 		int seq;
 		char *rip, *pip, *vip, *trillian = NULL;
@@ -1026,7 +1014,6 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 		} else if (!strcmp(uuid, VOICE_UID)) {
 			/* oh goody. voice over ip. fun stuff. */
 		} else if (!strcmp(uuid, B_ICON_UID)) {
-			/*
 			int unk[4], i;
 			char *messages[4];
 			struct buddy_icon *icon;
@@ -1044,7 +1031,6 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 			g_free(icon);
 			for (i--; i >= 0; i--)
 				g_free(messages[i]);
-			*/
 		} else if (!strcmp(uuid, IMAGE_UID)) {
 			/* aka Direct IM */
 		} else {
@@ -1052,28 +1038,53 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 					   "Don't know what to do with RVOUS UUID %s\n", uuid);
 			/* do we have to do anything here? i think it just times out */
 		}
+#endif
 	} else {
 		gaim_debug(GAIM_DEBUG_ERROR, "toc",
 				   "don't know what to do with %s\n", c);
 	}
 }
 
-static int toc_send_im(GaimConnection *gc, const char *name, const char *message, int len, GaimImFlags flags)
+static int toc_send_im(GaimConnection *gc, const char *name, const char *message, GaimConvImFlags flags)
 {
 	char *buf1, *buf2;
 
-gaim_debug(GAIM_DEBUG_ERROR, "xxx", "1 - Sending message %s\n", message);
+#if 1
+	/* This is the old, non-i18n way */
 	buf1 = escape_text(message);
 	if (strlen(buf1) + 52 > MSG_LEN) {
 		g_free(buf1);
 		return -E2BIG;
 	}
-
-gaim_debug(GAIM_DEBUG_ERROR, "xxx", "1 - Sending message %s\n", buf1);
-	buf2 = g_strdup_printf("toc_send_im %s \"%s\"%s", normalize(name), buf1, 
-						   ((flags & GAIM_IM_AUTO_RESP) ? " auto" : ""));
+	buf2 = g_strdup_printf("toc_send_im %s \"%s\"%s", gaim_normalize(gc->account, name), buf1,
+						   ((flags & GAIM_CONV_IM_AUTO_RESP) ? " auto" : ""));
 	g_free(buf1);
-gaim_debug(GAIM_DEBUG_ERROR, "xxx", "1 - Sending message %s\n", buf2);
+#else
+	/* This doesn't work yet.  See the comments below for details */
+	buf1 = gaim_strreplace(message, "\"", "\\\"");
+
+	/*
+	 * We still need to determine what encoding should be used and send the 
+	 * message in that encoding.  This should be done the same as in 
+	 * oscar_encoding_check() in oscar.c.  There is no encoding flag sent 
+	 * along with the message--the TOC to OSCAR proxy server must just 
+	 * use a lil' algorithm to determine what the actual encoding is.
+	 *
+	 * After that, you need to convert buf1 to that encoding, and keep track 
+	 * of the length of the resulting string.  Then you need to make sure 
+	 * that length is passed to sflap_send().
+	 */
+
+	if (strlen(buf1) + 52 > MSG_LEN) {
+		g_free(buf1);
+		return -E2BIG;
+	}
+
+	buf2 = g_strdup_printf("toc2_send_im_enc %s F U en \"%s\" %s", gaim_normalize(gc->account, name), buf1, 
+						   ((flags & GAIM_CONV_IM_AUTO_RESP) ? "auto" : ""));
+	g_free(buf1);
+#endif
+
 	sflap_send(gc, buf2, -1, TYPE_DATA);
 	g_free(buf2);
 
@@ -1089,20 +1100,32 @@ static void toc_set_config(GaimConnection *gc)
 	g_free(buf);
 }
 
-static void toc_get_info(GaimConnection *g, const char *name)
+static void toc_get_info(GaimConnection *gc, const char *name)
 {
 	char buf[BUF_LEN * 2];
-	g_snprintf(buf, MSG_LEN, "toc_get_info %s", normalize(name));
-	sflap_send(g, buf, -1, TYPE_DATA);
+	g_snprintf(buf, MSG_LEN, "toc_get_info %s", gaim_normalize(gc->account, name));
+	sflap_send(gc, buf, -1, TYPE_DATA);
 }
 
-static void toc_get_dir(GaimConnection *g, const char *name)
+/* Should be implemented as an Account Action? */
+static void toc_get_dir(GaimBlistNode *node, gpointer data)
 {
+	GaimBuddy *buddy;
+	GaimConnection *gc;
 	char buf[BUF_LEN * 2];
-	g_snprintf(buf, MSG_LEN, "toc_get_dir %s", normalize(name));
-	sflap_send(g, buf, -1, TYPE_DATA);
+
+	g_return_if_fail(GAIM_BLIST_NODE_IS_BUDDY(node));
+
+	buddy = (GaimBuddy *) node;
+	gc = gaim_account_get_connection(buddy->account);
+
+	g_snprintf(buf, MSG_LEN, "toc_get_dir %s",
+			gaim_normalize(buddy->account, buddy->name));
+	sflap_send(gc, buf, -1, TYPE_DATA);
 }
 
+#if 0
+/* Should be implemented as an Account Action */
 static void toc_set_dir(GaimConnection *g, const char *first, const char *middle, const char *last,
 			const char *maiden, const char *city, const char *state, const char *country, int web)
 {
@@ -1114,7 +1137,10 @@ static void toc_set_dir(GaimConnection *g, const char *first, const char *middle
 	g_free(buf3);
 	sflap_send(g, buf, -1, TYPE_DATA);
 }
+#endif
 
+#if 0
+/* Should be implemented as an Account Action */
 static void toc_dir_search(GaimConnection *g, const char *first, const char *middle, const char *last,
 			   const char *maiden, const char *city, const char *state, const char *country, const char *email)
 {
@@ -1127,6 +1153,7 @@ static void toc_dir_search(GaimConnection *g, const char *first, const char *mid
 		     city, state, country);
 	sflap_send(g, buf, -1, TYPE_DATA);
 }
+#endif
 
 static void toc_set_away(GaimConnection *g, const char *state, const char *message)
 {
@@ -1162,55 +1189,60 @@ static void toc_change_passwd(GaimConnection *g, const char *orig, const char *n
 	sflap_send(g, buf, -1, TYPE_DATA);
 }
 
-static void toc_add_buddy(GaimConnection *g, const char *name)
+static void
+toc_add_buddy(GaimConnection *gc, GaimBuddy *buddy, GaimGroup *group)
 {
 	char buf[BUF_LEN * 2];
-	g_snprintf(buf, sizeof(buf), "toc_add_buddy %s", normalize(name));
-	sflap_send(g, buf, -1, TYPE_DATA);
-	toc_set_config(g);
+	g_snprintf(buf, sizeof(buf), "toc_add_buddy %s", gaim_normalize(gc->account, buddy->name));
+	sflap_send(gc, buf, -1, TYPE_DATA);
+	toc_set_config(gc);
 }
 
-static void toc_add_buddies(GaimConnection *g, GList *buddies)
+static void toc_add_buddies(GaimConnection *gc, GList *buddies, GList *groups)
 {
 	char buf[BUF_LEN * 2];
 	int n;
+	GList *cur;
 
 	n = g_snprintf(buf, sizeof(buf), "toc_add_buddy");
-	while (buddies) {
-		if (strlen(normalize(buddies->data)) + n + 32 > MSG_LEN) {
-			sflap_send(g, buf, -1, TYPE_DATA);
+	for (cur = buddies; cur != NULL; cur = cur->next) {
+		GaimBuddy *buddy = cur->data;
+
+		if (strlen(gaim_normalize(gc->account, buddy->name)) + n + 32 > MSG_LEN) {
+			sflap_send(gc, buf, -1, TYPE_DATA);
 			n = g_snprintf(buf, sizeof(buf), "toc_add_buddy");
 		}
-		n += g_snprintf(buf + n, sizeof(buf) - n, " %s", normalize(buddies->data));
-		buddies = buddies->next;
+		n += g_snprintf(buf + n, sizeof(buf) - n, " %s", gaim_normalize(gc->account, buddy->name));
 	}
-	sflap_send(g, buf, -1, TYPE_DATA);
+	sflap_send(gc, buf, -1, TYPE_DATA);
 }
 
-static void toc_remove_buddy(GaimConnection *g, const char *name, const char *group)
+static void toc_remove_buddy(GaimConnection *gc, GaimBuddy *buddy, GaimGroup *group)
 {
 	char buf[BUF_LEN * 2];
-	g_snprintf(buf, sizeof(buf), "toc_remove_buddy %s", normalize(name));
-	sflap_send(g, buf, -1, TYPE_DATA);
-	toc_set_config(g);
+	g_snprintf(buf, sizeof(buf), "toc_remove_buddy %s", gaim_normalize(gc->account, buddy->name));
+	sflap_send(gc, buf, -1, TYPE_DATA);
+	toc_set_config(gc);
 }
 
-static void toc_remove_buddies(GaimConnection *g, GList *buddies, const char *group)
+static void toc_remove_buddies(GaimConnection *gc, GList *buddies, GList *groups)
 {
 	char buf[BUF_LEN * 2];
 	int n;
+	GList *cur;
 
 	n = g_snprintf(buf, sizeof(buf), "toc_remove_buddy");
-	while (buddies) {
-		if (strlen(normalize(buddies->data)) + n + 32 > MSG_LEN) {
-			sflap_send(g, buf, -1, TYPE_DATA);
+	for (cur = buddies; cur != NULL; cur = cur->next) {
+		GaimBuddy *buddy = cur->data;
+
+		if (strlen(gaim_normalize(gc->account, buddy->name)) + n + 32 > MSG_LEN) {
+			sflap_send(gc, buf, -1, TYPE_DATA);
 			n = g_snprintf(buf, sizeof(buf), "toc_remove_buddy");
 		}
-		n += g_snprintf(buf + n, sizeof(buf) - n, " %s", normalize(buddies->data));
-		buddies = buddies->next;
+		n += g_snprintf(buf + n, sizeof(buf) - n, " %s", gaim_normalize(gc->account, buddy->name));
 	}
-	sflap_send(g, buf, -1, TYPE_DATA);
-	toc_set_config(g);
+	sflap_send(gc, buf, -1, TYPE_DATA);
+	toc_set_config(gc);
 }
 
 static void toc_set_idle(GaimConnection *g, int time)
@@ -1233,12 +1265,12 @@ static GList *toc_chat_info(GaimConnection *gc)
 	struct proto_chat_entry *pce;
 
 	pce = g_new0(struct proto_chat_entry, 1);
-	pce->label = _("Join what group:");
+	pce->label = _("_Group:");
 	pce->identifier = "room";
 	m = g_list_append(m, pce);
 
 	pce = g_new0(struct proto_chat_entry, 1);
-	pce->label = _("Exchange:");
+	pce->label = _("_Exchange:");
 	pce->identifier = "exchange";
 	pce->is_int = TRUE;
 	pce->min = 4;
@@ -1248,6 +1280,18 @@ static GList *toc_chat_info(GaimConnection *gc)
 	return m;
 }
 
+GHashTable *toc_chat_info_defaults(GaimConnection *gc, const char *chat_name)
+{
+	GHashTable *defaults;
+
+	defaults = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, g_free);
+
+	if (chat_name != NULL)
+		g_hash_table_insert(defaults, "room", g_strdup(chat_name));
+
+	return defaults;
+}
+
 static void toc_join_chat(GaimConnection *g, GHashTable *data)
 {
 	char buf[BUF_LONG];
@@ -1267,12 +1311,12 @@ static void toc_join_chat(GaimConnection *g, GHashTable *data)
 	sflap_send(g, buf, -1, TYPE_DATA);
 }
 
-static void toc_chat_invite(GaimConnection *g, int id, const char *message, const char *name)
+static void toc_chat_invite(GaimConnection *gc, int id, const char *message, const char *name)
 {
 	char buf[BUF_LONG];
 	g_snprintf(buf, sizeof(buf) / 2, "toc_chat_invite %d \"%s\" %s", id,
-			message ? message : "", normalize(name));
-	sflap_send(g, buf, -1, TYPE_DATA);
+			message ? message : "", gaim_normalize(gc->account, name));
+	sflap_send(gc, buf, -1, TYPE_DATA);
 }
 
 static void toc_chat_leave(GaimConnection *g, int id)
@@ -1283,7 +1327,7 @@ static void toc_chat_leave(GaimConnection *g, int id)
 
 	while (bcs) {
 		b = (GaimConversation *)bcs->data;
-		if (id == gaim_chat_get_id(GAIM_CHAT(b)))
+		if (id == gaim_conv_chat_get_id(GAIM_CONV_CHAT(b)))
 			break;
 		b = NULL;
 		bcs = bcs->next;
@@ -1302,13 +1346,13 @@ static void toc_chat_leave(GaimConnection *g, int id)
 	}
 }
 
-static void toc_chat_whisper(GaimConnection *g, int id, const char *who, const char *message)
+static void toc_chat_whisper(GaimConnection *gc, int id, const char *who, const char *message)
 {
 	char *buf1, *buf2;
 	buf1 = escape_text(message);
-	buf2 = g_strdup_printf("toc_chat_whisper %d %s \"%s\"", id, normalize(who), buf1);
+	buf2 = g_strdup_printf("toc_chat_whisper %d %s \"%s\"", id, gaim_normalize(gc->account, who), buf1);
 	g_free(buf1);
-	sflap_send(g, buf2, -1, TYPE_DATA);
+	sflap_send(gc, buf2, -1, TYPE_DATA);
 	g_free(buf2);
 }
 
@@ -1332,10 +1376,10 @@ static void toc_keepalive(GaimConnection *gc)
 	sflap_send(gc, "", 0, TYPE_KEEPALIVE);
 }
 
-static const char *toc_list_icon(GaimAccount *a, struct buddy *b)
+static const char *toc_list_icon(GaimAccount *a, GaimBuddy *b)
 {
 	if (!b || (b && b->name && b->name[0] == '+')) {
-		if (a != NULL && isdigit(*gaim_account_get_username(a))) 
+		if (a != NULL && isdigit(*gaim_account_get_username(a)))
 			return "icq";
 		else
 			return "aim";
@@ -1346,7 +1390,7 @@ static const char *toc_list_icon(GaimAccount *a, struct buddy *b)
 	return "aim";
 }
 
-static void toc_list_emblems(struct buddy *b, char **se, char **sw, char **nw, char **ne)
+static void toc_list_emblems(GaimBuddy *b, char **se, char **sw, char **nw, char **ne)
 {
 	char *emblems[4] = {NULL,NULL,NULL,NULL};
 	int i = 0;
@@ -1370,16 +1414,16 @@ static void toc_list_emblems(struct buddy *b, char **se, char **sw, char **nw, c
 	*ne = emblems[3];
 }
 
-static GList *toc_buddy_menu(GaimConnection *gc, const char *who)
+static GList *toc_blist_node_menu(GaimBlistNode *node)
 {
 	GList *m = NULL;
-	struct proto_buddy_menu *pbm;
+	GaimBlistNodeAction *act;
 
-	pbm = g_new0(struct proto_buddy_menu, 1);
-	pbm->label = _("Get Dir Info");
-	pbm->callback = toc_get_dir;
-	pbm->gc = gc;
-	m = g_list_append(m, pbm);
+	if(GAIM_BLIST_NODE_IS_BUDDY(node)) {
+		act = gaim_blist_node_action_new(_("Get Dir Info"),
+				toc_get_dir, NULL);
+		m = g_list_append(m, act);
+	}
 
 	return m;
 }
@@ -1389,7 +1433,7 @@ static void toc_add_permit(GaimConnection *gc, const char *who)
 	char buf2[BUF_LEN * 2];
 	if (gc->account->perm_deny != 3)
 		return;
-	g_snprintf(buf2, sizeof(buf2), "toc_add_permit %s", normalize(who));
+	g_snprintf(buf2, sizeof(buf2), "toc_add_permit %s", gaim_normalize(gc->account, who));
 	sflap_send(gc, buf2, -1, TYPE_DATA);
 	toc_set_config(gc);
 }
@@ -1399,7 +1443,7 @@ static void toc_add_deny(GaimConnection *gc, const char *who)
 	char buf2[BUF_LEN * 2];
 	if (gc->account->perm_deny != 4)
 		return;
-	g_snprintf(buf2, sizeof(buf2), "toc_add_deny %s", normalize(who));
+	g_snprintf(buf2, sizeof(buf2), "toc_add_deny %s", gaim_normalize(gc->account, who));
 	sflap_send(gc, buf2, -1, TYPE_DATA);
 	toc_set_config(gc);
 }
@@ -1436,7 +1480,7 @@ static void toc_set_permit_deny(GaimConnection *gc)
 		at = g_snprintf(buf2, sizeof(buf2), "toc_add_permit ");
 		list = gc->account->permit;
 		while (list) {
-			at += g_snprintf(buf2 + at, sizeof(buf2) - at, "%s ", normalize(list->data));
+			at += g_snprintf(buf2 + at, sizeof(buf2) - at, "%s ", gaim_normalize(gc->account, list->data));
 			if (at > MSG_LEN + 32) {	/* from out my ass comes greatness */
 				sflap_send(gc, buf2, -1, TYPE_DATA);
 				at = g_snprintf(buf2, sizeof(buf2), "toc_add_permit ");
@@ -1454,7 +1498,7 @@ static void toc_set_permit_deny(GaimConnection *gc)
 		at = g_snprintf(buf2, sizeof(buf2), "toc_add_deny ");
 		list = gc->account->deny;
 		while (list) {
-			at += g_snprintf(buf2 + at, sizeof(buf2) - at, "%s ", normalize(list->data));
+			at += g_snprintf(buf2 + at, sizeof(buf2) - at, "%s ", gaim_normalize(gc->account, list->data));
 			if (at > MSG_LEN + 32) {	/* from out my ass comes greatness */
 				sflap_send(gc, buf2, -1, TYPE_DATA);
 				at = g_snprintf(buf2, sizeof(buf2), "toc_add_deny ");
@@ -1488,32 +1532,43 @@ static GList *toc_away_states(GaimConnection *gc)
 	return g_list_append(NULL, GAIM_AWAY_CUSTOM);
 }
 
-static GList *toc_actions(GaimConnection *gc)
+static void
+show_set_info(GaimPluginAction *action)
+{
+	GaimConnection *gc = (GaimConnection *) action->context;
+	gaim_account_request_change_user_info(gaim_connection_get_account(gc));
+}
+
+static void
+change_pass(GaimPluginAction *action)
+{
+	GaimConnection *gc = (GaimConnection *) action->context;
+	gaim_account_request_change_password(gaim_connection_get_account(gc));
+}
+
+static GList *toc_actions(GaimPlugin *plugin, gpointer context)
 {
 	GList *m = NULL;
-	struct proto_actions_menu *pam;
+	GaimPluginAction *act;
 
-	pam = g_new0(struct proto_actions_menu, 1);
-	pam->label = _("Set User Info");
-	pam->callback = show_set_info;
-	pam->gc = gc;
-	m = g_list_append(m, pam);
+	act = gaim_plugin_action_new(_("Set User Info"),
+			show_set_info);
+	m = g_list_append(m, act);
 
-	pam = g_new0(struct proto_actions_menu, 1);
-	pam->label = _("Set Dir Info");
-	pam->callback = show_set_dir;
-	pam->gc = gc;
-	m = g_list_append(m, pam);
+#if 0
+	act = gaim_plugin_action_new(_("Set Dir Info"),
+			show_set_dir);
+	m = g_list_append(m, act);
+#endif
 
-	pam = g_new0(struct proto_actions_menu, 1);
-	pam->label = _("Change Password");
-	pam->callback = show_change_passwd;
-	pam->gc = gc;
-	m = g_list_append(m, pam);
+	act = gaim_plugin_action_new(_("Change Password"),
+			change_pass);
+	m = g_list_append(m, act);
 
 	return m;
 }
 
+#if 0
 /*********
  * RVOUS ACTIONS
  ********/
@@ -1609,8 +1664,8 @@ static void toc_send_file_callback(gpointer data, gint source, GaimInputConditio
 		char *buf;
 		frombase64(ft->cookie, &buf, NULL);
 
-		toc_read(source, ft, 8);
-		toc_read(source, &ft->hdr.bcookie, MIN(256 - 8, ntohs(ft->hdr.hdrlen) - 8));
+		read(source, ft, 8);
+		read(source, &ft->hdr.bcookie, MIN(256 - 8, ntohs(ft->hdr.hdrlen) - 8));
 		debug_header(ft);
 
 		ft->hdr.hdrtype = 0x202;
@@ -1618,7 +1673,7 @@ static void toc_send_file_callback(gpointer data, gint source, GaimInputConditio
 		g_free(buf);
 		ft->hdr.encrypt = 0; ft->hdr.compress = 0;
 		debug_header(ft);
-		toc_write(source, ft, 256);
+		write(source, ft, 256);
 
 		if (ft->files == 1) {
 			ft->file = fopen(ft->filename, "w");
@@ -1627,7 +1682,7 @@ static void toc_send_file_callback(gpointer data, gint source, GaimInputConditio
 				gaim_notify_error(ft->gc, NULL, buf, strerror(errno));
 				g_free(buf);
 				gaim_input_remove(ft->inpa);
-				toc_soc_close(source);
+				close(source);
 				g_free(ft->filename);
 				g_free(ft->user);
 				g_free(ft->ip);
@@ -1644,7 +1699,7 @@ static void toc_send_file_callback(gpointer data, gint source, GaimInputConditio
 				gaim_notify_error(ft->gc, NULL, buf, strerror(errno));
 				g_free(buf);
 				gaim_input_remove(ft->inpa);
-				toc_soc_close(source);
+				close(source);
 				g_free(ft->filename);
 				g_free(ft->user);
 				g_free(ft->ip);
@@ -1656,13 +1711,13 @@ static void toc_send_file_callback(gpointer data, gint source, GaimInputConditio
 		return;
 	}
 
-	rt = toc_read(source, buf, MIN(ntohl(ft->hdr.size) - ft->recvsize, 1024));
+	rt = read(source, buf, MIN(ntohl(ft->hdr.size) - ft->recvsize, 1024));
 	if (rt < 0) {
 		gaim_notify_error(ft->gc, NULL,
 						  _("File transfer failed; other side probably "
 							"canceled."), NULL);
 		gaim_input_remove(ft->inpa);
-		toc_soc_close(source);
+		close(source);
 		g_free(ft->user);
 		g_free(ft->ip);
 		g_free(ft->cookie);
@@ -1682,13 +1737,13 @@ static void toc_send_file_callback(gpointer data, gint source, GaimInputConditio
 		ft->hdr.recvcsum = ft->hdr.checksum; /* uh... */
 		ft->hdr.nrecvd = htons(ntohs(ft->hdr.nrecvd) + 1);
 		ft->hdr.flags = 0;
-		toc_write(source, ft, 256);
+		write(source, ft, 256);
 		debug_header(ft);
 		ft->recvsize = 0;
 		fclose(ft->file);
 		if (ft->hdr.filesleft == 0) {
 			gaim_input_remove(ft->inpa);
-			toc_soc_close(source);
+			close(source);
 			g_free(ft->filename);
 			g_free(ft->user);
 			g_free(ft->ip);
@@ -1765,7 +1820,7 @@ static void toc_get_file_callback(gpointer data, gint source, GaimInputCondition
 		int i;
 		for (i = 0; i < remain; i++)
 			fscanf(ft->file, "%c", &buf[i]);
-		toc_write(source, buf, remain);
+		write(source, buf, remain);
 		ft->recvsize += remain;
 		if (ft->recvsize == ntohl(ft->hdr.totsize)) {
 			gaim_input_remove(ft->inpa);
@@ -1780,8 +1835,8 @@ static void toc_get_file_callback(gpointer data, gint source, GaimInputCondition
 		struct stat st;
 		char *basename;
 
-		toc_read(source, ft, 8);
-		toc_read(source, &ft->hdr.bcookie, MIN(256 - 8, ntohs(ft->hdr.hdrlen) - 8));
+		read(source, ft, 8);
+		read(source, &ft->hdr.bcookie, MIN(256 - 8, ntohs(ft->hdr.hdrlen) - 8));
 		debug_header(ft);
 
 		stat(ft->filename, &st);
@@ -1791,28 +1846,28 @@ static void toc_get_file_callback(gpointer data, gint source, GaimInputCondition
 				fortime->tm_mon + 1, fortime->tm_mday, fortime->tm_year + 1900,
 				fortime->tm_hour + 1, fortime->tm_min + 1, (long)st.st_size,
 				basename);
-		toc_write(source, buf, ntohl(ft->hdr.size));
+		write(source, buf, ntohl(ft->hdr.size));
 		g_free(basename);
 		return;
 	}
 
 	if (ft->hdr.hdrtype == htons(0x1209)) {
-		toc_read(source, ft, 8);
-		toc_read(source, &ft->hdr.bcookie, MIN(256 - 8, ntohs(ft->hdr.hdrlen) - 8));
+		read(source, ft, 8);
+		read(source, &ft->hdr.bcookie, MIN(256 - 8, ntohs(ft->hdr.hdrlen) - 8));
 		debug_header(ft);
 		return;
 	}
 
 	if (ft->hdr.hdrtype == htons(0x120b)) {
-		toc_read(source, ft, 8);
-		toc_read(source, &ft->hdr.bcookie, MIN(256 - 8, ntohs(ft->hdr.hdrlen) - 8));
+		read(source, ft, 8);
+		read(source, &ft->hdr.bcookie, MIN(256 - 8, ntohs(ft->hdr.hdrlen) - 8));
 		debug_header(ft);
 
 		if (ft->hdr.hdrtype != htons(0x120c)) {
 			g_snprintf(buf, sizeof(buf), "%s decided to cancel the transfer", ft->user);
 			gaim_notify_error(ft->gc, NULL, buf, NULL);
 			gaim_input_remove(ft->inpa);
-			toc_soc_close(source);
+			close(source);
 			g_free(ft->filename);
 			g_free(ft->user);
 			g_free(ft->ip);
@@ -1826,13 +1881,13 @@ static void toc_get_file_callback(gpointer data, gint source, GaimInputCondition
 		ft->hdr.hdrtype = 0x0101;
 		ft->hdr.totfiles = htons(1); ft->hdr.filesleft = htons(1);
 		ft->hdr.flags = 0x20;
-		toc_write(source, ft, 256);
+		write(source, ft, 256);
 		return;
 	}
 
 	if (ft->hdr.hdrtype == 0x0101) {
-		toc_read(source, ft, 8);
-		toc_read(source, &ft->hdr.bcookie, MIN(256 - 8, ntohs(ft->hdr.hdrlen) - 8));
+		read(source, ft, 8);
+		read(source, &ft->hdr.bcookie, MIN(256 - 8, ntohs(ft->hdr.hdrlen) - 8));
 		debug_header(ft);
 
 		gaim_input_remove(ft->inpa);
@@ -1842,12 +1897,12 @@ static void toc_get_file_callback(gpointer data, gint source, GaimInputCondition
 	}
 
 	if (ft->hdr.hdrtype == 0x0202) {
-		toc_read(source, ft, 8);
-		toc_read(source, &ft->hdr.bcookie, MIN(256 - 8, ntohs(ft->hdr.hdrlen) - 8));
+		read(source, ft, 8);
+		read(source, &ft->hdr.bcookie, MIN(256 - 8, ntohs(ft->hdr.hdrlen) - 8));
 		debug_header(ft);
 
 		gaim_input_remove(ft->inpa);
-		toc_soc_close(source);
+		close(source);
 		g_free(ft->filename);
 		g_free(ft->user);
 		g_free(ft->ip);
@@ -1868,7 +1923,7 @@ static void toc_get_file_connect(gpointer data, gint src, GaimInputCondition con
 
 	if (src == -1) {
 		gaim_notify_error(ft->gc, NULL,
-						  _("Could not connect for transfer!"), NULL);
+						  _("Could not connect for transfer."), NULL);
 		fclose(ft->file);
 		g_free(ft->filename);
 		g_free(ft->cookie);
@@ -1899,7 +1954,7 @@ static void toc_get_file_connect(gpointer data, gint src, GaimInputCondition con
 	hdr->lnameoffset = 0x1A;
 	hdr->lsizeoffset = 0x10;
 	g_snprintf(hdr->name, 64, "listing.txt");
-	if (toc_write(src, hdr, 256) < 0) {
+	if (write(src, hdr, 256) < 0) {
 		gaim_notify_error(ft->gc, NULL,
 						  _("Could not write file header.  The file will "
 							"not be transferred."), NULL);
@@ -2045,68 +2100,80 @@ static void accept_file_dialog(struct ft_request *ft) {
 		g_snprintf(buf, sizeof(buf), _("%s requests you to send them a file"), ft->user);
 	}
 
-	gaim_request_accept_cancel(ft->gc, NULL, buf, NULL, 0, ft,
+	gaim_request_accept_cancel(ft->gc, NULL, buf, NULL, 
+								GAIM_DEFAULT_ACTION_NONE, ft,
 							   G_CALLBACK(toc_accept_ft),
 							   G_CALLBACK(toc_reject_ft));
 }
+#endif
 
 static GaimPluginProtocolInfo prpl_info =
 {
-	GAIM_PROTO_TOC,
-	OPT_PROTO_CORRECT_TIME,
-	NULL,
-	NULL,
-	toc_list_icon,
-	toc_list_emblems,
-	NULL,
-	NULL,
-	toc_away_states,
-	toc_actions,
-	toc_buddy_menu,
-	toc_chat_info,
-	toc_login,
-	toc_close,
-	toc_send_im,
-	toc_set_info,
-	NULL,
-	toc_get_info,
-	toc_set_away,
-	NULL,
-	toc_set_dir,
-	toc_get_dir,
-	toc_dir_search,
-	toc_set_idle,
-	toc_change_passwd,
-	toc_add_buddy,
-	toc_add_buddies,
-	toc_remove_buddy,
-	toc_remove_buddies,
-	toc_add_permit,
-	toc_add_deny,
-	toc_rem_permit,
-	toc_rem_deny,
-	toc_set_permit_deny,
-	toc_warn,
-	toc_join_chat,
-	toc_chat_invite,
-	toc_chat_leave,
-	toc_chat_whisper,
-	toc_chat_send,
-	toc_keepalive,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL
+	0,
+	NULL,					/* user_splits */
+	NULL,					/* protocol_options */
+	NO_BUDDY_ICONS,			/* icon_spec */
+	toc_list_icon,			/* list_icon */
+	toc_list_emblems,		/* list_emblems */
+	NULL,					/* status_text */
+	NULL,					/* tooltip_text */
+	toc_away_states,		/* away_states */
+	toc_blist_node_menu,	/* blist_node_menu */
+	toc_chat_info,			/* chat_info */
+	toc_chat_info_defaults,	/* chat_info_defaults */
+	toc_login,				/* login */
+	toc_close,				/* close */
+	toc_send_im,			/* send_im */
+	toc_set_info,			/* set_info */
+	NULL,					/* send_typing */
+	toc_get_info,			/* get_info */
+	toc_set_away,			/* set_away */
+	toc_set_idle,			/* set_idle */
+	toc_change_passwd,		/* change_passwd */
+	toc_add_buddy,			/* add_buddy */
+	toc_add_buddies,		/* add_buddies */
+	toc_remove_buddy,		/* remove_buddy */
+	toc_remove_buddies,		/* remove_buddies */
+	toc_add_permit,			/* add_permit */
+	toc_add_deny,			/* add_deny */
+	toc_rem_permit,			/* rem_permit */
+	toc_rem_deny,			/* rem_deny */
+	toc_set_permit_deny,	/* set_permit_deny */
+	toc_warn,				/* warn */
+	toc_join_chat,			/* join_chat */
+	NULL,					/* reject_chat */
+	NULL,				/* get_chat_name */
+	toc_chat_invite,		/* chat_invite */
+	toc_chat_leave,			/* chat_leave */
+	toc_chat_whisper,		/* chat_whisper */
+	toc_chat_send,			/* chat_send */
+	toc_keepalive,			/* keepalive */
+	NULL,					/* register_user */
+	NULL,					/* get_cb_info */
+	NULL,					/* get_cb_away */
+	NULL,					/* alias_buddy */
+	NULL,					/* group_buddy */
+	NULL,					/* rename_group */
+	NULL,					/* buddy_free */
+	NULL,					/* convo_closed */
+	NULL,					/* normalize */
+	NULL,					/* set_buddy_icon */
+	NULL,					/* remove_group */
+	NULL,					/* get_cb_real_name */
+	NULL,					/* set_chat_topic */
+	NULL,					/* find_blist_chat */
+	NULL,					/* roomlist_get_list */
+	NULL,					/* roomlist_cancel */
+	NULL,					/* roomlist_expand_category */
+	NULL,					/* can_receive_file */
+	NULL					/* send_file */
 };
 
 static GaimPluginInfo info =
 {
-	2,                                                /**< api_version    */
+	GAIM_PLUGIN_MAGIC,
+	GAIM_MAJOR_VERSION,
+	GAIM_MINOR_VERSION,
 	GAIM_PLUGIN_PROTOCOL,                             /**< type           */
 	NULL,                                             /**< ui_requirement */
 	0,                                                /**< flags          */
@@ -2128,7 +2195,9 @@ static GaimPluginInfo info =
 	NULL,                                             /**< destroy        */
 
 	NULL,                                             /**< ui_info        */
-	&prpl_info                                        /**< extra_info     */
+	&prpl_info,                                       /**< extra_info     */
+	NULL,
+	toc_actions
 };
 
 static void
