@@ -36,11 +36,11 @@
 #include "conversation.h"
 #include "notify.h"
 #include "util.h"
-#include "multi.h"
 #include "internal.h"
 
 #include "yahoo.h"
 #include "yahoochat.h"
+#include "ycht.h"
 
 #define YAHOO_CHAT_ID (1)
 
@@ -53,6 +53,10 @@ static void yahoo_chat_online(GaimConnection *gc)
 	struct yahoo_data *yd = gc->proto_data;
 	struct yahoo_packet *pkt;
 
+	if (yd->wm) {
+		ycht_connection_open(gc);
+		return;
+	}
 
 	pkt = yahoo_packet_new(YAHOO_SERVICE_CHATONLINE, YAHOO_STATUS_AVAILABLE,0);
 	yahoo_packet_hash(pkt, 1, gaim_connection_get_display_name(gc));
@@ -64,36 +68,24 @@ static void yahoo_chat_online(GaimConnection *gc)
 	yahoo_packet_free(pkt);
 }
 
-static gint _mystrcmpwrapper(gconstpointer a, gconstpointer b)
-{
-	return strcmp(a, b);
-}
-
 /* this is slow, and different from the gaim_* version in that it (hopefully) won't add a user twice */
-static void yahoo_chat_add_users(GaimConvChat *chat, GList *newusers)
+void yahoo_chat_add_users(GaimConvChat *chat, GList *newusers)
 {
-	GList *users, *i, *j;
-
-	users = gaim_conv_chat_get_users(chat);
+	GList *i;
 
 	for (i = newusers; i; i = i->next) {
-		j = g_list_find_custom(users, i->data, _mystrcmpwrapper);
-		if (j)
+		if (gaim_conv_chat_find_user(chat, i->data))
 			continue;
-		gaim_conv_chat_add_user(chat, i->data, NULL);
+		gaim_conv_chat_add_user(chat, i->data, NULL, GAIM_CBFLAGS_NONE, TRUE);
 	}
 }
 
-static void yahoo_chat_add_user(GaimConvChat *chat, const char *user, const char *reason)
+void yahoo_chat_add_user(GaimConvChat *chat, const char *user, const char *reason)
 {
-	GList *users;
-
-	users = gaim_conv_chat_get_users(chat);
-
-	if ((g_list_find_custom(users, user, _mystrcmpwrapper)))
+	if (gaim_conv_chat_find_user(chat, user))
 		return;
 
-	gaim_conv_chat_add_user(chat, user, reason);
+	gaim_conv_chat_add_user(chat, user, reason, GAIM_CBFLAGS_NONE, TRUE);
 }
 
 static GaimConversation *yahoo_find_conference(GaimConnection *gc, const char *name)
@@ -140,7 +132,8 @@ void yahoo_process_conference_invite(GaimConnection *gc, struct yahoo_packet *pk
 			who = pair->value;
 			g_string_append_printf(members, "%s\n", who);
 			break;
-		case 52: /* members */
+		case 52: /* invitee (me) */
+		case 53: /* members */
 			g_string_append_printf(members, "%s\n", pair->value);
 			break;
 		case 58:
@@ -193,12 +186,16 @@ void yahoo_process_conference_decline(GaimConnection *gc, struct yahoo_packet *p
 	}
 
 	if (who && room) {
-		char *tmp;
+		/* make sure we're in the room before we process a decline message for it */
+		if(yahoo_find_conference(gc, room)) {
+			char *tmp;
+
+			tmp = g_strdup_printf(_("%s declined your conference invitation to room \"%s\" because \"%s\"."),
+							who, room, msg?msg:"");
+			gaim_notify_info(gc, NULL, _("Invitation Rejected"), tmp);
+			g_free(tmp);
+		}
 
-		tmp = g_strdup_printf(_("%s declined your conference invitation to room \"%s\" because \"%s\"."),
-						who, room, msg?msg:"");
-		gaim_notify_info(gc, NULL, _("Invitation Rejected"), tmp);
-		g_free(tmp);
 		g_free(room);
 		if (msg)
 			g_free(msg);
@@ -398,18 +395,20 @@ void yahoo_process_chat_join(GaimConnection *gc, struct yahoo_packet *pkt)
 		}
 	}
 
-	if (!room)
-		return;
 
-	if (yd->chat_name && gaim_utf8_strcasecmp(room, yd->chat_name))
+	if (room && yd->chat_name && gaim_utf8_strcasecmp(room, yd->chat_name))
 		yahoo_chat_leave(gc, room,
 				gaim_connection_get_display_name(gc), FALSE);
 
 	c = gaim_find_chat(gc, YAHOO_CHAT_ID);
 
-	if ((!c || gaim_conv_chat_has_left(GAIM_CONV_CHAT(c))) && members &&
+	if (room && (!c || gaim_conv_chat_has_left(GAIM_CONV_CHAT(c))) && members &&
 	   ((g_list_length(members) > 1) ||
 	     !g_ascii_strcasecmp(members->data, gaim_connection_get_display_name(gc)))) {
+		int i;
+		GList *flags = NULL;
+		for (i = 0; i < g_list_length(members); i++)
+			flags = g_list_append(flags, GINT_TO_POINTER(GAIM_CBFLAGS_NONE));
 		if (c && gaim_conv_chat_has_left(GAIM_CONV_CHAT(c))) {
 			/* this might be a hack, but oh well, it should nicely */
 			char *tmpmsg;
@@ -421,7 +420,7 @@ void yahoo_process_chat_join(GaimConnection *gc, struct yahoo_packet *pkt)
 				gaim_conv_chat_set_topic(GAIM_CONV_CHAT(c), NULL, topic);
 			yd->in_chat = 1;
 			yd->chat_name = g_strdup(room);
-			gaim_conv_chat_add_users(GAIM_CONV_CHAT(c), members);
+			gaim_conv_chat_add_users(GAIM_CONV_CHAT(c), members, flags);
 
 			tmpmsg = g_strdup_printf(_("You are now chatting in %s."), room);
 			gaim_conv_chat_write(GAIM_CONV_CHAT(c), "", tmpmsg, GAIM_MESSAGE_SYSTEM, time(NULL));
@@ -432,7 +431,7 @@ void yahoo_process_chat_join(GaimConnection *gc, struct yahoo_packet *pkt)
 				gaim_conv_chat_set_topic(GAIM_CONV_CHAT(c), NULL, topic);
 			yd->in_chat = 1;
 			yd->chat_name = g_strdup(room);
-			gaim_conv_chat_add_users(GAIM_CONV_CHAT(c), members);
+			gaim_conv_chat_add_users(GAIM_CONV_CHAT(c), members, flags);
 		}
 	} else if (c) {
 		yahoo_chat_add_users(GAIM_CONV_CHAT(c), members);
@@ -587,17 +586,19 @@ void yahoo_process_chat_goto(GaimConnection *gc, struct yahoo_packet *pkt)
  * I think conference names are always ascii.
  */
 
-static void yahoo_conf_leave(struct yahoo_data *yd, const char *room, const char *dn, GList *who)
+void yahoo_conf_leave(struct yahoo_data *yd, const char *room, const char *dn, GList *who)
 {
 	struct yahoo_packet *pkt;
 	GList *w;
 
-
+	gaim_debug_misc("yahoo", "leaving conference %s\n", room);
+	
 	pkt = yahoo_packet_new(YAHOO_SERVICE_CONFLOGOFF, YAHOO_STATUS_AVAILABLE, 0);
 
 	yahoo_packet_hash(pkt, 1, dn);
 	for (w = who; w; w = w->next) {
-		yahoo_packet_hash(pkt, 3, (char *)w->data);
+		const char *name = gaim_conv_chat_cb_get_name(w->data);
+		yahoo_packet_hash(pkt, 3, name);
 	}
 
 	yahoo_packet_hash(pkt, 57, room);
@@ -623,8 +624,10 @@ static int yahoo_conf_send(GaimConnection *gc, const char *dn, const char *room,
 	pkt = yahoo_packet_new(YAHOO_SERVICE_CONFMSG, YAHOO_STATUS_AVAILABLE, 0);
 
 	yahoo_packet_hash(pkt, 1, dn);
-	for (who = members; who; who = who->next)
-		yahoo_packet_hash(pkt, 53, (char *)who->data);
+	for (who = members; who; who = who->next) {
+		const char *name = gaim_conv_chat_cb_get_name(who->data);
+		yahoo_packet_hash(pkt, 53, name);
+	}
 	yahoo_packet_hash(pkt, 57, room);
 	yahoo_packet_hash(pkt, 14, msg2);
 	if (utf8)
@@ -660,7 +663,7 @@ static void yahoo_conf_join(struct yahoo_data *yd, GaimConversation *c, const ch
 			if (!strcmp(memarr[i], "") || !strcmp(memarr[i], dn))
 					continue;
 			yahoo_packet_hash(pkt, 3, memarr[i]);
-			gaim_conv_chat_add_user(GAIM_CONV_CHAT(c), memarr[i], NULL);
+			gaim_conv_chat_add_user(GAIM_CONV_CHAT(c), memarr[i], NULL, GAIM_CBFLAGS_NONE, TRUE);
 		}
 	}
 	yahoo_send_packet(yd, pkt);
@@ -692,10 +695,11 @@ static void yahoo_conf_invite(GaimConnection *gc, GaimConversation *c,
 	yahoo_packet_hash(pkt, 58, msg?msg2:"");
 	yahoo_packet_hash(pkt, 13, "0");
 	for(; members; members = members->next) {
-		if (!strcmp(members->data, dn))
+		const char *name = gaim_conv_chat_cb_get_name(members->data);
+		if (!strcmp(name, dn))
 			continue;
-		yahoo_packet_hash(pkt, 52, (char *)members->data);
-		yahoo_packet_hash(pkt, 53, (char *)members->data);
+		yahoo_packet_hash(pkt, 52, name);
+		yahoo_packet_hash(pkt, 53, name);
 	}
 	yahoo_send_packet(yd, pkt);
 
@@ -716,6 +720,13 @@ static void yahoo_chat_leave(GaimConnection *gc, const char *room, const char *d
 	char *eroom;
 	gboolean utf8 = 1;
 
+	if (yd->wm) {
+		g_return_if_fail(yd->ycht != NULL);
+
+		ycht_chat_leave(yd->ycht, room, logout);
+		return;
+	}
+
 	eroom = yahoo_string_encode(gc, room, &utf8);
 
 	pkt = yahoo_packet_new(YAHOO_SERVICE_CHATEXIT, YAHOO_STATUS_AVAILABLE, 0);
@@ -798,6 +809,12 @@ static int yahoo_chat_send(GaimConnection *gc, const char *dn, const char *room,
 	char *msg1, *msg2, *room2;
 	gboolean utf8 = TRUE;
 
+	if (yd->wm) {
+		g_return_val_if_fail(yd->ycht != NULL, 1);
+
+		return ycht_chat_send(yd->ycht, room, what);
+	}
+
 	msg1 = g_strdup(what);
 
 	if (meify(msg1, -1))
@@ -837,6 +854,13 @@ static void yahoo_chat_join(GaimConnection *gc, const char *dn, const char *room
 	char *room2;
 	gboolean utf8 = TRUE;
 
+	if (yd->wm) {
+		g_return_if_fail(yd->ycht != NULL);
+
+		ycht_chat_join(yd->ycht, room);
+		return;
+	}
+
 	/* apparently room names are always utf8, or else always not utf8,
 	 * so we don't have to actually pass the flag in the packet. Or something. */
 	room2 = yahoo_string_encode(gc, room, &utf8);
@@ -862,6 +886,13 @@ static void yahoo_chat_invite(GaimConnection *gc, const char *dn, const char *bu
 	char *room2, *msg2 = NULL;
 	gboolean utf8 = TRUE;
 
+	if (yd->wm) {
+		g_return_if_fail(yd->ycht != NULL);
+
+		ycht_chat_send_invite(yd->ycht, room, buddy, msg);
+		return;
+	}
+
 	room2 = yahoo_string_encode(gc, room, &utf8);
 	if (msg)
 		msg2 = yahoo_string_encode(gc, msg, NULL);
@@ -888,6 +919,13 @@ void yahoo_chat_goto(GaimConnection *gc, const char *name)
 
 	yd = gc->proto_data;
 
+	if (yd->wm) {
+		g_return_if_fail(yd->ycht != NULL);
+
+		ycht_chat_goto_user(yd->ycht, name);
+		return;
+	}
+
 	if (!yd->chat_online)
 		yahoo_chat_online(gc);
 
@@ -969,6 +1007,23 @@ GList *yahoo_c_info(GaimConnection *gc)
 	return m;
 }
 
+GHashTable *yahoo_c_info_defaults(GaimConnection *gc, const char *chat_name)
+{
+	GHashTable *defaults;
+
+	defaults = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, g_free);
+
+	if (chat_name != NULL)
+		g_hash_table_insert(defaults, "room", g_strdup(chat_name));
+
+	return defaults;
+}
+
+char *yahoo_get_chat_name(GHashTable *data)
+{
+	return g_strdup(g_hash_table_lookup(data, "room"));
+}
+
 void yahoo_c_join(GaimConnection *gc, GHashTable *data)
 {
 	struct yahoo_data *yd;
@@ -1055,6 +1110,7 @@ static void yahoo_roomlist_destroy(struct yahoo_roomlist *yrl)
 		g_free(yrl->host);
 	if (yrl->parse)
 		g_markup_parse_context_free(yrl->parse);
+	g_free(yrl);
 }
 
 enum yahoo_room_type {
@@ -1331,7 +1387,7 @@ GaimRoomlist *yahoo_roomlist_get_list(GaimConnection *gc)
 	rl = gaim_roomlist_new(gaim_connection_get_account(gc));
 	yrl->list = rl;
 
-	gaim_url_parse(url, &(yrl->host), NULL, &(yrl->path));
+	gaim_url_parse(url, &(yrl->host), NULL, &(yrl->path), NULL, NULL);
 	g_free(url);
 
 	f = gaim_roomlist_field_new(GAIM_ROOMLIST_FIELD_STRING, "", "room", TRUE);
@@ -1408,7 +1464,7 @@ void yahoo_roomlist_expand_category(GaimRoomlist *list, GaimRoomlistRoom *catego
 	yrl->cat = category;
 	list->proto_data = g_list_append(list->proto_data, yrl);
 
-	gaim_url_parse(url, &(yrl->host), NULL, &(yrl->path));
+	gaim_url_parse(url, &(yrl->host), NULL, &(yrl->path), NULL, NULL);
 	g_free(url);
 
 	yrl->ucat = gaim_roomlist_room_new(GAIM_ROOMLIST_ROOMTYPE_CATEGORY, _("User Rooms"), yrl->cat);
@@ -1426,4 +1482,3 @@ void yahoo_roomlist_expand_category(GaimRoomlist *list, GaimRoomlistRoom *catego
 	gaim_roomlist_set_in_progress(list, TRUE);
 	gaim_roomlist_ref(list);
 }
-
