@@ -36,9 +36,9 @@
 #include "proxy.h"
 #include "util.h"
 
-static GaimProxyInfo *global_proxy_info = NULL;
+#include "gaim.h"
 
-static int opt_debug = 0;
+static GaimProxyInfo *global_proxy_info = NULL;
 
 struct PHB {
 	GaimInputFunction func;
@@ -48,8 +48,11 @@ struct PHB {
 	gint inpa;
 	GaimProxyInfo *gpi;
 	GaimAccount *account;
+	GSList *hosts;
 };
 
+static void try_connect(struct PHB *);
+
 const char* socks5errors[] = {
 	"succeeded\n",
 	"general SOCKS server failure\n",
@@ -187,7 +190,7 @@ gaim_global_proxy_get_info(void)
 typedef void (*dns_callback_t)(GSList *hosts, gpointer data,
 		const char *error_message);
 
-#ifdef __unix__
+#if defined __unix__
 
 /*  This structure represents both a pending DNS request and
  *  a free child process.
@@ -242,7 +245,7 @@ static int send_dns_request_to_child(pending_dns_request_t *req, dns_params_t *d
 				   "DNS child %d no longer exists\n", req->dns_pid);
 		return -1;
 	}
-	
+
 	/* Let's contact this lost child! */
 	rc = write(req->fd_in, dns_params, sizeof(*dns_params));
 	if(rc<0) {
@@ -252,9 +255,9 @@ static int send_dns_request_to_child(pending_dns_request_t *req, dns_params_t *d
 		close(req->fd_in);
 		return -1;
 	}
-	
+
 	g_return_val_if_fail(rc == sizeof(*dns_params), -1);
-	
+
 	/* Did you hear me? (This avoids some race conditions) */
 	rc = read(req->fd_out, &ch, 1);
 	if(rc != 1 || ch!='Y') {
@@ -264,7 +267,7 @@ static int send_dns_request_to_child(pending_dns_request_t *req, dns_params_t *d
 		kill(req->dns_pid, SIGKILL);
 		return -1;
 	}
-	
+
 	gaim_debug(GAIM_DEBUG_INFO, "dns",
 			   "Successfully sent DNS request to child %d\n", req->dns_pid);
 	return 0;
@@ -300,7 +303,7 @@ static void release_dns_child(pending_dns_request_t *req)
 			req->inpa = gaim_input_add(req->fd_out, GAIM_INPUT_READ, host_resolved, req);
 			g_free(r);
 		}
-		
+
 	} else {
 		req->host = NULL;
 		req->callback = NULL;
@@ -368,12 +371,6 @@ static void host_resolved(gpointer data, gint source, GaimInputCondition cond)
 
 	req->callback(hosts, req->data, NULL);
 
-	while(hosts) {
-		hosts = g_slist_remove(hosts, hosts->data);
-		g_free(hosts->data);
-		hosts = g_slist_remove(hosts, hosts->data);
-	}
-
 	release_dns_child(req);
 }
 
@@ -395,12 +392,12 @@ static void trap_gdb_bug()
 
 static void cope_with_gdb_brokenness()
 {
+#ifdef __linux__
 	static gboolean already_done = FALSE;
 	char s[256], e[512];
 	int n;
 	pid_t ppid;
 
-#ifdef __linux__
 	if(already_done)
 		return;
 	already_done = TRUE;
@@ -420,196 +417,301 @@ static void cope_with_gdb_brokenness()
 #endif
 }
 
+static void
+gaim_dns_childthread(int child_out, int child_in, dns_params_t *dns_params, gboolean show_debug)
+{
+	const int zero = 0;
+	int rc;
+#if HAVE_GETADDRINFO
+	struct addrinfo hints, *res, *tmp;
+	char servname[20];
+#else
+	struct sockaddr_in sin;
+	const size_t addrlen = sizeof(sin);
+#endif
+
+#ifdef HAVE_SIGNAL_H
+	signal(SIGHUP, SIG_DFL);
+	signal(SIGINT, SIG_DFL);
+	signal(SIGQUIT, SIG_DFL);
+	signal(SIGCHLD, SIG_DFL);
+	signal(SIGTERM, SIG_DFL);
+	signal(SIGTRAP, trap_gdb_bug);
+#endif
+
+	while (1) {
+		if (dns_params->hostname[0] == '\0') {
+			const char Y = 'Y';
+			fd_set fds;
+			struct timeval tv = { .tv_sec = 40 , .tv_usec = 0 };
+			FD_ZERO(&fds);
+			FD_SET(child_in, &fds);
+			rc = select(child_in + 1, &fds, NULL, NULL, &tv);
+			if (!rc) {
+				if (show_debug)
+					fprintf(stderr,"dns[%d]: nobody needs me... =(\n", getpid());
+				break;
+			}
+			rc = read(child_in, dns_params, sizeof(dns_params_t));
+			if (rc < 0) {
+				perror("read()");
+				break;
+			}
+			if (rc==0) {
+				if (show_debug)
+					fprintf(stderr,"dns[%d]: Oops, father has gone, wait for me, wait...!\n", getpid());
+				_exit(0);
+			}
+			if (dns_params->hostname[0] == '\0') {
+				fprintf(stderr, "dns[%d]: hostname = \"\" (port = %d)!!!\n", getpid(), dns_params->port);
+				_exit(1);
+			}
+			write(child_out, &Y, 1);
+		}
+
+#if HAVE_GETADDRINFO
+		g_snprintf(servname, sizeof(servname), "%d", dns_params->port);
+		memset(&hints,0,sizeof(hints));
+
+		/* This is only used to convert a service
+		 * name to a port number. As we know we are
+		 * passing a number already, we know this
+		 * value will not be really used by the C
+		 * library.
+		 */
+		hints.ai_socktype = SOCK_STREAM;
+		rc = getaddrinfo(dns_params->hostname, servname, &hints, &res);
+		if (rc) {
+			write(child_out, &rc, sizeof(int));
+			close(child_out);
+			if (show_debug)
+				fprintf(stderr,"dns[%d] Error: getaddrinfo returned %d\n",
+					getpid(), rc);
+			dns_params->hostname[0] = '\0';
+			continue;
+		}
+		write(child_out, &zero, sizeof(zero));
+		tmp = res;
+		while(res) {
+			size_t ai_addrlen = res->ai_addrlen;
+			write(child_out, &ai_addrlen, sizeof(ai_addrlen));
+			write(child_out, res->ai_addr, res->ai_addrlen);
+			res = res->ai_next;
+		}
+		freeaddrinfo(tmp);
+		write(child_out, &zero, sizeof(zero));
+#else
+		if (!inet_aton(hostname, &sin.sin_addr)) {
+			struct hostent *hp;
+			if (!(hp = gethostbyname(dns_params->hostname))) {
+				write(child_out, &h_errno, sizeof(int));
+				close(child_out);
+				if (show_debug)
+					fprintf(stderr,"DNS Error: %d\n", h_errno);
+				_exit(0);
+			}
+			memset(&sin, 0, sizeof(struct sockaddr_in));
+			memcpy(&sin.sin_addr.s_addr, hp->h_addr, hp->h_length);
+			sin.sin_family = hp->h_addrtype;
+		} else
+			sin.sin_family = AF_INET;
+
+		sin.sin_port = htons(dns_params->port);
+		write(child_out, &zero, sizeof(zero));
+		write(child_out, &addrlen, sizeof(addrlen));
+		write(child_out, &sin, addrlen);
+		write(child_out, &zero, sizeof(zero));
+#endif
+		dns_params->hostname[0] = '\0';
+	}
+
+	close(child_out);
+	close(child_in);
+
+	_exit(0);
+}
+
 int gaim_gethostbyname_async(const char *hostname, int port, dns_callback_t callback, gpointer data)
 {
 	pending_dns_request_t *req = NULL;
 	dns_params_t dns_params;
-	
-	strncpy(dns_params.hostname, hostname, sizeof(dns_params.hostname)-1);
-	dns_params.hostname[sizeof(dns_params.hostname)-1] = '\0';
+	gchar *host_temp;
+
+	host_temp = g_strstrip(g_strdup(hostname));
+	strncpy(dns_params.hostname, host_temp, sizeof(dns_params.hostname) - 1);
+	g_free(host_temp);
+	dns_params.hostname[sizeof(dns_params.hostname) - 1] = '\0';
 	dns_params.port = port;
-	
-	/* Is there a free available child? */
-	while(free_dns_children && !req) {
-		GSList *l = free_dns_children;
-		free_dns_children = g_slist_remove_link(free_dns_children, l);
-		req = l->data;
-		g_slist_free(l);
-		
-		if(send_dns_request_to_child(req, &dns_params) != 0) {
-			req_free(req);
-			req = NULL;
-			continue;
-		}
-		
+
+	/*
+	 * If we have any children, attempt to have them perform the DNS
+	 * query.  If we're able to send the query to a child, then req
+	 * will be set to the pending_dns_request_t.  Otherwise, req will
+	 * be NULL and we'll need to create a new DNS request child.
+	 */
+	while (free_dns_children != NULL) {
+		req = free_dns_children->data;
+		free_dns_children = g_slist_remove(free_dns_children, req);
+
+		if (send_dns_request_to_child(req, &dns_params) == 0)
+			/* We found an acceptable child, yay */
+			break;
+
+		req_free(req);
+		req = NULL;
 	}
 
-	if(!req) {
+	/* We need to create a new DNS request child */
+	if (req == NULL) {
 		int child_out[2], child_in[2];
 
-		if(number_of_dns_children >= MAX_DNS_CHILDREN) {
+		if (number_of_dns_children >= MAX_DNS_CHILDREN) {
 			queued_dns_request_t *r = g_new(queued_dns_request_t, 1);
 			memcpy(&(r->params), &dns_params, sizeof(dns_params));
 			r->callback = callback;
 			r->data = data;
-			if(!queued_requests)
+			if (!queued_requests)
 				queued_requests = g_queue_new();
 			g_queue_push_tail(queued_requests, r);
 
 			gaim_debug(GAIM_DEBUG_INFO, "dns",
-					   "DNS query for '%s' queued\n", hostname);
+					   "DNS query for '%s' queued\n", dns_params.hostname);
 
 			return 0;
 		}
 
-		if(pipe(child_out) || pipe(child_in)) {
+		/* Create pipes for communicating with the child process */
+		if (pipe(child_out) || pipe(child_in)) {
 			gaim_debug(GAIM_DEBUG_ERROR, "dns",
 					   "Could not create pipes: %s\n", strerror(errno));
 			return -1;
 		}
 
-		/* We need to create a new child. */
-		req = g_new(pending_dns_request_t,1);
-		
+		req = g_new(pending_dns_request_t, 1);
+
 		cope_with_gdb_brokenness();
-	
-		req->dns_pid=fork();
-		if(req->dns_pid==0) {
-			const int zero = 0;
-			int rc;
-
-#if HAVE_GETADDRINFO
-			struct addrinfo hints, *res, *tmp;
-			char servname[20];
-#else
-			struct sockaddr_in sin;
-			const size_t addrlen = sizeof(sin);
-#endif
-#ifdef HAVE_SIGNAL_H
-			signal(SIGHUP, SIG_DFL);
-			signal(SIGINT, SIG_DFL);
-			signal(SIGQUIT, SIG_DFL);
-			signal(SIGCHLD, SIG_DFL);
-			signal(SIGTERM, SIG_DFL);
-			signal(SIGTRAP, trap_gdb_bug);
-#endif
 
+		/* Fork! */
+		req->dns_pid = fork();
 
+		/* If we are the child process... */
+		if (req->dns_pid == 0) {
+			/* We should not access the parent's side of the pipe, so close them... */
 			close(child_out[0]);
 			close(child_in[1]);
 
-			while(1) {
-				if(dns_params.hostname[0] == '\0') {
-					const char Y = 'Y';
-					fd_set fds;
-					struct timeval tv = { .tv_sec = 40 , .tv_usec = 0 };
-					FD_ZERO(&fds);
-					FD_SET(child_in[0], &fds);
-					rc = select(child_in[0]+1, &fds, NULL, NULL, &tv);
-					if(!rc) {
-						if (opt_debug)
-							fprintf(stderr,"dns[%d]: nobody needs me... =(\n", getpid());
-						break;
-					}
-					rc = read(child_in[0], &dns_params, sizeof(dns_params));
-					if(rc < 0) {
-						perror("read()");
-						break;
-					}
-					if(rc==0) {
-						if(opt_debug)
-							fprintf(stderr,"dns[%d]: Ops, father has gone, wait for me, wait...!\n", getpid());
-						_exit(0);
-					}
-					if(dns_params.hostname[0] == '\0') {
-						fprintf(stderr, "dns[%d]: hostname = \"\" (port = %d)!!!\n", getpid(), dns_params.port);
-						_exit(1);
-					}
-					write(child_out[1], &Y, 1);
-				}
-
-#if HAVE_GETADDRINFO
-				g_snprintf(servname, sizeof(servname), "%d", dns_params.port);
-				memset(&hints,0,sizeof(hints));
-				
-				/* This is only used to convert a service
-				 * name to a port number. As we know we are
-				 * passing a number already, we know this
-				 * value will not be really used by the C
-				 * library.
-				 */
-				hints.ai_socktype = SOCK_STREAM;
-				rc = getaddrinfo(dns_params.hostname, servname, &hints, &res);
-				if(rc) {
-					write(child_out[1], &rc, sizeof(int));
-					close(child_out[1]);
-					if(opt_debug)
-						fprintf(stderr,"dns[%d] Error: getaddrinfo returned %d\n",
-							getpid(), rc);
-					dns_params.hostname[0] = '\0';
-					continue;
-				}
-				write(child_out[1], &zero, sizeof(zero));
-				tmp = res;
-				while(res) {
-					size_t ai_addrlen = res->ai_addrlen;
-					write(child_out[1], &ai_addrlen, sizeof(ai_addrlen));
-					write(child_out[1], res->ai_addr, res->ai_addrlen);
-					res = res->ai_next;
-				}
-				freeaddrinfo(tmp);
-				write(child_out[1], &zero, sizeof(zero));
-#else
-				if (!inet_aton(hostname, &sin.sin_addr)) {
-					struct hostent *hp;
-					if(!(hp = gethostbyname(dns_params.hostname))) {
-						write(child_out[1], &h_errno, sizeof(int));
-						close(child_out[1]);
-						if(opt_debug)
-							fprintf(stderr,"DNS Error: %d\n", h_errno);
-						_exit(0);
-					}
-					memset(&sin, 0, sizeof(struct sockaddr_in));
-					memcpy(&sin.sin_addr.s_addr, hp->h_addr, hp->h_length);
-					sin.sin_family = hp->h_addrtype;
-				} else
-					sin.sin_family = AF_INET;
-				sin.sin_port = htons(dns_params.port);
-				write(child_out[1], &zero, sizeof(zero));
-				write(child_out[1], &addrlen, sizeof(addrlen));
-				write(child_out[1], &sin, addrlen);
-				write(child_out[1], &zero, sizeof(zero));
-#endif
-				dns_params.hostname[0] = '\0';
-			}
-			close(child_out[1]);
-			close(child_in[0]);
-			_exit(0);
+			gaim_dns_childthread(child_out[1], child_in[0], &dns_params, opt_debug);
+			/* The thread calls _exit() rather than returning, so we never get here */
 		}
+
+		/* We should not access the child's side of the pipe, so close them... */
 		close(child_out[1]);
 		close(child_in[0]);
-		if(req->dns_pid==-1) {
+		if (req->dns_pid == -1) {
 			gaim_debug(GAIM_DEBUG_ERROR, "dns",
 					   "Could not create child process for DNS: %s\n",
 					   strerror(errno));
 			g_free(req);
 			return -1;
 		}
-		req->fd_in = child_in[1];
+
 		req->fd_out = child_out[0];
+		req->fd_in = child_in[1];
 		number_of_dns_children++;
 		gaim_debug(GAIM_DEBUG_INFO, "dns",
 				   "Created new DNS child %d, there are now %d children.\n",
 				   req->dns_pid, number_of_dns_children);
 	}
-	req->host=g_strdup(hostname);
-	req->port=port;
-	req->callback=callback;
-	req->data=data;
+
+	req->host = g_strdup(hostname);
+	req->port = port;
+	req->callback = callback;
+	req->data = data;
 	req->inpa = gaim_input_add(req->fd_out, GAIM_INPUT_READ, host_resolved, req);
+
 	return 0;
 }
-#else
+
+#elif defined _WIN32 /* end __unix__ */
+
+/* Note: The below win32 implementation is actually platform independent.
+   Perhaps this can be used in place of the above platform dependent code.
+*/ 
+
+typedef struct _dns_tdata {
+	char *hostname;
+	int port;
+	dns_callback_t callback;
+	gpointer data;
+	GSList *hosts;
+	char *errmsg;
+} dns_tdata;
+
+static gboolean dns_main_thread_cb(gpointer data) {
+	dns_tdata *td = (dns_tdata*)data;
+	td->callback(td->hosts, td->data, td->errmsg);
+	g_free(td->hostname);
+	g_free(td);
+	return FALSE;
+}
+
+static gpointer dns_thread(gpointer data) {
+	struct sockaddr_in sin;
+	dns_tdata *td = (dns_tdata*)data;
+	struct hostent *hp;
+
+	if(!(hp = gethostbyname(td->hostname)))
+		goto failure;
+	memset(&sin, 0, sizeof(struct sockaddr_in));
+	memcpy(&sin.sin_addr.s_addr, hp->h_addr, hp->h_length);
+	sin.sin_family = hp->h_addrtype;
+	sin.sin_port = htons(td->port);
+
+	td->hosts = g_slist_append(td->hosts, GINT_TO_POINTER(sizeof(sin)));
+	td->hosts = g_slist_append(td->hosts, g_memdup(&sin, sizeof(sin)));
+	/* back to main thread */
+	g_idle_add(dns_main_thread_cb, td);
+	return 0;
+ failure:
+	g_free(td->hostname);
+	g_free(td);
+	return 0;
+} 
+
+int gaim_gethostbyname_async(const char *hostname, int port,
+							  dns_callback_t callback, gpointer data) {
+	dns_tdata *td;
+	struct sockaddr_in sin;
+	GError* err = NULL;
+
+	if(inet_aton(hostname, &sin.sin_addr)) {
+		GSList *hosts = NULL;
+		sin.sin_family = AF_INET;
+		sin.sin_port = htons(port);
+		hosts = g_slist_append(hosts, GINT_TO_POINTER(sizeof(sin)));
+		hosts = g_slist_append(hosts, g_memdup(&sin, sizeof(sin)));
+		callback(hosts, data, NULL);
+		return 0;
+	}
+
+	gaim_debug_info("dns", "DNS Lookup for: %s\n", hostname);
+	td = g_new0(dns_tdata, 1);
+	td->hostname = g_strdup(hostname);
+	td->port = port;
+	td->callback = callback;
+	td->data = data;
+
+	if(!g_thread_create(dns_thread, td, FALSE, &err)) {
+		gaim_debug_error("dns", "DNS thread create failure: %s\n", err?err->message:"");
+		g_error_free(err);
+		g_free(td->hostname);
+		g_free(td);
+		return -1;
+	}
+	return 0;
+}
+
+#else /* not __unix__ or _WIN32 */
 
 typedef struct {
 	gpointer data;
@@ -625,8 +727,6 @@ static gboolean host_resolved(gpointer data)
 	hosts = g_slist_append(hosts, GINT_TO_POINTER(req->addrlen));
 	hosts = g_slist_append(hosts, req->addr);
 	req->callback(hosts, req->data, NULL);
-	g_slist_free(hosts);
-	g_free(req->addr);
 	g_free(req);
 	return FALSE;
 }
@@ -662,7 +762,7 @@ gaim_gethostbyname_async(const char *hostname, int port,
 	return 0;
 }
 
-#endif
+#endif /* not __unix__ or _WIN32 */
 
 static void
 no_one_calls(gpointer data, gint source, GaimInputCondition cond)
@@ -688,23 +788,16 @@ no_one_calls(gpointer data, gint source, GaimInputCondition cond)
 	 */
 	ret = getsockopt(source, SOL_SOCKET, SO_ERROR, &error, &len);
 	if (ret == 0 && error == EINPROGRESS)
-		return; // we'll be called again later
+		return; /* we'll be called again later */
 	if (ret < 0 || error != 0) {
 		if(ret!=0) error = errno;
 		close(source);
 		gaim_input_remove(phb->inpa);
 
-		if (phb->account == NULL ||
-			gaim_account_get_connection(phb->account) != NULL) {
-
-			phb->func(phb->data, -1, GAIM_INPUT_READ);
-		}
-
-		g_free(phb->host);
-		g_free(phb);
-
 		gaim_debug(GAIM_DEBUG_ERROR, "proxy",
 			   "getsockopt SO_ERROR check: %s\n", strerror(error));
+
+		try_connect(phb);
 		return;
 	}
 
@@ -752,6 +845,9 @@ proxy_connect_none(struct PHB *phb, struct sockaddr *addr, socklen_t addrlen)
 		return -1;
 	}
 	fcntl(fd, F_SETFL, O_NONBLOCK);
+#ifndef _WIN32
+	fcntl(fd, F_SETFD, FD_CLOEXEC);
+#endif
 
 	if (connect(fd, (struct sockaddr *)addr, addrlen) < 0) {
 		if ((errno == EINPROGRESS) || (errno == EINTR)) {
@@ -793,10 +889,13 @@ static void
 http_complete(struct PHB *phb, gint source)
 {
 	gaim_debug(GAIM_DEBUG_INFO, "http proxy", "proxy connection established\n");
-	if(!phb->account || phb->account->gc)
+	if(source < 0) {
+		try_connect(phb);
+	} else if(!phb->account || phb->account->gc) {
 		phb->func(phb->data, source, GAIM_INPUT_READ);
-	g_free(phb->host);
-	g_free(phb);
+		g_free(phb->host);
+		g_free(phb);
+	}
 }
 
 
@@ -836,7 +935,7 @@ http_canread(gpointer data, gint source, GaimInputCondition cond)
 			}
 		}
 	}
-	
+
 	if(error) {
 		gaim_debug(GAIM_DEBUG_ERROR, "proxy",
 				   "Unable to parse proxy's response: %s\n", inputline);
@@ -846,9 +945,10 @@ http_canread(gpointer data, gint source, GaimInputCondition cond)
 	else if(status!=200) {
 		gaim_debug(GAIM_DEBUG_ERROR, "proxy",
 				   "Proxy server replied with:\n%s\n", p);
- 		close(source);
+		close(source);
 		source = -1;
-		
+
+		/* XXX: why in the hell are we calling gaim_connection_error() here? */
 		if ( status == 403 /* Forbidden */ ) {
 			gchar *msg = g_strdup_printf(_("Access denied: proxy server forbids port %d tunnelling."), phb->port);
 			gaim_connection_error(phb->account->gc, msg);
@@ -858,11 +958,11 @@ http_canread(gpointer data, gint source, GaimInputCondition cond)
 			gaim_connection_error(phb->account->gc, msg);
 			g_free(msg);
 		}
-		
+
 	} else {
 		http_complete(phb, source);
 	}
-	
+
 	return;
 }
 
@@ -885,14 +985,7 @@ http_canwrite(gpointer data, gint source, GaimInputCondition cond)
 	if (getsockopt(source, SOL_SOCKET, SO_ERROR, &error, &len) < 0) {
 		close(source);
 
-		if (phb->account == NULL ||
-			gaim_account_get_connection(phb->account) != NULL) {
-
-			phb->func(phb->data, -1, GAIM_INPUT_READ);
-		}
-
-		g_free(phb->host);
-		g_free(phb);
+		try_connect(phb);
 		return;
 	}
 
@@ -923,14 +1016,7 @@ http_canwrite(gpointer data, gint source, GaimInputCondition cond)
 	if (write(source, request, request_len) < 0) {
 		close(source);
 
-		if (phb->account == NULL ||
-			gaim_account_get_connection(phb->account) != NULL) {
-
-			phb->func(phb->data, -1, GAIM_INPUT_READ);
-		}
-
-		g_free(phb->host);
-		g_free(phb);
+		try_connect(phb);
 		return;
 	}
 
@@ -954,12 +1040,15 @@ proxy_connect_http(struct PHB *phb, struct sockaddr *addr, socklen_t addrlen)
 	}
 
 	fcntl(fd, F_SETFL, O_NONBLOCK);
+#ifndef _WIN32
+	fcntl(fd, F_SETFD, FD_CLOEXEC);
+#endif
 
 	if (connect(fd, addr, addrlen) < 0) {
 		if ((errno == EINPROGRESS) || (errno == EINTR)) {
 			gaim_debug(GAIM_DEBUG_WARNING, "http proxy",
 					   "Connect would have blocked.\n");
-			
+
 			if (phb->port != 80) {
 				/* we need to do CONNECT first */
 				phb->inpa = gaim_input_add(fd, GAIM_INPUT_WRITE,
@@ -1015,14 +1104,7 @@ s4_canread(gpointer data, gint source, GaimInputCondition cond)
 
 	close(source);
 
-	if (phb->account == NULL ||
-		gaim_account_get_connection(phb->account) != NULL) {
-
-		phb->func(phb->data, -1, GAIM_INPUT_READ);
-	}
-
-	g_free(phb->host);
-	g_free(phb);
+	try_connect(phb);
 }
 
 static void
@@ -1044,14 +1126,7 @@ s4_canwrite(gpointer data, gint source, GaimInputCondition cond)
 	if (getsockopt(source, SOL_SOCKET, SO_ERROR, &error, &len) < 0) {
 		close(source);
 
-		if (phb->account == NULL ||
-			gaim_account_get_connection(phb->account) != NULL) {
-
-			phb->func(phb->data, -1, GAIM_INPUT_READ);
-		}
-
-		g_free(phb->host);
-		g_free(phb);
+		try_connect(phb);
 		return;
 	}
 	fcntl(source, F_SETFL, 0);
@@ -1060,14 +1135,7 @@ s4_canwrite(gpointer data, gint source, GaimInputCondition cond)
 	if (!(hp = gethostbyname(phb->host))) {
 		close(source);
 
-		if (phb->account == NULL ||
-			gaim_account_get_connection(phb->account) != NULL) {
-
-			phb->func(phb->data, -1, GAIM_INPUT_READ);
-		}
-
-		g_free(phb->host);
-		g_free(phb);
+		try_connect(phb);
 		return;
 	}
 
@@ -1084,14 +1152,7 @@ s4_canwrite(gpointer data, gint source, GaimInputCondition cond)
 	if (write(source, packet, 9) != 9) {
 		close(source);
 
-		if (phb->account == NULL ||
-			gaim_account_get_connection(phb->account) != NULL) {
-
-			phb->func(phb->data, -1, GAIM_INPUT_READ);
-		}
-
-		g_free(phb->host);
-		g_free(phb);
+		try_connect(phb);
 		return;
 	}
 
@@ -1113,6 +1174,9 @@ proxy_connect_socks4(struct PHB *phb, struct sockaddr *addr, socklen_t addrlen)
 		return -1;
 
 	fcntl(fd, F_SETFL, O_NONBLOCK);
+#ifndef _WIN32
+	fcntl(fd, F_SETFD, FD_CLOEXEC);
+#endif
 
 	if (connect(fd, addr, addrlen) < 0) {
 		if ((errno == EINPROGRESS) || (errno == EINTR)) {
@@ -1158,14 +1222,7 @@ s5_canread_again(gpointer data, gint source, GaimInputCondition cond)
 		gaim_debug(GAIM_DEBUG_WARNING, "socks5 proxy", "or not...\n");
 		close(source);
 
-		if (phb->account == NULL ||
-			gaim_account_get_connection(phb->account) != NULL) {
-
-			phb->func(phb->data, source, GAIM_INPUT_READ);
-		}
-
-		g_free(phb->host);
-		g_free(phb);
+		try_connect(phb);
 		return;
 	}
 	if ((buf[0] != 0x05) || (buf[1] != 0x00)) {
@@ -1175,14 +1232,7 @@ s5_canread_again(gpointer data, gint source, GaimInputCondition cond)
 			gaim_debug(GAIM_DEBUG_ERROR, "socks5 proxy", "Bad data.\n");
 		close(source);
 
-		if (phb->account == NULL ||
-			gaim_account_get_connection(phb->account) != NULL) {
-
-			phb->func(phb->data, -1, GAIM_INPUT_READ);
-		}
-
-		g_free(phb->host);
-		g_free(phb);
+		try_connect(phb);
 		return;
 	}
 
@@ -1234,14 +1284,7 @@ s5_sendconnect(gpointer data, gint source)
 	if (write(source, buf, (5 + strlen(phb->host) + 2)) < (5 + strlen(phb->host) + 2)) {
 		close(source);
 
-		if (phb->account == NULL ||
-			gaim_account_get_connection(phb->account) != NULL) {
-
-			phb->func(phb->data, -1, GAIM_INPUT_READ);
-		}
-
-		g_free(phb->host);
-		g_free(phb);
+		try_connect(phb);
 		return;
 	}
 
@@ -1260,28 +1303,14 @@ s5_readauth(gpointer data, gint source, GaimInputCondition cond)
 	if (read(source, buf, 2) < 2) {
 		close(source);
 
-		if (phb->account == NULL ||
-			gaim_account_get_connection(phb->account) != NULL) {
-
-			phb->func(phb->data, -1, GAIM_INPUT_READ);
-		}
-
-		g_free(phb->host);
-		g_free(phb);
+		try_connect(phb);
 		return;
 	}
 
 	if ((buf[0] != 0x01) || (buf[1] != 0x00)) {
 		close(source);
 
-		if (phb->account == NULL ||
-			gaim_account_get_connection(phb->account) != NULL) {
-
-			phb->func(phb->data, -1, GAIM_INPUT_READ);
-		}
-
-		g_free(phb->host);
-		g_free(phb);
+		try_connect(phb);
 		return;
 	}
 
@@ -1300,34 +1329,20 @@ s5_canread(gpointer data, gint source, GaimInputCondition cond)
 	if (read(source, buf, 2) < 2) {
 		close(source);
 
-		if (phb->account == NULL ||
-			gaim_account_get_connection(phb->account) != NULL) {
-
-			phb->func(phb->data, source, GAIM_INPUT_READ);
-		}
-
-		g_free(phb->host);
-		g_free(phb);
+		try_connect(phb);
 		return;
 	}
 
 	if ((buf[0] != 0x05) || (buf[1] == 0xff)) {
 		close(source);
 
-		if (phb->account == NULL ||
-			gaim_account_get_connection(phb->account) != NULL) {
-
-			phb->func(phb->data, -1, GAIM_INPUT_READ);
-		}
-
-		g_free(phb->host);
-		g_free(phb);
+		try_connect(phb);
 		return;
 	}
 
 	if (buf[1] == 0x02) {
 		unsigned int i, j;
-		
+
 		i = strlen(gaim_proxy_info_get_username(phb->gpi));
 		j = strlen(gaim_proxy_info_get_password(phb->gpi));
 
@@ -1340,14 +1355,7 @@ s5_canread(gpointer data, gint source, GaimInputCondition cond)
 		if (write(source, buf, 3 + i + j) < 3 + i + j) {
 			close(source);
 
-			if (phb->account == NULL ||
-				gaim_account_get_connection(phb->account) != NULL) {
-
-				phb->func(phb->data, -1, GAIM_INPUT_READ);
-			}
-
-			g_free(phb->host);
-			g_free(phb);
+			try_connect(phb);
 			return;
 		}
 
@@ -1375,14 +1383,8 @@ s5_canwrite(gpointer data, gint source, GaimInputCondition cond)
 	len = sizeof(error);
 	if (getsockopt(source, SOL_SOCKET, SO_ERROR, &error, &len) < 0) {
 		close(source);
-		if (phb->account == NULL ||
-			gaim_account_get_connection(phb->account) != NULL) {
 
-			phb->func(phb->data, -1, GAIM_INPUT_READ);
-		}
-
-		g_free(phb->host);
-		g_free(phb);
+		try_connect(phb);
 		return;
 	}
 	fcntl(source, F_SETFL, 0);
@@ -1406,14 +1408,7 @@ s5_canwrite(gpointer data, gint source, GaimInputCondition cond)
 		gaim_debug(GAIM_DEBUG_ERROR, "socks5 proxy", "Unable to write\n");
 		close(source);
 
-		if (phb->account == NULL ||
-			gaim_account_get_connection(phb->account) != NULL) {
-
-			phb->func(phb->data, -1, GAIM_INPUT_READ);
-		}
-
-		g_free(phb->host);
-		g_free(phb);
+		try_connect(phb);
 		return;
 	}
 
@@ -1435,6 +1430,9 @@ proxy_connect_socks5(struct PHB *phb, struct sockaddr *addr, socklen_t addrlen)
 		return -1;
 
 	fcntl(fd, F_SETFL, O_NONBLOCK);
+#ifndef _WIN32
+	fcntl(fd, F_SETFD, FD_CLOEXEC);
+#endif
 
 	if (connect(fd, addr, addrlen) < 0) {
 		if ((errno == EINPROGRESS) || (errno == EINTR)) {
@@ -1469,23 +1467,19 @@ proxy_connect_socks5(struct PHB *phb, struct sockaddr *addr, socklen_t addrlen)
 	return fd;
 }
 
-static void
-connection_host_resolved(GSList *hosts, gpointer data,
-						 const char *error_message)
+static void try_connect(struct PHB *phb)
 {
-	struct PHB *phb = (struct PHB*)data;
 	size_t addrlen;
 	struct sockaddr *addr;
 	int ret = -1;
 
-	while (hosts) {
-		addrlen = GPOINTER_TO_INT(hosts->data);
-		hosts = hosts->next;
-		addr = hosts->data;
-		hosts = hosts->next;
+	while (phb->hosts) {
+		addrlen = GPOINTER_TO_INT(phb->hosts->data);
+		phb->hosts = g_slist_remove(phb->hosts, phb->hosts->data);
+		addr = phb->hosts->data;
+		phb->hosts = g_slist_remove(phb->hosts, phb->hosts->data);
 
-		switch (gaim_proxy_info_get_type(phb->gpi))
-		{
+		switch (gaim_proxy_info_get_type(phb->gpi)) {
 			case GAIM_PROXY_NONE:
 				ret = proxy_connect_none(phb, addr, addrlen);
 				break;
@@ -1510,6 +1504,8 @@ connection_host_resolved(GSList *hosts, gpointer data,
 				break;
 		}
 
+		g_free(addr);
+
 		if (ret > 0)
 			break;
 	}
@@ -1526,6 +1522,17 @@ connection_host_resolved(GSList *hosts, gpointer data,
 	}
 }
 
+static void
+connection_host_resolved(GSList *hosts, gpointer data,
+						 const char *error_message)
+{
+	struct PHB *phb = (struct PHB*)data;
+
+	phb->hosts = hosts;
+
+	try_connect(phb);
+}
+
 int
 gaim_proxy_connect(GaimAccount *account, const char *host, int port,
 				   GaimInputFunction func, gpointer data)
@@ -1555,29 +1562,42 @@ gaim_proxy_connect(GaimAccount *account, const char *host, int port,
 	if (gaim_proxy_info_get_type(phb->gpi) == GAIM_PROXY_USE_ENVVAR) {
 		if ((tmp = g_getenv("HTTP_PROXY")) != NULL ||
 			(tmp = g_getenv("http_proxy")) != NULL ||
-			(tmp= g_getenv("HTTPPROXY")) != NULL) {
-			char *proxyhost,*proxypath;
+			(tmp = g_getenv("HTTPPROXY")) != NULL) {
+			char *proxyhost,*proxypath,*proxyuser,*proxypasswd;
 			int proxyport;
 
-			/* http_proxy-format: 
-			 * export http_proxy="http://your.proxy.server:port/" 
+			/* http_proxy-format:
+			 * export http_proxy="http://user:passwd@your.proxy.server:port/"
 			 */
-			if(gaim_url_parse(tmp, &proxyhost, &proxyport, &proxypath)) {
+			if(gaim_url_parse(tmp, &proxyhost, &proxyport, &proxypath, &proxyuser, &proxypasswd)) {
 				gaim_proxy_info_set_host(phb->gpi, proxyhost);
 				g_free(proxyhost);
 				g_free(proxypath);
+				if (proxyuser != NULL) {
+					gaim_proxy_info_set_username(phb->gpi, proxyuser);
+					g_free(proxyuser);
+				}
+				if (proxypasswd != NULL) {
+					gaim_proxy_info_set_password(phb->gpi, proxypasswd);
+					g_free(proxypasswd);
+				}
 
 				/* only for backward compatibility */
-				if (proxyport == 80 && 
+				if (proxyport == 80 &&
 				    ((tmp = g_getenv("HTTP_PROXY_PORT")) != NULL ||
 				     (tmp = g_getenv("http_proxy_port")) != NULL ||
 				     (tmp = g_getenv("HTTPPROXYPORT")) != NULL))
 				    proxyport = atoi(tmp);
-				
+
 				gaim_proxy_info_set_port(phb->gpi, proxyport);
 			}
+		} else {
+			/* no proxy environment variable found, don't use a proxy */
+			gaim_debug_info("proxy", "No environment settings found, not using a proxy\n");
+			gaim_proxy_info_set_type(phb->gpi, GAIM_PROXY_NONE);
 		}
 
+		/* XXX: Do we want to skip this step if user/password were part of url? */
 		if ((tmp = g_getenv("HTTP_PROXY_USER")) != NULL ||
 			(tmp = g_getenv("http_proxy_user")) != NULL ||
 			(tmp = g_getenv("HTTPPROXYUSER")) != NULL)
@@ -1699,4 +1719,8 @@ gaim_proxy_init(void)
 								proxy_pref_cb, NULL);
 	gaim_prefs_connect_callback("/core/proxy/password",
 								proxy_pref_cb, NULL);
+#ifdef _WIN32
+	if(!g_thread_supported())
+		g_thread_init(NULL);
+#endif
 }
