@@ -24,7 +24,6 @@
 #include "conversation.h"
 #include "debug.h"
 #include "log.h"
-#include "multi.h"
 #include "notify.h"
 #include "prefs.h"
 #include "prpl.h"
@@ -35,12 +34,17 @@
 #include "util.h"
 
 /* XXX UI Stuff */
+#include "away.h"
+#include "gtkdialogs.h"
 #include "gaim.h"
 #include "gtkimhtml.h"
 #include "gtkutils.h"
-#include "ui.h"
 
 #define SECS_BEFORE_RESENDING_AUTORESPONSE 600
+#define SEX_BEFORE_RESENDING_AUTORESPONSE "Only after you're married"
+
+static void add_idle_buddy(GaimBuddy *buddy);
+static void remove_idle_buddy(GaimBuddy *buddy);
 
 void serv_login(GaimAccount *account)
 {
@@ -61,9 +65,10 @@ void serv_login(GaimAccount *account)
 			return;
 		}
 
-		gaim_debug(GAIM_DEBUG_INFO, "server",
-				   PACKAGE " " VERSION " logging in %s using %s\n",
-				   account->username, p->info->name);
+		gaim_debug_info("server", PACKAGE " " VERSION
+						" logging in %s using %s\n",
+						gaim_account_get_username(account),
+						gaim_account_get_protocol_name(account));
 
 		gaim_signal_emit(gaim_accounts_get_handle(),
 						 "account-connecting", account);
@@ -99,7 +104,6 @@ static void update_keepalive(GaimConnection *gc, gboolean on)
 
 void serv_close(GaimConnection *gc)
 {
-	GaimPlugin *prpl;
 	GaimPluginProtocolInfo *prpl_info = NULL;
 
 	while (gc->buddy_chats) {
@@ -121,8 +125,6 @@ void serv_close(GaimConnection *gc)
 		if (prpl_info->close)
 			(prpl_info->close)(gc);
 	}
-
-	prpl = gc->prpl;
 }
 
 void serv_touch_idle(GaimConnection *gc)
@@ -257,6 +259,7 @@ int serv_send_im(GaimConnection *gc, const char *name, const char *message,
 	GaimConversation *c;
 	int val = -EINVAL;
 	GaimPluginProtocolInfo *prpl_info = NULL;
+	const gchar *auto_reply_pref;
 
 	if (gc != NULL && gc->prpl != NULL)
 		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
@@ -269,8 +272,13 @@ int serv_send_im(GaimConnection *gc, const char *name, const char *message,
 	if (!(imflags & GAIM_CONV_IM_AUTO_RESP))
 		serv_touch_idle(gc);
 
+	/*
+	 * XXX - If "only auto-reply when away & idle" is set, then shouldn't
+	 * this only reset lar->sent if we're away AND idle?
+	 */
+	auto_reply_pref = gaim_prefs_get_string("/core/away/auto_reply");
 	if (gc->away &&	(gc->flags & GAIM_CONNECTION_AUTO_RESP) &&
-		gaim_prefs_get_bool("/core/away/auto_response/enabled")) {
+		strcmp(auto_reply_pref, "never")) {
 
 		struct last_auto_response *lar;
 		lar = get_last_auto_response(gc, name);
@@ -283,15 +291,15 @@ int serv_send_im(GaimConnection *gc, const char *name, const char *message,
 	return val;
 }
 
-void serv_get_info(GaimConnection *g, const char *name)
+void serv_get_info(GaimConnection *gc, const char *name)
 {
 	GaimPluginProtocolInfo *prpl_info = NULL;
 
-	if (g != NULL && g->prpl != NULL)
-		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(g->prpl);
+	if (gc != NULL && gc->prpl != NULL)
+		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
 
-	if (g && prpl_info && prpl_info->get_info)
-		prpl_info->get_info(g, name);
+	if (gc && prpl_info && prpl_info->get_info)
+		prpl_info->get_info(gc, name);
 }
 
 void serv_set_away(GaimConnection *gc, const char *state, const char *message)
@@ -351,50 +359,50 @@ void serv_set_away_all(const char *message)
 	}
 }
 
-void serv_set_info(GaimConnection *g, const char *info)
+void serv_set_info(GaimConnection *gc, const char *info)
 {
 	GaimPluginProtocolInfo *prpl_info = NULL;
 	GaimAccount *account;
 
-	if (g != NULL && g->prpl != NULL)
-		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(g->prpl);
+	if (gc != NULL && gc->prpl != NULL)
+		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
 
-	if (prpl_info && g_list_find(gaim_connections_get_all(), g) &&
+	if (prpl_info && g_list_find(gaim_connections_get_all(), gc) &&
 		prpl_info->set_info) {
 
-		account = gaim_connection_get_account(g);
+		account = gaim_connection_get_account(gc);
 
 		if (gaim_signal_emit_return_1(gaim_accounts_get_handle(),
 									  "account-setting-info", account, info))
 			return;
 
-		prpl_info->set_info(g, info);
+		prpl_info->set_info(gc, info);
 
 		gaim_signal_emit(gaim_accounts_get_handle(),
 						 "account-set-info", account, info);
 	}
 }
 
-void serv_change_passwd(GaimConnection *g, const char *orig, const char *new)
+void serv_change_passwd(GaimConnection *gc, const char *orig, const char *new)
 {
 	GaimPluginProtocolInfo *prpl_info = NULL;
 
-	if (g != NULL && g->prpl != NULL)
-		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(g->prpl);
+	if (gc != NULL && gc->prpl != NULL)
+		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
 
-	if (prpl_info && g_list_find(gaim_connections_get_all(), g) && prpl_info->change_passwd)
-		prpl_info->change_passwd(g, orig, new);
+	if (prpl_info && g_list_find(gaim_connections_get_all(), gc) && prpl_info->change_passwd)
+		prpl_info->change_passwd(gc, orig, new);
 }
 
-void serv_add_buddy(GaimConnection *g, const char *name, GaimGroup *group)
+void serv_add_buddy(GaimConnection *gc, GaimBuddy *buddy)
 {
 	GaimPluginProtocolInfo *prpl_info = NULL;
 
-	if (g != NULL && g->prpl != NULL)
-		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(g->prpl);
+	if (gc != NULL && gc->prpl != NULL)
+		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
 
-	if (prpl_info && g_list_find(gaim_connections_get_all(), g) && prpl_info->add_buddy)
-		prpl_info->add_buddy(g, name, group);
+	if (prpl_info && g_list_find(gaim_connections_get_all(), gc) && prpl_info->add_buddy)
+		prpl_info->add_buddy(gc, buddy, gaim_find_buddys_group(buddy));
 }
 
 void serv_add_buddies(GaimConnection *gc, GList *buddies)
@@ -405,31 +413,77 @@ void serv_add_buddies(GaimConnection *gc, GList *buddies)
 		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
 
 	if (prpl_info && g_list_find(gaim_connections_get_all(), gc)) {
+		GList *cur, *groups = NULL;
+
+		/* Make a list of what the groups each buddy is in */
+		for (cur = buddies; cur != NULL; cur = cur->next) {
+			GaimBlistNode *node = cur->data;
+			groups = g_list_append(groups, node->parent->parent);
+		}
+
 		if (prpl_info->add_buddies)
-			prpl_info->add_buddies(gc, buddies);
+			prpl_info->add_buddies(gc, buddies, groups);
 		else if (prpl_info->add_buddy) {
-			while (buddies) {
-				prpl_info->add_buddy(gc, buddies->data, NULL);
-				buddies = buddies->next;
+			GList *curb = buddies;
+			GList *curg = groups;
+			while ((curb != NULL) && (curg != NULL)) {
+				prpl_info->add_buddy(gc, curb->data, curg->data);
+				curb = curb->next;
+				curg = curg->next;
+			}
+		}
+
+		g_list_free(groups);
+	}
+}
+
+
+void serv_remove_buddy(GaimConnection *gc, GaimBuddy *buddy, GaimGroup *group)
+{
+	GaimPluginProtocolInfo *prpl_info = NULL;
+
+	if (buddy->idle > 0)
+		remove_idle_buddy(buddy);
+
+	if (gc != NULL && gc->prpl != NULL)
+		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
+
+	if (prpl_info && g_list_find(gaim_connections_get_all(), gc) && prpl_info->remove_buddy)
+		prpl_info->remove_buddy(gc, buddy, group);
+}
+
+void serv_remove_buddies(GaimConnection *gc, GList *buddies, GList *groups)
+{
+	GaimPluginProtocolInfo *prpl_info = NULL;
+
+	if (!g_list_find(gaim_connections_get_all(), gc))
+		return;
+
+	if (gc != NULL && gc->prpl != NULL)
+		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
+
+	if (prpl_info && g_list_find(gaim_connections_get_all(), gc)) {
+		if (prpl_info->remove_buddies) {
+			GList *curb;
+			for (curb = buddies; curb != NULL; curb = curb->next) {
+				GaimBuddy *buddy = curb->data;
+				if (buddy->idle > 0)
+					remove_idle_buddy(buddy);
+			}
+			prpl_info->remove_buddies(gc, buddies, groups);
+		} else {
+			GList *curb = buddies;
+			GList *curg = groups;
+			while ((curb != NULL) && (curg != NULL)) {
+				serv_remove_buddy(gc, curb->data, curg->data);
+				curb = curb->next;
+				curg = curg->next;
 			}
 		}
 	}
 }
 
-
-void serv_remove_buddy(GaimConnection *g, const char *name, const char *group)
-{
-	GaimPluginProtocolInfo *prpl_info = NULL;
-
-	if (g != NULL && g->prpl != NULL)
-		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(g->prpl);
-
-	if (prpl_info && g_list_find(gaim_connections_get_all(), g) && prpl_info->remove_buddy)
-		prpl_info->remove_buddy(g, name, group);
-}
-
-void
-serv_remove_group(GaimConnection *gc, const char *name)
+void serv_remove_group(GaimConnection *gc, GaimGroup *group)
 {
 	GaimPluginProtocolInfo *prpl_info = NULL;
 
@@ -439,30 +493,7 @@ serv_remove_group(GaimConnection *gc, const char *name)
 	if (prpl_info && g_list_find(gaim_connections_get_all(), gc) &&
 		prpl_info->remove_group)
 	{
-		prpl_info->remove_group(gc, name);
-	}
-}
-
-void serv_remove_buddies(GaimConnection *gc, GList *g, const char *group)
-{
-	GaimPluginProtocolInfo *prpl_info = NULL;
-
-	if (!g_list_find(gaim_connections_get_all(), gc))
-		return;
-
-	if (!gc->prpl)
-		return;		/* how the hell did that happen? */
-
-	if (gc != NULL && gc->prpl != NULL)
-		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
-
-	if (prpl_info->remove_buddies)
-		prpl_info->remove_buddies(gc, g, group);
-	else {
-		while (g) {
-			serv_remove_buddy(gc, g->data, group);
-			g = g->next;
-		}
+		prpl_info->remove_group(gc, group);
 	}
 }
 
@@ -481,13 +512,32 @@ void serv_alias_buddy(GaimBuddy *b)
 	}
 }
 
-void serv_got_alias(GaimConnection *gc, const char *who, const char *alias) {
-	GSList *buds, *buddies = gaim_find_buddies(gc->account, who);
+void
+serv_got_alias(GaimConnection *gc, const char *who, const char *alias)
+{
+	GaimAccount *account = gaim_connection_get_account(gc);
+	GSList *buds, *buddies = gaim_find_buddies(account, who);
 	GaimBuddy *b;
+	GaimConversation *conv;
 
-	for(buds = buddies; buds; buds = buds->next) {
+	for (buds = buddies; buds; buds = buds->next)
+	{
 		b = buds->data;
 		gaim_blist_server_alias_buddy(b, alias);
+
+		conv = gaim_find_conversation_with_account(b->name, account);
+
+		if (conv != NULL && b->server_alias != NULL &&
+			strcmp(b->server_alias, alias))
+		{
+			char *tmp = g_strdup_printf(_("%s is now known as %s.\n"),
+										who, alias);
+
+			gaim_conversation_write(conv, NULL, tmp, GAIM_MESSAGE_SYSTEM,
+									time(NULL));
+
+			g_free(tmp);
+		}
 	}
 	g_slist_free(buddies);
 }
@@ -496,7 +546,8 @@ void serv_got_alias(GaimConnection *gc, const char *who, const char *alias) {
  * Move a buddy from one group to another on server.
  *
  * Note: For now we'll not deal with changing gc's at the same time, but
- * it should be possible.  Probably needs to be done, someday.
+ * it should be possible.  Probably needs to be done, someday.  Although,
+ * the UI for that would be difficult, because groups are Gaim-wide.
  */
 void serv_move_buddy(GaimBuddy *b, GaimGroup *og, GaimGroup *ng)
 {
@@ -515,44 +566,32 @@ void serv_move_buddy(GaimBuddy *b, GaimGroup *og, GaimGroup *ng)
 /*
  * Rename a group on server roster/list.
  */
-void serv_rename_group(GaimConnection *g, GaimGroup *old_group,
-					   const char *new_name)
+void serv_rename_group(GaimConnection *gc, const char *old_name,
+					   GaimGroup *group, GList *moved_buddies)
 {
 	GaimPluginProtocolInfo *prpl_info = NULL;
 
-	if (g != NULL && g->prpl != NULL)
-		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(g->prpl);
-
-	if (prpl_info && old_group && new_name) {
-		GList *tobemoved = NULL;
-		GaimBlistNode *cnode, *bnode;
-
-		for(cnode = ((GaimBlistNode*)old_group)->child; cnode; cnode = cnode->next) {
-			if(!GAIM_BLIST_NODE_IS_CONTACT(cnode))
-				continue;
-			for(bnode = cnode->child; bnode; bnode = bnode->next) {
-				GaimBuddy *b;
-				if(!GAIM_BLIST_NODE_IS_BUDDY(bnode))
-					continue;
-				b = (GaimBuddy*)bnode;
-
-				if(b->account == g->account)
-					tobemoved = g_list_append(tobemoved, b->name);
-
-			}
-
-		}
+	if (gc != NULL && gc->prpl != NULL)
+		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
 
+	if (prpl_info && old_name && group && strcmp(old_name, group->name)) {
 		if (prpl_info->rename_group) {
 			/* prpl's might need to check if the group already
 			 * exists or not, and handle that differently */
-			prpl_info->rename_group(g, old_group->name, new_name, tobemoved);
+			prpl_info->rename_group(gc, old_name, group, moved_buddies);
 		} else {
-			serv_remove_buddies(g, tobemoved, old_group->name);
-			serv_add_buddies(g, tobemoved);
-		}
+			GList *cur, *groups = NULL;
 
-		g_list_free(tobemoved);
+			/* Make a list of what the groups each buddy is in */
+			for (cur = moved_buddies; cur != NULL; cur = cur->next) {
+				GaimBlistNode *node = cur->data;
+				groups = g_list_append(groups, node->parent->parent);
+			}
+
+			serv_remove_buddies(gc, moved_buddies, groups);
+			g_list_free(groups);
+			serv_add_buddies(gc, moved_buddies);
+		}
 	}
 }
 
@@ -629,7 +668,7 @@ void serv_set_idle(GaimConnection *g, int time)
 		prpl_info->set_idle(g, time);
 }
 
-void serv_warn(GaimConnection *g, const char *name, int anon)
+void serv_warn(GaimConnection *g, const char *name, gboolean anonymous)
 {
 	GaimPluginProtocolInfo *prpl_info = NULL;
 
@@ -637,7 +676,7 @@ void serv_warn(GaimConnection *g, const char *name, int anon)
 		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(g->prpl);
 
 	if (prpl_info && g_list_find(gaim_connections_get_all(), g) && prpl_info->warn)
-		prpl_info->warn(g, name, anon);
+		prpl_info->warn(g, name, anonymous);
 }
 
 void serv_join_chat(GaimConnection *g, GHashTable *data)
@@ -797,6 +836,7 @@ void serv_got_im(GaimConnection *gc, const char *who, const char *msg,
 {
 	GaimConversation *cnv;
 	GaimMessageFlags msgflags;
+	gboolean queue_to_away, queue_to_docklet;
 	char *message, *name;
 	char *angel, *buffy;
 	int plugin_return;
@@ -845,14 +885,29 @@ void serv_got_im(GaimConnection *gc, const char *who, const char *msg,
 	 * it's nice to pass whether or not it was an auto-response. So if it
 	 * was an auto-response, we set the appropriate flag. This is just so
 	 * prpls don't have to know about GAIM_MESSAGE_* (though some do anyway).
-	 * We also need to preserve the flag that tells the UI to look for the
-	 * associated images.
 	 */
 	msgflags = GAIM_MESSAGE_RECV;
 	if (imflags & GAIM_CONV_IM_AUTO_RESP)
 		msgflags |= GAIM_MESSAGE_AUTO_RESP;
-	if (imflags & GAIM_CONV_IM_IMAGES)
-		msgflags |= GAIM_MESSAGE_IMAGES;
+
+	/* queue to the docklet instead of writing to a conversation window if:
+	 *  - there is a docklet
+	 *  - there is no conversation window
+	 *  - the evil hack of a pref is enabled
+	 *
+	 * queue to away instead of writing to a conversation window if:
+	 *  - we are away
+	 *  - there is no conversation window
+	 *  - there *is* a global away window
+	 *  - away queueing is enabled
+	 *
+	 * Robot101 won't fix this any further because the code is already gone
+	 * in HEAD. Queue code however... :)
+	 */
+	queue_to_away    = (gc->away && !cnv && awayqueue &&
+			    gaim_prefs_get_bool("/gaim/gtk/away/queue_messages"));
+	queue_to_docklet = (docklet_count && !cnv &&
+			    gaim_prefs_get_bool("/plugins/gtk/docklet/queue_messages"));
 
 	/*
 	 * Alright. Two cases for how to handle this. Either we're away or
@@ -864,9 +919,10 @@ void serv_got_im(GaimConnection *gc, const char *who, const char *msg,
 		time_t t = time(NULL);
 		char *tmpmsg;
 		GaimBuddy *b = gaim_find_buddy(gc->account, name);
-		const char *alias = b ? gaim_get_buddy_alias(b) : name;
+		const char *alias = b ? gaim_buddy_get_alias(b) : name;
 		int row;
 		struct last_auto_response *lar;
+		const gchar *auto_reply_pref;
 
 		/*
 		 * Either we're going to queue it or not. Because of the way
@@ -877,17 +933,14 @@ void serv_got_im(GaimConnection *gc, const char *who, const char *msg,
 		 * this to be queued properly, we have to make sure that the
 		 * imaway dialog actually exists, first.
 		 */
-		if (!cnv && awayqueue &&
-			gaim_prefs_get_bool("/gaim/gtk/away/queue_messages")) {
+		if (queue_to_away || queue_to_docklet) {
 			/*
 			 * Alright, so we're going to queue it. Neat, eh? :)
 			 * So first we create something to store the message, and add
 			 * it to our queue. Then we update the away dialog to indicate
-			 * that we've queued something.
+			 * that we've queued something, unless we queued it to the docklet
 			 */
 			struct queued_message *qm;
-			GtkTreeIter iter;
-			gchar path[10];
 
 			qm = g_new0(struct queued_message, 1);
 			g_snprintf(qm->name, sizeof(qm->name), "%s", name);
@@ -897,31 +950,38 @@ void serv_got_im(GaimConnection *gc, const char *who, const char *msg,
 			qm->account = gc->account;
 			qm->tm = mtime;
 			qm->flags = msgflags;
-			message_queue = g_slist_append(message_queue, qm);
 
-			row = find_queue_row_by_name(qm->name);
-			if (row >= 0) {
-				char number[32];
-				int qtotal;
+			if (queue_to_away) {
+				GtkTreeIter iter;
+				gchar path[10];
 
-				qtotal = find_queue_total_by_name(qm->name);
-				g_snprintf(number, 32, ngettext("(%d message)",
-						   "(%d messages)", qtotal), qtotal);
-				g_snprintf(path, 10, "%d", row);
-				gtk_tree_model_get_iter_from_string(
-								GTK_TREE_MODEL(awayqueuestore), &iter, path);
-				gtk_list_store_set(awayqueuestore, &iter,
-								2, number, -1);
-			} else {
-				gtk_tree_model_get_iter_first(GTK_TREE_MODEL(awayqueuestore),
-								&iter);
-				gtk_list_store_append(awayqueuestore, &iter);
-				gtk_list_store_set(awayqueuestore, &iter,
-								0, qm->name,
-								1, qm->alias,
-								2, _("(1 message)"),
-								-1);
-			}
+				message_queue = g_slist_append(message_queue, qm);
+
+				row = find_queue_row_by_name(qm->name);
+				if (row >= 0) {
+					char number[32];
+					int qtotal;
+
+					qtotal = find_queue_total_by_name(qm->name);
+					g_snprintf(number, 32, ngettext("(%d message)",
+							   "(%d messages)", qtotal), qtotal);
+					g_snprintf(path, 10, "%d", row);
+					gtk_tree_model_get_iter_from_string(
+									GTK_TREE_MODEL(awayqueuestore), &iter, path);
+					gtk_list_store_set(awayqueuestore, &iter,
+									2, number, -1);
+				} else {
+					gtk_tree_model_get_iter_first(GTK_TREE_MODEL(awayqueuestore),
+									&iter);
+					gtk_list_store_append(awayqueuestore, &iter);
+					gtk_list_store_set(awayqueuestore, &iter,
+									0, qm->name,
+									1, qm->alias,
+									2, _("(1 message)"),
+									-1);
+				}
+			} else /* queue_to_docklet */
+				unread_message_queue = g_slist_append(unread_message_queue, qm);
 		} else {
 			/*
 			 * Make sure the conversation
@@ -933,6 +993,7 @@ void serv_got_im(GaimConnection *gc, const char *who, const char *msg,
 				cnv = gaim_conversation_new(GAIM_CONV_IM, gc->account, name);
 
 			gaim_conv_im_write(GAIM_CONV_IM(cnv), NULL, message, msgflags, mtime);
+			gaim_conv_window_flash(gaim_conversation_get_window(cnv));
 		}
 
 		/*
@@ -941,16 +1002,17 @@ void serv_got_im(GaimConnection *gc, const char *who, const char *msg,
 		 * Don't autorespond if:
 		 *
 		 *  - it's not supported on this connection
-		 *  - or it's disabled
 		 *  - or the away message is empty
+		 *  - or it's disabled
 		 *  - or we're not idle and the 'only auto respond if idle' pref
 		 *    is set
 		 */
+		auto_reply_pref = gaim_prefs_get_string("/core/away/auto_reply");
 		if (!(gc->flags & GAIM_CONNECTION_AUTO_RESP) ||
-			!gaim_prefs_get_bool("/core/away/auto_response/enabled") ||
 			*gc->away == '\0' ||
+			!strcmp(auto_reply_pref, "never") ||
 			(!gc->is_idle &&
-			 gaim_prefs_get_bool("/core/away/auto_response/idle_only"))) {
+			 !strcmp(auto_reply_pref, "awayidle"))) {
 
 			g_free(name);
 			g_free(message);
@@ -980,54 +1042,46 @@ void serv_got_im(GaimConnection *gc, const char *who, const char *msg,
 		/* Move this to oscar.c! */
 		buffy = gaim_str_sub_away_formatters(tmpmsg, alias);
 		serv_send_im(gc, name, buffy, GAIM_CONV_IM_AUTO_RESP);
-		g_free(buffy);
-
-		if (!cnv && awayqueue &&
-			gaim_prefs_get_bool("/gaim/gtk/away/queue_messages")) {
 
+		if (queue_to_away || queue_to_docklet) {
 			struct queued_message *qm;
 
 			qm = g_new0(struct queued_message, 1);
 			g_snprintf(qm->name, sizeof(qm->name), "%s", name);
-			qm->message = g_strdup(gaim_str_sub_away_formatters(tmpmsg, alias));
+			qm->message = g_strdup(buffy);
 			qm->account = gc->account;
 			qm->tm = mtime;
 			qm->flags = GAIM_MESSAGE_SEND | GAIM_MESSAGE_AUTO_RESP;
-			message_queue = g_slist_append(message_queue, qm);
+
+			if (queue_to_away)
+				message_queue = g_slist_append(message_queue, qm);
+			else /* queue_to_docklet */
+				unread_message_queue = g_slist_append(unread_message_queue, qm);
 		} else if (cnv != NULL)
-			gaim_conv_im_write(GAIM_CONV_IM(cnv), NULL, gaim_str_sub_away_formatters(tmpmsg, alias),
+			gaim_conv_im_write(GAIM_CONV_IM(cnv), NULL, buffy,
 						  GAIM_MESSAGE_SEND | GAIM_MESSAGE_AUTO_RESP, mtime);
 
+		g_free(buffy);
 		g_free(tmpmsg);
 	} else {
 		/*
-		 * We're not away. This is easy. If the convo window doesn't
-		 * exist, create and update it (if it does exist it was updated
-		 * earlier), then play a sound indicating we've received it and
-		 * then display it. Easy.
+		 * We're not away. This is easy. Queue it to the docklet if we're
+		 * doing that, or if the convo window doesn't exist, create it,
+		 * then display the message.
 		 */
 
-		/* XXX UGLY HACK OF THE YEAR
-		 * Robot101 will fix this after his exams. honest.
-		 * I guess he didn't specify WHICH exams, exactly...
-		 */
-		if (docklet_count &&
-		    gaim_prefs_get_bool("/plugins/gtk/docklet/queue_messages") &&
-		    !gaim_find_conversation_with_account(name, gc->account)) {
-			/*
-			 * We're gonna queue it up and wait for the user to ask for
-			 * it... probably by clicking the docklet or windows tray icon.
-			 */
+		if (queue_to_docklet) {
 			struct queued_message *qm;
+
 			qm = g_new0(struct queued_message, 1);
 			g_snprintf(qm->name, sizeof(qm->name), "%s", name);
 			qm->message = g_strdup(message);
 			qm->account = gc->account;
 			qm->tm = mtime;
 			qm->flags = msgflags;
+
 			unread_message_queue = g_slist_append(unread_message_queue, qm);
-		}
-		else {
+		} else {
 			if (cnv == NULL)
 				cnv = gaim_conversation_new(GAIM_CONV_IM, gc->account, name);
 
@@ -1040,14 +1094,74 @@ void serv_got_im(GaimConnection *gc, const char *who, const char *msg,
 	g_free(message);
 }
 
+/*
+ * NOTE: This is a bit hacky, but needed for core support for the
+ *       buddy-idle-updated signal. It's temporary, and will be replaced
+ *       with better code in the status rewrite.
+ */
+static GList *idle_buddies = NULL;
+static guint idle_buddy_timeout_id = 0;
 
+static gboolean
+idle_timeout_cb(void)
+{
+	GList *l, *l_next;
 
-void serv_got_update(GaimConnection *gc, const char *name, int loggedin,
+	for (l = idle_buddies; l != NULL; l = l_next)
+	{
+		GaimBuddy *buddy = (GaimBuddy *)l->data;
+
+		l_next = l->next;
+
+		if (!GAIM_BUDDY_IS_ONLINE(buddy) || buddy->idle <= 0)
+		{
+			remove_idle_buddy(buddy);
+		}
+		else
+		{
+			gaim_signal_emit(gaim_blist_get_handle(), "buddy-idle-updated",
+							 l->data);
+		}
+	}
+
+	return TRUE;
+}
+
+static void
+add_idle_buddy(GaimBuddy *buddy)
+{
+	idle_buddies = g_list_append(idle_buddies, buddy);
+
+	if (idle_buddy_timeout_id == 0)
+	{
+		idle_buddy_timeout_id = gaim_timeout_add(10000,
+			(GSourceFunc)idle_timeout_cb, NULL);
+	}
+}
+
+static void
+remove_idle_buddy(GaimBuddy *buddy)
+{
+	idle_buddies = g_list_remove(idle_buddies, buddy);
+
+	if ((idle_buddies == NULL) && (idle_buddy_timeout_id > 0))
+	{
+		gaim_timeout_remove(idle_buddy_timeout_id);
+		idle_buddy_timeout_id = 0;
+	}
+}
+
+/*
+ * @param idle The time at which the buddy became idle, in seconds
+ *             since the epoch.
+ */
+void serv_got_update(GaimConnection *gc, const char *name, gboolean loggedin,
 					 int evil, time_t signon, time_t idle, int type)
 {
 	GaimAccount *account;
 	GaimConversation *c;
 	GaimBuddy *b;
+	char *alias;
 	GSList *buddies;
 	int old_idle;
 	time_t current_time = time(NULL);
@@ -1069,9 +1183,10 @@ void serv_got_update(GaimConnection *gc, const char *name, int loggedin,
 	/* store things how THEY want it... */
 	if (strcmp(name, b->name)) {
 		gaim_blist_rename_buddy(b, name);
-		gaim_blist_save();
 	}
 
+	alias = gaim_escape_html(gaim_buddy_get_alias(b));
+
 	old_idle = b->idle;
 
 	if (loggedin) {
@@ -1085,8 +1200,7 @@ void serv_got_update(GaimConnection *gc, const char *name, int loggedin,
 
 	if (signing_on) {
 		if (c != NULL) {
-			char *tmp = g_strdup_printf(_("%s logged in."),
-										gaim_get_buddy_alias(b));
+			char *tmp = g_strdup_printf(_("%s logged in."), alias);
 
 			gaim_conversation_write(c, NULL, tmp, GAIM_MESSAGE_SYSTEM,
 									time(NULL));
@@ -1095,8 +1209,7 @@ void serv_got_update(GaimConnection *gc, const char *name, int loggedin,
 		else if (awayqueue && find_queue_total_by_name(b->name)) {
 			struct queued_message *qm = g_new0(struct queued_message, 1);
 			g_snprintf(qm->name, sizeof(qm->name), "%s", b->name);
-			qm->message = g_strdup_printf(_("%s logged in."),
-										  gaim_get_buddy_alias(b));
+			qm->message = g_strdup_printf(_("%s logged in."), alias);
 			qm->account = gc->account;
 			qm->tm = time(NULL);
 			qm->flags = GAIM_MESSAGE_SYSTEM;
@@ -1108,10 +1221,9 @@ void serv_got_update(GaimConnection *gc, const char *name, int loggedin,
 		   gaim_prefs_get_bool("/core/logging/log_signon_signoff")) {
 			GaimAccount *account = gaim_connection_get_account(gc);
 			GaimLog *log = gaim_account_get_log(account);
-			char *tmp = g_strdup_printf(_("%s signed on"),
-										gaim_get_buddy_alias(b));
+			char *tmp = g_strdup_printf(_("%s signed on"), alias);
 
-			gaim_log_write(log, GAIM_MESSAGE_SYSTEM, gaim_get_buddy_alias(b),
+			gaim_log_write(log, GAIM_MESSAGE_SYSTEM, (alias ? alias : name),
 						   current_time, tmp);
 			g_free(tmp);
 		}
@@ -1124,12 +1236,12 @@ void serv_got_update(GaimConnection *gc, const char *name, int loggedin,
 		char *tmp = NULL;
 
 		if((b->uc & UC_UNAVAILABLE) && !(type & UC_UNAVAILABLE))
-			tmp = g_strdup_printf(_("%s came back"), gaim_get_buddy_alias(b));
+			tmp = g_strdup_printf(_("%s came back"), alias);
 		else if(!(b->uc & UC_UNAVAILABLE) && (type & UC_UNAVAILABLE))
-			tmp = g_strdup_printf(_("%s went away"), gaim_get_buddy_alias(b));
+			tmp = g_strdup_printf(_("%s went away"), alias);
 
 		if(tmp){
-			gaim_log_write(log, GAIM_MESSAGE_SYSTEM, gaim_get_buddy_alias(b),
+			gaim_log_write(log, GAIM_MESSAGE_SYSTEM, (alias ? alias : name),
 						   current_time, tmp);
 			g_free(tmp);
 		}
@@ -1140,10 +1252,9 @@ void serv_got_update(GaimConnection *gc, const char *name, int loggedin,
 		   gaim_prefs_get_bool("/core/logging/log_idle_state")) {
 			GaimAccount *account = gaim_connection_get_account(gc);
 			GaimLog *log = gaim_account_get_log(account);
-			char *tmp = g_strdup_printf(_("%s became idle"),
-										gaim_get_buddy_alias(b));
+			char *tmp = g_strdup_printf(_("%s became idle"), alias);
 
-			gaim_log_write(log, GAIM_MESSAGE_SYSTEM, gaim_get_buddy_alias(b),
+			gaim_log_write(log, GAIM_MESSAGE_SYSTEM, (alias ? alias : name),
 						   current_time, tmp);
 			g_free(tmp);
 		}
@@ -1152,10 +1263,9 @@ void serv_got_update(GaimConnection *gc, const char *name, int loggedin,
 		   gaim_prefs_get_bool("/core/logging/log_idle_state")) {
 			GaimAccount *account = gaim_connection_get_account(gc);
 			GaimLog *log = gaim_account_get_log(account);
-			char *tmp = g_strdup_printf(_("%s became unidle"),
-										gaim_get_buddy_alias(b));
+			char *tmp = g_strdup_printf(_("%s became unidle"), alias);
 
-			gaim_log_write(log, GAIM_MESSAGE_SYSTEM, gaim_get_buddy_alias(b),
+			gaim_log_write(log, GAIM_MESSAGE_SYSTEM, (alias ? alias : name),
 						   current_time, tmp);
 			g_free(tmp);
 		}
@@ -1163,16 +1273,14 @@ void serv_got_update(GaimConnection *gc, const char *name, int loggedin,
 
 	if (signing_off) {
 		if (c != NULL) {
-			char *tmp = g_strdup_printf(_("%s logged out."),
-										gaim_get_buddy_alias(b));
+			char *tmp = g_strdup_printf(_("%s logged out."), alias);
 			gaim_conversation_write(c, NULL, tmp,
 									GAIM_MESSAGE_SYSTEM, time(NULL));
 			g_free(tmp);
 		} else if (awayqueue && find_queue_total_by_name(b->name)) {
 			struct queued_message *qm = g_new0(struct queued_message, 1);
 			g_snprintf(qm->name, sizeof(qm->name), "%s", b->name);
-			qm->message = g_strdup_printf(_("%s logged out."),
-										  gaim_get_buddy_alias(b));
+			qm->message = g_strdup_printf(_("%s logged out."), alias);
 			qm->account = gc->account;
 			qm->tm = time(NULL);
 			qm->flags = GAIM_MESSAGE_SYSTEM;
@@ -1185,10 +1293,9 @@ void serv_got_update(GaimConnection *gc, const char *name, int loggedin,
 		   gaim_prefs_get_bool("/core/logging/log_signon_signoff")) {
 			GaimAccount *account = gaim_connection_get_account(gc);
 			GaimLog *log = gaim_account_get_log(account);
-			char *tmp = g_strdup_printf(_("%s signed off"),
-										gaim_get_buddy_alias(b));
+			char *tmp = g_strdup_printf(_("%s signed off"), alias);
 
-			gaim_log_write(log, GAIM_MESSAGE_SYSTEM, gaim_get_buddy_alias(b),
+			gaim_log_write(log, GAIM_MESSAGE_SYSTEM, (alias ? alias : name),
 						   current_time, tmp);
 			g_free(tmp);
 		}
@@ -1204,14 +1311,14 @@ void serv_got_update(GaimConnection *gc, const char *name, int loggedin,
 	if (!old_idle && idle)
 	{
 		gaim_signal_emit(gaim_blist_get_handle(), "buddy-idle", b);
+
+		add_idle_buddy(b);
 	}
 	else if (old_idle && !idle)
 	{
 		gaim_signal_emit(gaim_blist_get_handle(), "buddy-unidle", b);
-	}
-	else if (old_idle != idle)
-	{
-		gaim_signal_emit(gaim_blist_get_handle(), "buddy-idle-updated", b);
+
+		remove_idle_buddy(b);
 	}
 
 	if (c != NULL)
@@ -1226,6 +1333,7 @@ void serv_got_update(GaimConnection *gc, const char *name, int loggedin,
 		gaim_blist_update_buddy_evil(b, evil);
 		gaim_blist_update_buddy_status(b, type);
 	}
+	g_free(alias);
 }
 
 
@@ -1356,22 +1464,26 @@ void serv_got_chat_invite(GaimConnection *gc, const char *name,
 	account = gaim_connection_get_account(gc);
 
 	gaim_signal_emit(gaim_conversations_get_handle(),
-					 "chat-invited", account, who, name, message);
+					 "chat-invited", account, who, name, message, data);
 
-	if (message)
+	if (message != NULL)
+	{
+		char *escaped = g_markup_escape_text(message, -1);
 		g_snprintf(buf2, sizeof(buf2),
-				   _("User '%s' invites %s to buddy chat room: '%s'\n%s"),
-				   who, gaim_account_get_username(account), name, message);
+				   _("%s has invited %s to the chat room %s:\n<b>%s</b>"),
+				   who, gaim_account_get_username(account), name, escaped);
+		g_free(escaped);
+	}
 	else
 		g_snprintf(buf2, sizeof(buf2),
-				   _("User '%s' invites %s to buddy chat room: '%s'\n"),
+				   _("%s has invited %s to the chat room %s\n"),
 				   who, gaim_account_get_username(account), name);
 
 	cid->gc = gc;
 	cid->components = data;
 
 	gaim_request_accept_cancel(gc, NULL, _("Accept chat invitation?"),
-							   buf2, 0, cid,
+							   buf2, GAIM_DEFAULT_ACTION_NONE, cid,
 							   G_CALLBACK(chat_invite_accept),
 							   G_CALLBACK(chat_invite_reject));
 }
@@ -1436,9 +1548,9 @@ void serv_got_chat_left(GaimConnection *g, int id)
 }
 
 void serv_got_chat_in(GaimConnection *g, int id, const char *who,
-					  int whisper, const char *message, time_t mtime)
+					  GaimConvChatFlags chatflags, const char *message, time_t mtime)
 {
-	GaimMessageFlags w;
+	GaimMessageFlags msgflags = 0;
 	GSList *bcs;
 	GaimConversation *conv = NULL;
 	GaimConvChat *chat = NULL;
@@ -1493,14 +1605,28 @@ void serv_got_chat_in(GaimConnection *g, int id, const char *who,
 	/* Make sure URLs are clickable */
 	buf = gaim_markup_linkify(message);
 
-	if (whisper)
-		w = GAIM_MESSAGE_WHISPER;
-	else
-		w = 0;
+	if (chatflags & GAIM_CONV_CHAT_WHISPER)
+		msgflags |= GAIM_MESSAGE_WHISPER;
+	if (chatflags & GAIM_CONV_CHAT_DELAYED)
+		msgflags |= GAIM_MESSAGE_DELAYED;
 
-	gaim_conv_chat_write(chat, who, buf, w, mtime);
+	gaim_conv_chat_write(chat, who, buf, msgflags, mtime);
 
 	g_free(angel);
 	g_free(buf);
 	g_free(buffy);
 }
+
+void serv_send_file(GaimConnection *gc, const char *who, const char *file)
+{
+	GaimPluginProtocolInfo *prpl_info = NULL;
+	
+	if (gc != NULL && gc->prpl != NULL)
+		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
+
+	if (prpl_info && prpl_info->send_file) {
+		if (!prpl_info->can_receive_file || prpl_info->can_receive_file(gc, who)) {
+			prpl_info->send_file(gc, who, file);
+		}
+	}
+}
