@@ -28,6 +28,9 @@
 #include "signals.h"
 #include "value.h"
 
+/* must include this to use G_VA_COPY */
+#include <string.h>
+
 typedef struct
 {
 	void *instance;
@@ -101,6 +104,7 @@ destroy_signal_data(GaimSignalData *signal_data)
 		g_free(signal_data->values);
 	}
 
+	g_free(signal_data->ret_value);
 	g_free(signal_data);
 }
 
@@ -585,6 +589,16 @@ gaim_marshal_VOID__POINTER(GaimCallback cb, va_list args, void *data,
 	((void (*)(void *, void *))cb)(arg1, data);
 }
 
+void
+gaim_marshal_VOID__POINTER_UINT(GaimCallback cb, va_list args,
+										void *data, void **return_val)
+{
+	void *arg1 = va_arg(args, void *);
+	guint arg2 = va_arg(args, guint);
+
+	((void (*)(void *, guint, void *))cb)(arg1, arg2, data);
+}
+
 void
 gaim_marshal_VOID__POINTER_POINTER(GaimCallback cb, va_list args,
 								   void *data, void **return_val)
@@ -606,6 +620,18 @@ gaim_marshal_VOID__POINTER_POINTER_UINT(GaimCallback cb, va_list args,
 	((void (*)(void *, void *, guint, void *))cb)(arg1, arg2, arg3, data);
 }
 
+void
+gaim_marshal_VOID__POINTER_POINTER_UINT_UINT(GaimCallback cb, va_list args,
+										     void *data, void **return_val)
+{
+	void *arg1 = va_arg(args, void *);
+	void *arg2 = va_arg(args, void *);
+	guint arg3 = va_arg(args, guint);
+	guint arg4 = va_arg(args, guint);
+
+	((void (*)(void *, void *, guint, guint, void *))cb)(arg1, arg2, arg3, arg4, data);
+}
+
 void
 gaim_marshal_VOID__POINTER_POINTER_POINTER(GaimCallback cb, va_list args,
 										   void *data, void **return_val)
@@ -631,6 +657,21 @@ gaim_marshal_VOID__POINTER_POINTER_POINTER_POINTER(GaimCallback cb,
 	((void (*)(void *, void *, void *, void *, void *))cb)(arg1, arg2, arg3, arg4, data);
 }
 
+void
+gaim_marshal_VOID__POINTER_POINTER_POINTER_POINTER_POINTER(GaimCallback cb,
+														   va_list args,
+														   void *data,
+														   void **return_val)
+{
+	void *arg1 = va_arg(args, void *);
+	void *arg2 = va_arg(args, void *);
+	void *arg3 = va_arg(args, void *);
+	void *arg4 = va_arg(args, void *);
+	void *arg5 = va_arg(args, void *);
+
+	((void (*)(void *, void *, void *, void *, void *, void *))cb)(arg1, arg2, arg3, arg4, arg5, data);
+}
+
 void
 gaim_marshal_VOID__POINTER_POINTER_POINTER_UINT(GaimCallback cb,
 												   va_list args,
