@@ -1,7 +1,9 @@
 /*
  * Themes for Gaim
  *
- * Copyright (C) 2003, Sean Egan <bj91704@binghamton.edu>
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +20,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#include "gtkinternal.h"
+#include "internal.h"
+#include "gtkgaim.h"
 
 #include "conversation.h"
 #include "debug.h"
@@ -26,10 +29,9 @@
 #include "util.h"
 
 #include "gtkconv.h"
+#include "gtkdialogs.h"
 #include "gtkimhtml.h"
 
-#include "ui.h"
-
 struct smiley_list {
 	char *sml;
 	GSList *smileys;
@@ -193,8 +195,10 @@ void load_smiley_theme(const char *file, gboolean load)
 		for (cnv = gaim_get_conversations(); cnv != NULL; cnv = cnv->next) {
 			GaimConversation *conv = cnv->data;
 
-			if (GAIM_IS_GTK_CONVERSATION(conv))
+			if (GAIM_IS_GTK_CONVERSATION(conv)) {
 				smiley_themeize(GAIM_GTK_CONVERSATION(conv)->imhtml);
+				smiley_themeize(GAIM_GTK_CONVERSATION(conv)->entry);
+			}
 		}
 	}
 
@@ -234,13 +238,16 @@ void smiley_theme_probe()
 	}
 }
 
-GSList *get_proto_smileys(int protocol) {
-	GaimPlugin *proto = gaim_find_prpl(protocol);
+GSList *get_proto_smileys(const char *id) {
+	GaimPlugin *proto = gaim_find_prpl(id);
 	struct smiley_list *list, *def;
 
 	if(!current_smiley_theme)
 		return NULL;
 
+	if(!current_smiley_theme->list)
+		return NULL;
+
 	def = list = current_smiley_theme->list;
 
 	while(list) {
