@@ -22,10 +22,6 @@
  */
 #include "internal.h"
 
-#include <errno.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-
 #include "conversation.h"
 #include "debug.h"
 #include "prpl.h"
@@ -39,6 +35,8 @@ typedef struct
 
 	struct
 	{
+		char *user;
+		char *passwd;
 		char *address;
 		int port;
 		char *page;
@@ -55,6 +53,7 @@ typedef struct
 	gboolean sentreq;
 	gboolean newline;
 	gboolean startsaving;
+	gboolean has_explicit_data_len;
 	char *webdata;
 	unsigned long len;
 	unsigned long data_len;
@@ -505,81 +504,75 @@ gaim_time_build(int year, int month, int day, int hour, int min, int sec)
 time_t
 gaim_str_to_time(const char *timestamp, gboolean utc)
 {
-    struct tm t;
-    time_t retval = 0;
-    char buf[32];
-    char *c;
-    int tzoff = 0;
+	time_t retval = 0;
+	struct tm *t;
+	char buf[32];
+	char *c;
+	int tzoff = 0;
 
-    time(&retval);
-    localtime_r(&retval, &t);
+	time(&retval);
+	t = localtime(&retval);
 
-    snprintf(buf, sizeof(buf), "%s", timestamp);
-    c = buf;
+	snprintf(buf, sizeof(buf), "%s", timestamp);
+	c = buf;
 
-    /* 4 digit year */
-    if(!sscanf(c, "%04d", &t.tm_year)) return 0;
-    c+=4;
-    if(*c == '-')
-        c++;
-
-	t.tm_year -= 1900;
-
-	/* 2 digit month */
-	if(!sscanf(c, "%02d", &t.tm_mon)) return 0;
-	c+=2;
-	if(*c == '-')
+	/* 4 digit year */
+	if (!sscanf(c, "%04d", &t->tm_year)) return 0;
+	c += 4;
+	if (*c == '-')
 		c++;
 
-    t.tm_mon -= 1;
+	t->tm_year -= 1900;
 
+	/* 2 digit month */
+	if (!sscanf(c, "%02d", &t->tm_mon)) return 0;
+	c += 2;
+	if (*c == '-')
+		c++;
 
-    /* 2 digit day */
-    if(!sscanf(c, "%02d", &t.tm_mday)) return 0;
-    c+=2;
-    if(*c == 'T' || *c == '.') { /* we have more than a date, keep going */
-        c++; /* skip the "T" */
+	t->tm_mon -= 1;
 
-        /* 2 digit hour */
-        if(sscanf(c, "%02d:%02d:%02d", &t.tm_hour, &t.tm_min, &t.tm_sec) == 3 ||
-				sscanf(c, "%02d%02d%02d", &t.tm_hour, &t.tm_min, &t.tm_sec) == 3) {
-            int tzhrs, tzmins;
-            c+=8;
-            if(*c == '.') /* dealing with precision we don't care about */
-                c += 4;
+	/* 2 digit day */
+	if (!sscanf(c, "%02d", &t->tm_mday)) return 0;
+	c += 2;
+	if (*c == 'T' || *c == '.') { /* we have more than a date, keep going */
+		c++; /* skip the "T" */
 
-            if((*c == '+' || *c == '-') &&
-                    sscanf(c+1, "%02d:%02d", &tzhrs, &tzmins)) {
-                tzoff = tzhrs*60*60 + tzmins*60;
-                if(*c == '+')
-                    tzoff *= -1;
-            }
-
-			if(tzoff || utc) {
+		/* 2 digit hour */
+		if (sscanf(c, "%02d:%02d:%02d", &t->tm_hour, &t->tm_min, &t->tm_sec) == 3 ||
+			sscanf(c, "%02d%02d%02d", &t->tm_hour, &t->tm_min, &t->tm_sec) == 3) {
+			int tzhrs, tzmins;
+			c += 8;
+			if (*c == '.') /* dealing with precision we don't care about */
+				c += 4;
+			if ((*c == '+' || *c == '-') &&
+				sscanf(c+1, "%02d:%02d", &tzhrs, &tzmins)) {
+				tzoff = tzhrs*60*60 + tzmins*60;
+				if (*c == '+')
+					tzoff *= -1;
+			}
 
+			if (tzoff || utc) {
 #ifdef HAVE_TM_GMTOFF
-                tzoff += t.tm_gmtoff;
+				tzoff += t->tm_gmtoff;
 #else
-#   ifdef HAVE_TIMEZONE
-                tzset();    /* making sure */
-                tzoff -= timezone;
-#   endif
+#	ifdef HAVE_TIMEZONE
+				tzset();    /* making sure */
+				tzoff -= timezone;
+#	endif
 #endif
 			}
-        }
-    }
+		}
+	}
 
-	t.tm_isdst = -1;
+	t->tm_isdst = -1;
+	retval = mktime(t);
+	retval += tzoff;
 
-	retval = mktime(&t);
-
-    retval += tzoff;
-
-    return retval;
+	return retval;
 }
 
 
-
 /**************************************************************************
  * Markup Functions
  **************************************************************************/
@@ -768,13 +761,25 @@ gaim_markup_extract_info_field(const char *str, int len, GString *dest,
 
 	q = strstr(p, end_token);
 
+	/* Trim leading blanks */
+	while (*p != '\n' && g_ascii_isspace(*p)) {
+		p += 1;
+	}
+
+	/* Trim trailing blanks */
+	while (q > p && g_ascii_isspace(*(q - 1))) {
+		q -= 1;
+	}
+
+	/* Don't bother with null strings */
+	if (p == q)
+		return FALSE;
+
 	if (q != NULL && (!no_value_token ||
 					  (no_value_token && strncmp(p, no_value_token,
 												 strlen(no_value_token)))))
 	{
-		g_string_append(dest, "<b>");
-		g_string_append(dest, display_name);
-		g_string_append(dest, ":</b> ");
+		g_string_append_printf(dest, _("<b>%s:</b> "), display_name);
 
 		if (is_link)
 		{
@@ -1208,12 +1213,23 @@ gaim_markup_html_to_xhtml(const char *html, char **xhtml_out,
 	g_string_free(plain, TRUE);
 }
 
+/* The following are probably reasonable changes:
+ * - \n should be converted to a normal space
+ * - in addition to <br>, <p> and <div> etc. should also be converted into \n
+ * - We want to turn </td>#whitespace<td> sequences into a single tab
+ * - We want to turn <td> into a single tab (for msn profile "parsing")
+ * - We want to turn </tr>#whitespace<tr> sequences into a single \n
+ * - <script>...</script> and <style>...</style> should be completely removed
+ */
+
 char *
 gaim_markup_strip_html(const char *str)
 {
 	int i, j, k;
 	gboolean visible = TRUE;
+	gboolean closing_td_p = FALSE;
 	gchar *str2;
+	const gchar *cdata_close_tag = NULL;
 
 	if(!str)
 		return NULL;
@@ -1224,42 +1240,90 @@ gaim_markup_strip_html(const char *str)
 	{
 		if (str2[i] == '<')
 		{
-			if (strncasecmp(str2 + i, "<br>", 4) == 0)
+			if (cdata_close_tag)
 			{
-				str2[j++] = '\n';
-				i = i + 3;
+				/* Note: Don't even assume any other tag is a tag in CDATA */
+				if (strncasecmp(str2 + i, cdata_close_tag,
+						strlen(cdata_close_tag)) == 0)
+				{
+					i += strlen(cdata_close_tag) - 1;
+					cdata_close_tag = NULL;
+				}
 				continue;
 			}
+			else if (strncasecmp(str2 + i, "<td", 3) == 0 && closing_td_p)
+			{
+				str2[j++] = '\t';
+				visible = TRUE;
+			}
+			else if (strncasecmp(str2 + i, "</td>", 5) == 0)
+			{
+				closing_td_p = TRUE;
+				visible = FALSE;
+			}
+			else
+			{
+				closing_td_p = FALSE;
+				visible = TRUE;
+			}
 
 			k = i + 1;
 
 			if(g_ascii_isspace(str2[k]))
 				visible = TRUE;
-			else
+			else if (str2[k])
 			{
-				while (str2[k])
+				/* Scan until we end the tag either implicitly (closed start
+				 * tag) or explicitly, using a sloppy method (i.e., < or >
+				 * inside quoted attributes will screw us up)
+				 */
+				while (str2[k] && str2[k] != '<' && str2[k] != '>')
 				{
-					if (str2[k] == '<')
-					{
-						visible = TRUE;
-						break;
-					}
-
-					if (str2[k] == '>')
-					{
-						visible = FALSE;
-						break;
-					}
-
 					k++;
 				}
+
+				/* Check for tags which should be mapped to newline */
+				if (strncasecmp(str2 + i, "<p>", 3) == 0
+				 || strncasecmp(str2 + i, "<tr", 3) == 0
+				 || strncasecmp(str2 + i, "<br", 3) == 0
+				 || strncasecmp(str2 + i, "<li", 3) == 0
+				 || strncasecmp(str2 + i, "<div", 4) == 0
+				 || strncasecmp(str2 + i, "</table>", 8) == 0)
+				{
+					str2[j++] = '\n';
+				}
+				/* Check for tags which begin CDATA and need to be closed */
+#if 0 /* FIXME.. option is end tag optional, we can't handle this right now */
+				else if (strncasecmp(str2 + i, "<option", 7) == 0)
+				{
+					/* FIXME: We should not do this if the OPTION is SELECT'd */
+					cdata_close_tag = "</option>";
+				}
+#endif
+				else if (strncasecmp(str2 + i, "<script", 7) == 0)
+				{
+					cdata_close_tag = "</script>";
+				}
+				else if (strncasecmp(str2 + i, "<style", 6) == 0)
+				{
+					cdata_close_tag = "</style>";
+				}
+				/* Update the index and continue checking after the tag */
+				i = (str2[k] == '<')? k - 1: k;
+				continue;
 			}
 		}
-		else if (str2[i] == '>' && !visible)
+		else if (cdata_close_tag)
 		{
-			visible = TRUE;
 			continue;
 		}
+		else if (!g_ascii_isspace(str2[i]))
+		{
+			visible = TRUE;
+		}
+
+		/* XXX: This sucks.  We need to be un-escaping all entities, which
+		 * includes these, as well as the &#num; ones */
 
 		if (str2[i] == '&' && strncasecmp(str2 + i, "&quot;", 6) == 0)
 		{
@@ -1289,8 +1353,15 @@ gaim_markup_strip_html(const char *str)
 			continue;
 		}
 
+		if (str2[i] == '&' && strncasecmp(str2 + i, "&apos;", 6) == 0)
+		{
+			str2[j++] = '\'';
+			i = i + 5;
+			continue;
+		}
+
 		if (visible)
-			str2[j++] = str2[i];
+			str2[j++] = g_ascii_isspace(str2[i])? ' ': str2[i];
 	}
 
 	str2[j] = '\0';
@@ -1298,38 +1369,56 @@ gaim_markup_strip_html(const char *str)
 	return str2;
 }
 
-static gint
+static gboolean
 badchar(char c)
 {
 	switch (c) {
 	case ' ':
 	case ',':
-	case '(':
-	case ')':
 	case '\0':
 	case '\n':
 	case '<':
 	case '>':
 	case '"':
 	case '\'':
-		return 1;
+		return TRUE;
 	default:
-		return 0;
+		return FALSE;
 	}
 }
 
+static gboolean
+badentity(const char *c)
+{
+	if (!g_ascii_strncasecmp(c, "&lt;", 4) ||
+		!g_ascii_strncasecmp(c, "&gt;", 4) ||
+		!g_ascii_strncasecmp(c, "&quot;", 6)) {
+		return TRUE;
+	}
+	return FALSE;
+}
+
 char *
 gaim_markup_linkify(const char *text)
 {
 	const char *c, *t, *q = NULL;
 	char *tmp, *tmpurlbuf;
 	char url_buf[BUF_LEN * 4];
+	gunichar g;
 	gboolean inside_html = FALSE;
+	int inside_paren = 0;
 	GString *ret = g_string_new("");
-	/* Assumes you have a buffer able to cary at least BUF_LEN * 2 bytes */
+	/* Assumes you have a buffer able to carry at least BUF_LEN * 2 bytes */
 
 	c = text;
 	while (*c) {
+
+		if(*c == '(' && !inside_html) {
+			inside_paren++;
+			ret = g_string_append_c(ret, *c);
+			c++;
+		}
+
 		if(inside_html) {
 			if(*c == '>') {
 				inside_html = FALSE;
@@ -1357,7 +1446,7 @@ gaim_markup_linkify(const char *text)
 					(!g_ascii_strncasecmp(c, "https://", 8)))) {
 			t = c;
 			while (1) {
-				if (badchar(*t)) {
+				if (badchar(*t) || badentity(t)) {
 
 					if (*(t) == ',' && (*(t + 1) != ' ')) {
 						t++;
@@ -1366,6 +1455,10 @@ gaim_markup_linkify(const char *text)
 
 					if (*(t - 1) == '.')
 						t--;
+					if ((*(t - 1) == ')' && (inside_paren > 0))) {
+						t--;
+					}
+
 					strncpy(url_buf, c, t - c);
 					url_buf[t - c] = 0;
 					tmpurlbuf = gaim_unescape_html(url_buf);
@@ -1384,7 +1477,7 @@ gaim_markup_linkify(const char *text)
 			if (c[4] != '.') {
 				t = c;
 				while (1) {
-					if (badchar(*t)) {
+					if (badchar(*t) || badentity(t)) {
 						if (t - c == 4) {
 							break;
 						}
@@ -1396,6 +1489,9 @@ gaim_markup_linkify(const char *text)
 
 						if (*(t - 1) == '.')
 							t--;
+						if ((*(t - 1) == ')' && (inside_paren > 0))) {
+							t--;
+						}
 						strncpy(url_buf, c, t - c);
 						url_buf[t - c] = 0;
 						tmpurlbuf = gaim_unescape_html(url_buf);
@@ -1414,9 +1510,12 @@ gaim_markup_linkify(const char *text)
 		} else if (!g_ascii_strncasecmp(c, "ftp://", 6)) {
 			t = c;
 			while (1) {
-				if (badchar(*t)) {
+				if (badchar(*t) || badentity(t)) {
 					if (*(t - 1) == '.')
 						t--;
+					if ((*(t - 1) == ')' && (inside_paren > 0))) {
+						t--;
+					}
 					strncpy(url_buf, c, t - c);
 					url_buf[t - c] = 0;
 					tmpurlbuf = gaim_unescape_html(url_buf);
@@ -1435,12 +1534,15 @@ gaim_markup_linkify(const char *text)
 			if (c[4] != '.') {
 				t = c;
 				while (1) {
-					if (badchar(*t)) {
+					if (badchar(*t) || badentity(t)) {
 						if (t - c == 4) {
 							break;
 						}
 						if (*(t - 1) == '.')
 							t--;
+						if ((*(t - 1) == ')' && (inside_paren > 0))) {
+							t--;
+						}
 						strncpy(url_buf, c, t - c);
 						url_buf[t - c] = 0;
 						tmpurlbuf = gaim_unescape_html(url_buf);
@@ -1459,7 +1561,7 @@ gaim_markup_linkify(const char *text)
 		} else if (!g_ascii_strncasecmp(c, "mailto:", 7)) {
 			t = c;
 			while (1) {
-				if (badchar(*t)) {
+				if (badchar(*t) || badentity(t)) {
 					if (*(t - 1) == '.')
 						t--;
 					strncpy(url_buf, c, t - c);
@@ -1477,9 +1579,8 @@ gaim_markup_linkify(const char *text)
 
 			}
 		} else if (c != text && (*c == '@')) {
-			char *tmp;
 			int flag;
-			int len = 0;
+			GString *gurl_buf;
 			const char illegal_chars[] = "!@#$%^&*()[]{}/|\\<>\":;\r\n \0";
 			url_buf[0] = 0;
 
@@ -1489,20 +1590,19 @@ gaim_markup_linkify(const char *text)
 				flag = 1;
 
 			t = c;
+			gurl_buf = g_string_new("");
 			while (flag) {
-				if (badchar(*t)) {
-					ret = g_string_truncate(ret, ret->len - (len - 1));
+				/* iterate backwards grabbing the local part of an email address */
+				g = g_utf8_get_char(t);
+				if (badchar(*t) || (g >= 127) || (*t == '(') ||
+						((*t == ';') && (t > (text+2)) && !g_ascii_strncasecmp(t - 3, "&lt;", 4))) {
+					/* local part will already be part of ret, strip it out */
+					ret = g_string_truncate(ret, ret->len - (c - t));
+					ret = g_string_append_unichar(ret, g);
 					break;
 				} else {
-					len++;
-					tmp = g_malloc(len + 1);
-					tmp[len] = 0;
-					tmp[0] = *t;
-					strncpy(tmp + 1, url_buf, len - 1);
-					strcpy(url_buf, tmp);
-					url_buf[len] = 0;
-					g_free(tmp);
-					t--;
+					g_string_prepend_unichar(gurl_buf, g);
+					t = g_utf8_find_prev_char(text, t);
 					if (t < text) {
 						ret = g_string_assign(ret, "");
 						break;
@@ -1510,12 +1610,17 @@ gaim_markup_linkify(const char *text)
 				}
 			}
 
-			t = c + 1;
+			t = g_utf8_find_next_char(c, NULL);
 
 			while (flag) {
-				if (badchar(*t)) {
+				/* iterate forwards grabbing the domain part of an email address */
+				g = g_utf8_get_char(t);
+				if (badchar(*t) || (g >= 127) || (*t == ')') || badentity(t)) {
 					char *d;
 
+					strcpy(url_buf, gurl_buf->str);
+
+					/* strip off trailing periods */
 					for (d = url_buf + strlen(url_buf) - 1; *d == '.'; d--, t--)
 						*d = '\0';
 
@@ -1531,13 +1636,17 @@ gaim_markup_linkify(const char *text)
 
 					break;
 				} else {
-					strncat(url_buf, t, 1);
-					len++;
-					url_buf[len] = 0;
+					g_string_append_unichar(gurl_buf, g);
+					t = g_utf8_find_next_char(t, NULL);
 				}
-
-				t++;
 			}
+			g_string_free(gurl_buf, TRUE);
+		}
+
+		if(*c == ')' && !inside_html) {
+			inside_paren--;
+			ret = g_string_append_c(ret, *c);
+			c++;
 		}
 
 		if (*c == 0)
@@ -1554,36 +1663,35 @@ gaim_markup_linkify(const char *text)
 
 char *
 gaim_escape_html(const char *html) {
-	char *escaped = NULL;
+	const char *c = html;
+	GString *ret;
 
-	if (html != NULL) {
-		const char *c = html;
-		GString *ret = g_string_new("");
-		while (*c) {
-			switch(*c) {
-				case '&':
-					ret = g_string_append(ret, "&amp;");
-					break;
-				case '<':
-					ret = g_string_append(ret, "&lt;");
-					break;
-				case '>':
-					ret = g_string_append(ret, "&gt;");
-					break;
-				case '"':
-					ret = g_string_append(ret, "&quot;");
-					break;
-				default:
-					ret = g_string_append_c(ret, *c);
-			}
-			c++;
+	if (html == NULL)
+		return NULL;
+
+	ret = g_string_new("");
+
+	while (*c) {
+		switch (*c) {
+			case '&':
+				ret = g_string_append(ret, "&amp;");
+				break;
+			case '<':
+				ret = g_string_append(ret, "&lt;");
+				break;
+			case '>':
+				ret = g_string_append(ret, "&gt;");
+				break;
+			case '"':
+				ret = g_string_append(ret, "&quot;");
+				break;
+			default:
+				ret = g_string_append_c(ret, *c);
 		}
-
-		escaped = ret->str;
-		g_string_free(ret, FALSE);
+		c++;
 	}
-	return escaped;
 
+	return g_string_free(ret, FALSE);
 }
 
 char *
@@ -1625,6 +1733,127 @@ gaim_unescape_html(const char *html) {
 
 }
 
+char *
+gaim_markup_slice(const char *str, guint x, guint y)
+{
+	GString *ret;
+	GQueue *q;
+	guint z = 0;
+	gboolean appended = FALSE;
+	gunichar c;
+	char *tag;
+
+	g_return_val_if_fail(x <= y, NULL);
+
+	if (x == y)
+		return g_strdup("");
+
+	ret = g_string_new("");
+	q = g_queue_new();
+
+	while (*str && (z < y)) {
+		c = g_utf8_get_char(str);
+
+		if (c == '<') {
+			char *end = strchr(str, '>');
+
+			if (!end) {
+				g_string_free(ret, TRUE);
+				while ((tag = g_queue_pop_head(q)))
+					g_free(tag);
+				g_queue_free(q);
+				return NULL;
+			}
+
+			if (!g_ascii_strncasecmp(str, "<img ", 5)) {
+				z += strlen("[Image]");
+			} else if (!g_ascii_strncasecmp(str, "<br", 3)) {
+				z += 1;
+			} else if (!g_ascii_strncasecmp(str, "<hr>", 4)) {
+				z += strlen("\n---\n");
+			} else if (!g_ascii_strncasecmp(str, "</", 2)) {
+				/* pop stack */
+				char *tmp;
+
+				tmp = g_queue_pop_head(q);
+				if (tmp)
+					g_free(tmp);
+				/* z += 0; */
+			} else {
+				/* push it unto the stack */
+				char *tmp;
+
+				tmp = g_strndup(str, end - str + 1);
+				g_queue_push_head(q, tmp);
+				/* z += 0; */
+			}
+
+			if (z == x && !appended) {
+				GList *l = q->tail;
+
+				while (l) {
+					tag = l->data;
+					g_string_append(ret, tag);
+					l = l->prev;
+				}
+				appended = TRUE;
+			} else if (z >= x) {
+				g_string_append_len(ret, str, end - str + 1);
+			}
+
+			str = end;
+		} else if (c == '&') {
+			char *end = strchr(str, ';');
+			if (!end) {
+				g_string_free(ret, TRUE);
+				while ((tag = g_queue_pop_head(q)))
+					g_free(tag);
+				g_queue_free(q);
+
+				return NULL;
+			}
+
+			if (z >= x)
+				g_string_append_len(ret, str, end - str + 1);
+
+			z++;
+			str = end;
+		} else {
+			if (z >= x)
+				g_string_append_unichar(ret, c);
+			z++;
+		}
+
+		str = g_utf8_next_char(str);
+	}
+
+	while ((tag = g_queue_pop_head(q))) {
+		char *name;
+
+		name = gaim_markup_get_tag_name(tag);
+		g_string_append_printf(ret, "</%s>", name);
+		g_free(name);
+		g_free(tag);
+	}
+
+	g_queue_free(q);
+	return g_string_free(ret, FALSE);
+}
+
+char *
+gaim_markup_get_tag_name(const char *tag)
+{
+	int i;
+	g_return_val_if_fail(tag != NULL, NULL);
+	g_return_val_if_fail(*tag == '<', NULL);
+
+	for (i = 1; tag[i]; i++)
+		if (tag[i] == '>' || tag[i] == ' ' || tag[i] == '/')
+			break;
+
+	return g_strndup(tag, i);
+}
+
 /**************************************************************************
  * Path/Filename Functions
  **************************************************************************/
@@ -1652,8 +1881,9 @@ gaim_user_dir(void)
 		const gchar *hd = gaim_home_dir();
 
 		if (hd) {
-			strcpy((char*) &home_dir, hd);
-			strcat((char*) &home_dir, G_DIR_SEPARATOR_S ".gaim");
+			g_strlcpy((char*) &home_dir, hd, sizeof(home_dir));
+			g_strlcat((char*) &home_dir, G_DIR_SEPARATOR_S ".gaim",
+					sizeof(home_dir));
 		}
 	}
 
@@ -1663,7 +1893,8 @@ gaim_user_dir(void)
 void set_gaim_user_dir(const char *dir)
 {
 	if (dir != NULL && strlen(dir) > 0) {
-		strcpy((char*) &custom_home_dir, dir);
+		g_strlcpy((char*) &custom_home_dir, dir,
+				sizeof(custom_home_dir));
 	}
 }
 
@@ -1690,14 +1921,14 @@ int gaim_build_dir (const char *path, int mode)
 		if(g_file_test(dir, G_FILE_TEST_IS_DIR)) {
 			continue;
 		} else if(g_file_test(dir, G_FILE_TEST_EXISTS)) {
-			gaim_debug(GAIM_DEBUG_WARNING, "build_dir", "bad path: %s\n", path);
+			gaim_debug_warning("build_dir", "bad path: %s\n", path);
 			g_strfreev(components);
 			g_free(dir);
 			return -1;
 		}
 
 		if (mkdir(dir, mode) < 0) {
-			gaim_debug(GAIM_DEBUG_WARNING, "build_dir", "mkdir: %s\n", strerror(errno));
+			gaim_debug_warning("build_dir", "mkdir: %s\n", strerror(errno));
 			g_strfreev(components);
 			g_free(dir);
 			return -1;
@@ -2144,80 +2375,104 @@ gaim_str_size_to_units(size_t size)
 			size_index++;
 		}
 
-		return g_strdup_printf("%.2f %s", size_mag, size_str[size_index]);
+		if (size_index == 0) {
+			return g_strdup_printf("%zd %s", size, size_str[size_index]);
+		} else {
+			return g_strdup_printf("%.2f %s", size_mag, size_str[size_index]);
+		}
 	}
 }
 
 char *
-gaim_str_seconds_to_string(guint sec)
+gaim_str_seconds_to_string(guint secs)
 {
-	guint daze, hrs, min;
-	char *ret = NULL;
+	GString *gstr;
+	const char *prefix = "";
+	guint days, hrs, mins;
 
-	daze = sec / (60 * 60 * 24);
-	hrs = (sec % (60 * 60 * 24)) / (60 * 60);
-	min = (sec % (60 * 60)) / 60;
-	sec = min % 60;
+	gstr = g_string_new("");
 
-	if (daze) {
-		if (hrs || min) {
-			if (hrs) {
-				if (min) {
-					ret = g_strdup_printf(
-						   "%d %s, %d %s, %d %s.",
-						   daze, ngettext("day","days",daze),
-						   hrs, ngettext("hour","hours",hrs), min, ngettext("minute","minutes",min));
-				} else {
-					ret = g_strdup_printf(
-						   "%d %s, %d %s.",
-						   daze, ngettext("day","days",daze), hrs, ngettext("hour","hours",hrs));
-				}
-			} else {
-				ret = g_strdup_printf(
-					   "%d %s, %d %s.",
-					   daze, ngettext("day","days",daze), min, ngettext("minute","minutes",min));
-			}
-		} else
-			ret = g_strdup_printf("%d %s.", daze, ngettext("day","days",daze));
-	} else {
-		if (hrs) {
-			if (min) {
-				ret = g_strdup_printf(
-					   "%d %s, %d %s.",
-					   hrs, ngettext("hour","hours",hrs), min, ngettext("minute","minutes",min));
-			} else {
-				ret = g_strdup_printf("%d %s.", hrs, ngettext("hour","hours",hrs));
-			}
-		} else {
-			ret = g_strdup_printf("%d %s.", min, ngettext("minute","minutes",min));
-		}
+	if (secs < 60)
+	{
+		g_string_append_printf(gstr, "%d %s", secs,
+							   ngettext("second", "seconds", secs));
+		return g_string_free(gstr, FALSE);
 	}
 
-	return ret;
+	days = secs / (60 * 60 * 24);
+	secs = secs % (60 * 60 * 24);
+	hrs  = secs / (60 * 60);
+	secs = secs % (60 * 60);
+	mins = secs / 60;
+	secs = secs % 60;
+
+	if (days > 0)
+	{
+		g_string_append_printf(gstr, "%d %s", days,
+							   ngettext("day", "days", days));
+
+		prefix = ", ";
+	}
+
+	if (hrs > 0)
+	{
+		g_string_append_printf(gstr, "%s%d %s", prefix, hrs,
+							   ngettext("hour", "hours", hrs));
+
+		prefix = ", ";
+	}
+
+	if (mins > 0)
+	{
+		g_string_append_printf(gstr, "%s%d %s", prefix, mins,
+							   ngettext("minute", "minutes", mins));
+	}
+
+	return g_string_free(gstr, FALSE);
+}
+
+
+char *
+gaim_str_binary_to_ascii(const unsigned char *binary, guint len)
+{
+	GString *ret;
+	guint i;
+
+	g_return_val_if_fail(len > 0, NULL);
+
+	ret = g_string_sized_new(len);
+
+	for (i = 0; i < len; i++)
+		if (binary[i] < 32 || binary[i] > 126)
+			g_string_append_printf(ret, "\\x%02hhx", binary[i]);
+		else if (binary[i] == '\\')
+			g_string_append(ret, "\\\\");
+		else
+			g_string_append_c(ret, binary[i]);
+
+	return g_string_free(ret, FALSE);
 }
 
 /**************************************************************************
  * URI/URL Functions
  **************************************************************************/
-/*
- * XXX - Make this also support extracting a username and password?
- * Would be nice when dissecting an environmental variable
- * that specifies proxy information.
- */
 gboolean
 gaim_url_parse(const char *url, char **ret_host, int *ret_port,
-			   char **ret_path)
+			   char **ret_path, char **ret_user, char **ret_passwd)
 {
 	char scan_info[255];
 	char port_str[6];
 	int f;
+	const char *at, *slash;
 	const char *turl;
-	char host[256], path[256];
+	char host[256], path[256], user[256], passwd[256];
 	int port = 0;
 	/* hyphen at end includes it in control set */
 	static char addr_ctrl[] = "A-Za-z0-9.-";
 	static char port_ctrl[] = "0-9";
 	static char page_ctrl[] = "A-Za-z0-9.~_/:*!@&%%?=+^-";
+	static char user_ctrl[] = "A-Za-z0-9.~_/*!&%%?=+^-";
+	static char passwd_ctrl[] = "A-Za-z0-9.~_/*!&%%?=+^-";
 
 	g_return_val_if_fail(url != NULL, FALSE);
 
@@ -2228,6 +2483,31 @@ gaim_url_parse(const char *url, char **ret_host, int *ret_port,
 		url = turl;
 	}
 
+	/* parse out authentication information if supplied */
+	/* Only care about @ char BEFORE the first / */
+	at = strchr(url, '@');
+	slash = strchr(url, '/');
+	if ((at != NULL) &&
+			(((slash != NULL) && (strlen(at) > strlen(slash))) ||
+			(slash == NULL))) {
+		g_snprintf(scan_info, sizeof(scan_info),
+					"%%255[%s]:%%255[%s]^@", user_ctrl, passwd_ctrl);
+		f = sscanf(url, scan_info, user, passwd);
+
+		if (f ==1 ) {
+			/* No passwd, possibly just username supplied */
+			g_snprintf(scan_info, sizeof(scan_info),
+						"%%255[%s]^@", user_ctrl);
+			f = sscanf(url, scan_info, user);
+			*passwd = '\0';
+		}
+
+		url = at+1; /* move pointer after the @ char */
+	} else {
+		*user = '\0';
+		*passwd = '\0';
+	}
+
 	g_snprintf(scan_info, sizeof(scan_info),
 			   "%%255[%s]:%%5[%s]/%%255[%s]", addr_ctrl, port_ctrl, page_ctrl);
 
@@ -2250,6 +2530,8 @@ gaim_url_parse(const char *url, char **ret_host, int *ret_port,
 	if (ret_host != NULL) *ret_host = g_strdup(host);
 	if (ret_port != NULL) *ret_port = port;
 	if (ret_path != NULL) *ret_path = g_strdup(path);
+	if (ret_user != NULL) *ret_user = g_strdup(user);
+	if (ret_passwd != NULL) *ret_passwd = g_strdup(passwd);
 
 	return TRUE;
 }
@@ -2262,6 +2544,8 @@ destroy_fetch_url_data(GaimFetchUrlData *gfud)
 	if (gfud->user_agent      != NULL) g_free(gfud->user_agent);
 	if (gfud->website.address != NULL) g_free(gfud->website.address);
 	if (gfud->website.page    != NULL) g_free(gfud->website.page);
+	if (gfud->website.user    != NULL) g_free(gfud->website.user);
+	if (gfud->website.passwd    != NULL) g_free(gfud->website.passwd);
 
 	g_free(gfud);
 }
@@ -2329,8 +2613,29 @@ static size_t
 parse_content_len(const char *data, size_t data_len)
 {
 	size_t content_len = 0;
+	const char *p = NULL;
 
-	sscanf(data, "Content-Length: %d", (int *)&content_len);
+	/* This is still technically wrong, since headers are case-insensitive
+	 * [RFC 2616, section 4.2], though this ought to catch the normal case.
+	 * Note: data is _not_ nul-terminated.
+	 */
+	if (data_len > 16) {
+		p = strncmp(data, "Content-Length: ", 16) == 0? data: NULL;
+		if (!p) {
+			p = g_strstr_len(data, data_len, "\nContent-Length: ");
+			if (p)
+				p += 1;
+		}
+	}
+
+	/* If we can find a Content-Length header at all, try to sscanf it.
+	 * Response headers should end with at least \r\n, so sscanf is safe,
+	 * if we make sure that there is indeed a \n in our header.
+	 */
+	if (p && g_strstr_len(p, data_len - (p - data), "\n")) {
+		sscanf(p, "Content-Length: %zu", &content_len);
+		gaim_debug_misc("parse_content_len", "parsed %u\n", content_len);
+	}
 
 	return content_len;
 }
@@ -2340,6 +2645,7 @@ url_fetched_cb(gpointer url_data, gint sock, GaimInputCondition cond)
 {
 	GaimFetchUrlData *gfud = url_data;
 	char data;
+	gboolean got_eof = FALSE;
 
 	if (sock == -1)
 	{
@@ -2356,44 +2662,29 @@ url_fetched_cb(gpointer url_data, gint sock, GaimInputCondition cond)
 
 		if (gfud->user_agent)
 		{
-			if (gfud->http11)
-			{
-				g_snprintf(buf, sizeof(buf),
-						   "GET %s%s HTTP/1.1\r\n"
-						   "User-Agent: \"%s\"\r\n"
-						   "Host: %s\r\n\r\n",
-						   (gfud->full ? "" : "/"),
-						   (gfud->full ? gfud->url : gfud->website.page),
-						   gfud->user_agent, gfud->website.address);
-			}
-			else
-			{
-				g_snprintf(buf, sizeof(buf),
-						   "GET %s%s HTTP/1.0\r\n"
-						   "User-Agent: \"%s\"\r\n\r\n",
-						   (gfud->full ? "" : "/"),
-						   (gfud->full ? gfud->url : gfud->website.page),
-						   gfud->user_agent);
-			}
+			/* Host header is not forbidden in HTTP/1.0 requests, and HTTP/1.1
+			 * clients must know how to handle the "chunked" transfer encoding.
+			 * Gaim doesn't know how to handle "chunked", so should always send
+			 * the Host header regardless, to get around some observed problems
+			 */
+			g_snprintf(buf, sizeof(buf),
+					   "GET %s%s HTTP/%s\r\n"
+					   "User-Agent: %s\r\n"
+					   "Host: %s\r\n\r\n",
+					   (gfud->full ? "" : "/"),
+					   (gfud->full ? gfud->url : gfud->website.page),
+					   (gfud->http11 ? "1.1" : "1.0"),
+					   gfud->user_agent, gfud->website.address);
 		}
 		else
 		{
-			if (gfud->http11)
-			{
-				g_snprintf(buf, sizeof(buf),
-						   "GET %s%s HTTP/1.1\r\n"
-						   "Host: %s\r\n\r\n",
-						   (gfud->full ? "" : "/"),
-						   (gfud->full ? gfud->url : gfud->website.page),
-						   gfud->website.address);
-			}
-			else
-			{
-				g_snprintf(buf, sizeof(buf),
-						   "GET %s%s HTTP/1.0\r\n\r\n",
-						   (gfud->full ? "" : "/"),
-						   (gfud->full ? gfud->url : gfud->website.page));
-			}
+			g_snprintf(buf, sizeof(buf),
+					   "GET %s%s HTTP/%s\r\n"
+					   "Host: %s\r\n\r\n",
+					   (gfud->full ? "" : "/"),
+					   (gfud->full ? gfud->url : gfud->website.page),
+					   (gfud->http11 ? "1.1" : "1.0"),
+					   gfud->website.address);
 		}
 
 		gaim_debug_misc("gaim_url_fetch", "Request: %s\n", buf);
@@ -2441,6 +2732,8 @@ url_fetched_cb(gpointer url_data, gint sock, GaimInputCondition cond)
 					size_t content_len;
 					gfud->startsaving = TRUE;
 
+					gaim_debug_misc("gaim_url_fetch", "Response headers: '%*.*s'\n", gfud->len, gfud->len, gfud->webdata);
+
 					/* See if we can find a redirect. */
 					if (parse_redirect(gfud->webdata, gfud->len, sock, gfud))
 						return;
@@ -2453,6 +2746,10 @@ url_fetched_cb(gpointer url_data, gint sock, GaimInputCondition cond)
 						/* We'll stick with an initial 8192 */
 						content_len = 8192;
 					}
+					else
+					{
+						gfud->has_explicit_data_len = TRUE;
+					}
 
 					/* Out with the old... */
 					gfud->len = 0;
@@ -2461,7 +2758,14 @@ url_fetched_cb(gpointer url_data, gint sock, GaimInputCondition cond)
 
 					/* In with the new. */
 					gfud->data_len = content_len;
-					gfud->webdata = g_malloc(gfud->data_len);
+					gfud->webdata = g_try_malloc(gfud->data_len);
+					if (gfud->webdata == NULL) {
+						gaim_debug_error("gaim_url_fetch", "Failed to allocate %u bytes: %s\n", gfud->data_len, strerror(errno));
+						gaim_input_remove(gfud->inpa);
+						close(sock);
+						gfud->callback(gfud->user_data, NULL, 0);
+						destroy_fetch_url_data(gfud);
+					}
 				}
 				else
 					gfud->newline = TRUE;
@@ -2471,19 +2775,14 @@ url_fetched_cb(gpointer url_data, gint sock, GaimInputCondition cond)
 
 			gfud->newline = FALSE;
 		}
+		else if (gfud->has_explicit_data_len && gfud->len == gfud->data_len)
+		{
+			got_eof = TRUE;
+		}
 	}
 	else if (errno != ETIMEDOUT)
 	{
-		gfud->webdata = g_realloc(gfud->webdata, gfud->len + 1);
-		gfud->webdata[gfud->len] = 0;
-
-		gaim_debug_misc("gaim_url_fetch", "Received: '%s'\n", gfud->webdata);
-
-		gaim_input_remove(gfud->inpa);
-		close(sock);
-		gfud->callback(gfud->user_data, gfud->webdata, gfud->len);
-
-		destroy_fetch_url_data(gfud);
+		got_eof = TRUE;
 	}
 	else
 	{
@@ -2494,6 +2793,19 @@ url_fetched_cb(gpointer url_data, gint sock, GaimInputCondition cond)
 
 		destroy_fetch_url_data(gfud);
 	}
+
+	if (got_eof) {
+		gfud->webdata = g_realloc(gfud->webdata, gfud->len + 1);
+		gfud->webdata[gfud->len] = 0;
+
+		/* gaim_debug_misc("gaim_url_fetch", "Received: '%s'\n", gfud->webdata); */
+
+		gaim_input_remove(gfud->inpa);
+		close(sock);
+		gfud->callback(gfud->user_data, gfud->webdata, gfud->len);
+
+		destroy_fetch_url_data(gfud);
+	}
 }
 
 void
@@ -2508,6 +2820,10 @@ gaim_url_fetch(const char *url, gboolean full,
 	g_return_if_fail(url != NULL);
 	g_return_if_fail(cb  != NULL);
 
+	gaim_debug_info("gaim_url_fetch",
+			 "requested to fetch (%s), full=%d, user_agent=(%s), http11=%d\n",
+			 url, full, user_agent?user_agent:"(null)", http11);
+
 	gfud = g_new0(GaimFetchUrlData, 1);
 
 	gfud->callback   = cb;
@@ -2518,7 +2834,7 @@ gaim_url_fetch(const char *url, gboolean full,
 	gfud->full       = full;
 
 	gaim_url_parse(url, &gfud->website.address, &gfud->website.port,
-				   &gfud->website.page);
+				   &gfud->website.page, &gfud->website.user, &gfud->website.passwd);
 
 	if ((sock = gaim_proxy_connect(NULL, gfud->website.address,
 								   gfud->website.port, url_fetched_cb,
@@ -2536,11 +2852,18 @@ gaim_url_decode(const char *str)
 	static char buf[BUF_LEN];
 	guint i, j = 0;
 	char *bum;
+	char hex[3];
 
 	g_return_val_if_fail(str != NULL, NULL);
 
+	/*
+	 * XXX - This check could be removed and buf could be made
+	 * dynamically allocated, but this is easier.
+	 */
+	if (strlen(str) >= BUF_LEN)
+		return NULL;
+
 	for (i = 0; i < strlen(str); i++) {
-		char hex[3];
 
 		if (str[i] != '%')
 			buf[j++] = str[i];
@@ -2575,10 +2898,12 @@ gaim_url_encode(const char *str)
 
 	g_return_val_if_fail(str != NULL, NULL);
 
-	for (i = 0; i < strlen(str); i++) {
+	for (i = 0; i < strlen(str) && j < (BUF_LEN - 1); i++) {
 		if (isalnum(str[i]))
 			buf[j++] = str[i];
 		else {
+			if (j > (BUF_LEN - 4))
+				break;
 			sprintf(buf + j, "%%%02x", (unsigned char)str[i]);
 			j += 3;
 		}
@@ -2641,15 +2966,87 @@ gaim_email_is_valid(const char *address)
 	return ((c - domain) > 3 ? TRUE : FALSE);
 }
 
+/** Stolen from gnome_uri_list_extract_uris **/
+GList* gaim_uri_list_extract_uris (const gchar* uri_list) {
+	const gchar *p, *q;
+	gchar *retval;
+	GList *result = NULL;
+
+	g_return_val_if_fail (uri_list != NULL, NULL);
+
+	p = uri_list;
+
+	/* We don't actually try to validate the URI according to RFC
+	* 2396, or even check for allowed characters - we just ignore
+	* comments and trim whitespace off the ends.  We also
+	* allow LF delimination as well as the specified CRLF.
+	*/
+	while (p) {
+		if (*p != '#') {
+			while (isspace(*p))
+				p++;
+
+			q = p;
+			while (*q && (*q != '\n') && (*q != '\r'))
+				q++;
+
+			if (q > p) {
+				q--;
+				while (q > p && isspace(*q))
+					q--;
+
+				retval = (gchar*)g_malloc (q - p + 2);
+				strncpy (retval, p, q - p + 1);
+				retval[q - p + 1] = '\0';
+
+				result = g_list_prepend (result, retval);
+			}
+		}
+		p = strchr (p, '\n');
+		if (p)
+			p++;
+	}
+
+	return g_list_reverse (result);
+}
+
+
+/** Stolen from gaim_uri_list_extract_filenames **/
+GList* gaim_uri_list_extract_filenames (const gchar* uri_list) {
+	GList *tmp_list, *node, *result;
+
+	g_return_val_if_fail (uri_list != NULL, NULL);
+
+	result = gaim_uri_list_extract_uris (uri_list);
+
+	tmp_list = result;
+	while (tmp_list) {
+		gchar *s = (gchar*)tmp_list->data;
+
+		node = tmp_list;
+		tmp_list = tmp_list->next;
+
+		if (!strncmp (s, "file:", 5)) {
+			node->data = g_filename_from_uri (s, NULL, NULL);
+			/* not sure if this fallback is useful at all */
+			if (!node->data) node->data = g_strdup (s+5);
+		} else {
+			result = g_list_remove_link(result, node);
+			g_list_free_1 (node);
+		}
+		g_free (s);
+	}
+	return result;
+}
 
 /**************************************************************************
  * UTF8 String Functions
  **************************************************************************/
-char *
+gchar *
 gaim_utf8_try_convert(const char *str)
 {
 	gsize converted;
-	char *utf8;
+	gchar *utf8;
 
 	g_return_val_if_fail(str != NULL, NULL);
 
@@ -2658,19 +3055,75 @@ gaim_utf8_try_convert(const char *str)
 	}
 
 	utf8 = g_locale_to_utf8(str, -1, &converted, NULL, NULL);
-	if (utf8)
-		return(utf8);
-
-	g_free(utf8);
+	if (utf8 != NULL)
+		return utf8;
 
 	utf8 = g_convert(str, -1, "UTF-8", "ISO-8859-15", &converted, NULL, NULL);
-	if (utf8 && converted == strlen (str)) {
-		return(utf8);
-	} else if (utf8) {
-		g_free(utf8);
-	}
+	if ((utf8 != NULL) && (converted == strlen(str)))
+		return utf8;
 
-	return(NULL);
+	if (utf8 != NULL)
+		g_free(utf8);
+
+	return NULL;
+}
+
+#define utf8_first(x) ((x & 0x80) == 0 || (x & 0xe0) == 0xc0 \
+		       || (x & 0xf0) == 0xe0 || (x & 0xf8) == 0xf)
+gchar *
+gaim_utf8_salvage(const char *str)
+{
+	GString *workstr;
+	const char *end;
+
+	g_return_val_if_fail(str != NULL, NULL);
+
+	workstr = g_string_sized_new(strlen(str));
+
+	do {
+		g_utf8_validate(str, -1, &end);
+		workstr = g_string_append_len(workstr, str, end - str);
+		str = end;
+		if (*str == '\0')
+			break;
+		do {
+			workstr = g_string_append_c(workstr, '?');
+			str++;
+		} while (!utf8_first(*str));
+	} while (*str != '\0');
+
+	return g_string_free(workstr, FALSE);
+}
+
+char *
+gaim_utf8_ncr_decode(const char *in)
+{
+	GString *out = g_string_new("");
+	int i;
+
+	g_return_val_if_fail(in != NULL, NULL);
+	g_return_val_if_fail(g_utf8_validate(in, -1, NULL), NULL);
+
+	for (i = 0; in[i]; i += 1) {
+		gboolean ncr_found_p = FALSE;
+		if (in[i] == '&' && in[i + 1] == '#' && isdigit(in[i + 2])) {
+			gunichar wc;
+			int j;
+			for (wc = 0, j = i + 2; isdigit(in[j]); j += 1) {
+				wc *= 10;
+				wc += in[j] - '0';
+			}
+			if (in[j] == ';') { /* Technically not completely correct */
+				g_string_append_unichar(out, wc);
+				i = j;
+				ncr_found_p = TRUE;
+			}
+		}
+		if (!ncr_found_p) {
+			g_string_append_c(out, in[i]);
+		}
+	}
+	return g_string_free(out, FALSE);
 }
 
 int
@@ -2794,3 +3247,37 @@ char *gaim_text_strip_mnemonic(const char *in)
 	return out;
 }
 
+const char* gaim_unescape_filename(const char *escaped) {
+	return gaim_url_decode(escaped);
+}
+
+
+/* this is almost identical to gaim_url_encode (hence gaim_url_decode
+ * being used above), but we want to keep certain characters unescaped
+ * for compat reasons */
+const char *
+gaim_escape_filename(const char *str)
+{
+	static char buf[BUF_LEN];
+	guint i, j = 0;
+
+	g_return_val_if_fail(str != NULL, NULL);
+
+	for (i = 0; i < strlen(str) && j < (BUF_LEN - 1); i++) {
+		if (isalnum(str[i]) || str[i] == '@' || str[i] == '-' ||
+				str[i] == '_' || str[i] == '.' || str[i] == '#')
+			buf[j++] = str[i];
+		else {
+			if (j > (BUF_LEN - 4))
+				break;
+			sprintf(buf + j, "%%%02x", (unsigned char)str[i]);
+			j += 3;
+		}
+	}
+
+	buf[j] = '\0';
+
+	return buf;
+}
+
+
